/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Data Management - Simple channel management API
 *
 *  Authors:
 *      Gabor Gombas <Gabor.Gombas@cern.ch>
 *      Ricardo Rocha <Ricardo.Rocha@cern.ch>
 *      Zoltan Farkas <Zoltan.Farkas@cern.ch>
 *      Paolo Tedesco <paolo.tedesco@cern.ch>
 */

#include <glite/data/transfer/c/channel-simple.h>

#include <glite/data/transfer/c/channel.nsmap>

#include "channel_internal.h"

#include <cgsi_plugin.h>

#include <glite/data/glite-util.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

static const char * size_strings[] = {
    "kb","mb","gb"
};
static const int size_strings_len = sizeof(size_strings) / sizeof(char*);

/**
 * validate a string containing a size
 * @param value the string to validate
 * @return 1 if successful, 0 otherwise
 */
int validate_size_string(const char * value){
    double v = 0;
    size_t value_len = strlen(value);
    char * s = (char *)calloc(value_len + 1,sizeof(char));
    if (0 == s) return 0;
    int ret = 0;
    if (sscanf(value,"%lf %s",&v,s) || sscanf(value,"%lf%s",&v,s)){
        if (0 == strlen(s)){
            ret = 1;
        } else {
            int i = 0;
            for (i = 0;i < size_strings_len;++i){
                if (0 == strcasecmp(s,size_strings[i])){
                    ret = 1;
                    break;
                }
            }
        }
    }
    free(s);
    return ret;
}

/**********************************************************************
 * Data type definitions
 */

struct channel_stateconv {
    const char          *name;
    glite_channel_state state;
};


/**********************************************************************
 * Prototypes
 */

static void fault_to_error(glite_transfer_ctx *ctx, const char *method);


/**********************************************************************
 * Global variables
 */

static const struct channel_stateconv channel_states[] = {
    { "Active",     GLITE_CHANNEL_ACTIVE    },
    { "Inactive",   GLITE_CHANNEL_INACTIVE  },
    { "Drain",      GLITE_CHANNEL_DRAIN     },
    { "Stopped",    GLITE_CHANNEL_STOPPED   },
    { "Halted",     GLITE_CHANNEL_HALTED    },
    { "Archived",   GLITE_CHANNEL_ARCHIVED  },
    { "Deleted",    GLITE_CHANNEL_DELETED   }
};

/**********************************************************************
 * Local helper functions
 */

/* Check if this is a http:// URL */
static int is_http(const char *url) {
    return url && !strncmp(url, HTTP_PREFIX, strlen(HTTP_PREFIX));
}

/* Check if this is a https:// URL */
static int is_https(const char *url) {
    return url && !strncmp(url, HTTPS_PREFIX, strlen(HTTPS_PREFIX));
}

/* Check if this is a httpg:// URL */
static int is_httpg(const char *url) {
    return url && !strncmp(url, HTTPG_PREFIX, strlen(HTTPG_PREFIX));
}

/* Check if the context strcuture is usable */
static int is_ctx_ok(glite_transfer_ctx *ctx) {
    int ret;

    /* A NULL context is not usable */
    if (!ctx)
        return FALSE;

    /* If the context is already initialized for the placement port,
     * do nothing */
    if (ctx->port_type == PORT_CHANNEL)
        return TRUE;
    /* If the context is already initialized for some other port,
     * bail out */
    if (ctx->port_type != PORT_NONE)
        return FALSE;

    if (is_http(ctx->default_endpoint) ||
        is_https(ctx->default_endpoint) ||
        is_httpg(ctx->default_endpoint))
    {
        ctx->endpoint = g_strdup(ctx->default_endpoint);
    }
    else if (ctx->default_endpoint != NULL)
    {
        glite_transfer_set_error(ctx, GLITE_TRANSFER_ERROR_INVALIDARGUMENT,
                                 "Wrongly formatted endpoint \"%s\"", ctx->default_endpoint);
        return FALSE;
    }
    else
    {
        char *error;
        const char * fts_srvtype;
        const char * fts_version;

        if (getenv(GLITE_CHANNEL_SD_ENV))
            fts_srvtype = getenv(GLITE_CHANNEL_SD_ENV);
        else
            fts_srvtype = GLITE_CHANNEL_SD_TYPE;

        if(getenv(GLITE_FTS_IFC_VERSION))
            fts_version = getenv(GLITE_FTS_IFC_VERSION);
        else
            fts_version = INTERFACE_VERSION;

        ctx->endpoint = glite_discover_endpoint_by_version(fts_srvtype,
                        ctx->default_endpoint, fts_version, &error);
        if (!ctx->endpoint) {
            glite_transfer_set_error(ctx,
                                     GLITE_TRANSFER_ERROR_SERVICEDISCOVERY,
                                     "Service discovery: %s", error);
            free(error);
            return FALSE;
        }
    }

    /* SOAP initialization */
    if (is_https(ctx->endpoint))
        ret = soap_cgsi_init(ctx->soap, CGSI_OPT_DISABLE_NAME_CHECK |
                             CGSI_OPT_SSL_COMPATIBLE);
    else if (is_httpg(ctx->endpoint))
        ret = soap_cgsi_init(ctx->soap, CGSI_OPT_DISABLE_NAME_CHECK);
    else
        ret = 0;
    if (ret) {
        glite_transfer_set_error(ctx, GLITE_TRANSFER_ERROR_SOAP,
                                 "Failed to initialize the GSI plugin");
        return FALSE;
    }

    if (soap_set_namespaces(ctx->soap, channel_namespaces)) {
        fault_to_error(ctx, "Setting SOAP namespaces");
        return FALSE;
    }

    ctx->port_type = PORT_CHANNEL;
    return TRUE;
}

/* Decode an exception inside a SOAP fault */
static void decode_exception(glite_transfer_ctx *ctx,
                             struct SOAP_ENV__Detail *detail, const char *method) {
    char *message;

    if (!detail)
        return;

    #if _GSOAP_VERSION >= 0x020700
    #define SET_EXCEPTION(exc, err) \
    message = ((struct transfer__ ## exc *)detail->fault)->message; \
    if (!message) \
        message = #exc " received from the service"; \
    glite_transfer_set_error(ctx, GLITE_TRANSFER_EXCEPTION_ ## err, \
                             "%s: %s", method, message);
    #define SET_EXCEPTION2(exc, err) \
    message = (NULL != ((struct channel__ ## exc *)detail->fault)->fault)?((struct channel__ ## exc *)detail->fault)->fault->message:NULL; \
    if (!message) \
        message = #exc " received from the service"; \
    glite_transfer_set_error(ctx, GLITE_TRANSFER_EXCEPTION_ ## err, \
                             "%s: %s", method, message);
    #else
    #define SET_EXCEPTION(exc, err) \
    message = ((struct transfer__ ## exc *)detail->value)->message; \
    if (!message) \
        message = #exc " received from the service"; \
    glite_transfer_set_error(ctx, GLITE_TRANSFER_EXCEPTION_ ## err, \
                             "%s: %s", method, message);
    #define SET_EXCEPTION2(exc, err) \
    message = (NULL != ((struct channel__ ## exc *)detail->value)->fault)?((struct channel__ ## exc *)detail->value)->fault->message:NULL; \
    if (!message) \
        message = #exc " received from the service"; \
    glite_transfer_set_error(ctx, GLITE_TRANSFER_EXCEPTION_ ## err, \
                             "%s: %s", method, message);

    #endif

    switch (detail->__type) {
    case SOAP_TYPE_transfer__AuthorizationException:
        SET_EXCEPTION(AuthorizationException, AUTHORIZATION);
        break;
    case SOAP_TYPE_transfer__InternalException:
        SET_EXCEPTION(InternalException, INTERNAL);
        break;
    case SOAP_TYPE_transfer__InvalidArgumentException:
        SET_EXCEPTION(InvalidArgumentException, INVALIDARGUMENT);
        break;
    case SOAP_TYPE_transfer__NotExistsException:
        SET_EXCEPTION(NotExistsException, NOTEXISTS);
        break;
    case SOAP_TYPE_transfer__ExistsException:
        SET_EXCEPTION(ExistsException, EXISTS);
        break;
    case SOAP_TYPE_transfer__TransferException:
        SET_EXCEPTION(TransferException, TRANSFER);
        break;
        #if _GSOAP_VERSION < 0x020709

    case SOAP_TYPE_channel__AuthorizationException:
        SET_EXCEPTION2(AuthorizationException, AUTHORIZATION);
        break;
    case SOAP_TYPE_channel__InternalException:
        SET_EXCEPTION2(InternalException, INTERNAL);
        break;
    case SOAP_TYPE_channel__InvalidArgumentException:
        SET_EXCEPTION2(InvalidArgumentException, INVALIDARGUMENT);
        break;
    case SOAP_TYPE_channel__NotExistsException:
        SET_EXCEPTION2(NotExistsException, NOTEXISTS);
        break;
    case SOAP_TYPE_channel__ExistsException:
        SET_EXCEPTION2(ExistsException, EXISTS);
        break;

        #endif

    default:
        /* Let the generic error decoding handle this */
        break;
    }
    #undef SET_EXCEPTION
    #undef SET_EXCEPTION2
}

/* Convert the SOAP fault to an error message */
static void fault_to_error(glite_transfer_ctx *ctx, const char *method) {
    const char **code, **string, **detail;

    /* Reset the error class */
    ctx->errclass = GLITE_TRANSFER_ERROR_NONE;

    /* Convert the error message to a fault */
    soap_set_fault(ctx->soap);

    /* Try to decode the exception that caused the fault */
    if (ctx->soap->fault) {
        /* Look for a SOAP 1.1 fault */
        if (ctx->soap->fault->detail)
            decode_exception(ctx, ctx->soap->fault->detail, method);
        /* Look for a SOAP 1.2 fault */
        if (ctx->soap->fault->SOAP_ENV__Detail)
            decode_exception(ctx,
                             ctx->soap->fault->SOAP_ENV__Detail, method);
    }

    /* If we did not manage to decode the exception, try generic error
     * decoding */
    if (ctx->errclass == GLITE_TRANSFER_ERROR_NONE) {
        code = soap_faultcode(ctx->soap);
        string = soap_faultstring(ctx->soap);
        detail = soap_faultdetail(ctx->soap);

        /* If the SOAP 1.1 detail is empty, try the SOAP 1.2 detail */
        if (!detail && ctx->soap->fault &&
                ctx->soap->fault->SOAP_ENV__Detail)
            detail = (const char **)&ctx->soap->fault->SOAP_ENV__Detail->__any;

        /* Provide default messages */
        if (!code || !*code) {
            code = alloca(sizeof(*code));
            *code = "(SOAP fault code missing)";
        }
        if (!string || !*string) {
            string = alloca(sizeof(*string));
            *string = "(SOAP fault string missing)";
        }

        if (detail && *detail)
            glite_transfer_set_error(ctx, GLITE_TRANSFER_ERROR_SOAP,
                                     "%s: SOAP fault: %s - %s (%s)", method, *code,
                                     *string, *detail);
        else
            glite_transfer_set_error(ctx, GLITE_TRANSFER_ERROR_SOAP,
                                     "%s: SOAP fault: %s - %s", method, *code,
                                     *string);
    }

    soap_end(ctx->soap);
}

/**********************************************************************
 * Library interface functions
 */

const char *glite_channel_get_endpoint(glite_transfer_ctx *ctx) {
    if (!is_ctx_ok(ctx))
        return NULL;
    return ctx->endpoint;
}

glite_channel_state glite_channel_state_parse(glite_transfer_ctx *ctx,
        const char *state) {
    unsigned i;

    if (!state) {
        err_invarg(ctx, "Channel state is missing");
        return GLITE_CHANNEL_UNKNOWN;
    }

    for (i = 0; i < G_N_ELEMENTS(channel_states); i++)
        if (!strcasecmp(channel_states[i].name, state))
            return channel_states[i].state;

    err_invarg(ctx, "Unknown channel state %s", state);
    return GLITE_CHANNEL_UNKNOWN;
}

const char *glite_channel_state_str(glite_channel_state state) {
    unsigned i;

    for (i = 0; i < G_N_ELEMENTS(channel_states); i++)
        if (channel_states[i].state == state)
            return channel_states[i].name;

    return "Unknown";
}

struct transfer_log_level_conv{
    const char          *name;
    transfer_log_level  level;
};
static const struct transfer_log_level_conv transfer_log_levels[] = {
    { "DEBUG",  TRANSFER_LOG_LEVEL_DEBUG    },
    { "INFO",   TRANSFER_LOG_LEVEL_INFO     },
    { "WARN",   TRANSFER_LOG_LEVEL_WARN     },
    { "ERROR",  TRANSFER_LOG_LEVEL_ERROR    }
};

/**
 * convert string -> transfer_log_level
 */
transfer_log_level transfer_log_level_parse(const char * value){
    unsigned i = 0;
    for (i = 0;i < G_N_ELEMENTS(transfer_log_levels);++i){
        if (0 == strcasecmp(transfer_log_levels[i].name,value)){
            return transfer_log_levels[i].level;
        }
    }
    return TRANSFER_LOG_LEVEL_INVALID;
}

/**
 * convert transfer_log_level -> string
 */
const char * transfer_log_level_str(transfer_log_level value){
    unsigned i = 0;
    for (i = 0;i < G_N_ELEMENTS(transfer_log_levels);++i){
        if (transfer_log_levels[i].level == value){
            return transfer_log_levels[i].name;
        }
    }
    return NULL;
}

struct srmcopy_direction_conv{
    const char          *name;
    srmcopy_direction   direction;
};
static const struct srmcopy_direction_conv srmcopy_directions[] = {
    { "PUSH",   SRMCOPY_DIRECTION_PUSH      },
    { "PULL",   SRMCOPY_DIRECTION_PULL      }
};


/**
 * convert string -> srmcopy_direction
 */
srmcopy_direction srmcopy_direction_parse(const char * value){
    unsigned i = 0;
    if (value == NULL){
      return SRMCOPY_DIRECTION_INVALID;
    }

    for (i = 0;i < G_N_ELEMENTS(srmcopy_directions);++i){
        if (0 == strcasecmp(srmcopy_directions[i].name,value)){
            return srmcopy_directions[i].direction;
        }
    }
    return TRANSFER_LOG_LEVEL_INVALID;
}

/**
 * convert srmcopy_direction -> string
 */
const char * srmcopy_direction_str(srmcopy_direction value){
    unsigned i = 0;
    for (i = 0;i < G_N_ELEMENTS(srmcopy_directions);++i){
        if (srmcopy_directions[i].direction == value){
            return srmcopy_directions[i].name;
        }
    }
    return NULL;
}

/**********************************************************************
 * Service requests
 *
 * Interface: org.glite.data.transfer.ServiceBase
 */

char *glite_channel_getVersion(glite_transfer_ctx *ctx) {
    struct channel__getVersionResponse resp;
    char *version;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;

    /* Make the SOAP call */
    ret = soap_call_channel__getVersion(ctx->soap, ctx->endpoint, NULL,
                                        &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getVersion");
        return NULL;
    }

    if (resp.getVersionReturn)
        version = strdup(resp.getVersionReturn);
    else
        version = NULL;
    soap_end(ctx->soap);
    return version;
}

char *glite_channel_getSchemaVersion(glite_transfer_ctx *ctx) {
    struct channel__getSchemaVersionResponse resp;
    char *version;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;

    /* Make the SOAP call */
    ret = soap_call_channel__getSchemaVersion(ctx->soap, ctx->endpoint,
            NULL, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getSchemaVersion");
        return NULL;
    }

    if (resp.getSchemaVersionReturn)
        version = strdup(resp.getSchemaVersionReturn);
    else
        version = NULL;
    soap_end(ctx->soap);
    return version;
}

char *glite_channel_getInterfaceVersion(glite_transfer_ctx *ctx) {
    struct channel__getInterfaceVersionResponse resp;
    char *version;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;

    /* Make the SOAP call */
    ret = soap_call_channel__getInterfaceVersion(ctx->soap, ctx->endpoint,
            NULL, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getInterfaceVersion");
        return NULL;
    }

    if (resp.getInterfaceVersionReturn)
        version = strdup(resp.getInterfaceVersionReturn);
    else
        version = NULL;
    soap_end(ctx->soap);
    return version;
}

char *glite_channel_getServiceMetadata(glite_transfer_ctx *ctx, const char *key) {
    struct channel__getServiceMetadataResponse resp;
    char *req_key, *result;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;
    if (!key) {
        err_invarg(ctx, "getServiceMetadata: Key is missing");
        return NULL;
    }

    /* We cannot turn a const pointer to a non-const, so we have to copy. */
    req_key = soap_strdup(ctx->soap, key);
    if (!req_key) {
        err_outofmemory(ctx);
        return NULL;
    }

    /* Make the SOAP call */
    ret = soap_call_channel__getServiceMetadata(ctx->soap, ctx->endpoint,
            NULL, req_key, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getServiceMetadata");
        return NULL;
    }

    if (resp._getServiceMetadataReturn)
        result = strdup(resp._getServiceMetadataReturn);
    else
        result = NULL;
    soap_end(ctx->soap);
    return result;
}

/**********************************************************************
 * Interface: org.glite.data.transfer.channel.ChannelManagement
 */

int glite_channel_add(glite_transfer_ctx *ctx,
                      const glite_transfer_Channel *channel) {
    struct transfer__Channel *req;
    struct channel__addResponse response;
    int ret;

    if (!is_ctx_ok(ctx))
        return -1;
    if (!channel) {
        err_invarg(ctx, "add: Channel descrtiptor is missing");
        return -1;
    }

    req = to_soap_Channel(ctx->soap, channel);
    if (!req) {
        err_outofmemory(ctx);
        soap_end(ctx->soap);
        return -1;
    }

    ret = soap_call_channel__add(ctx->soap, ctx->endpoint, NULL, req, &response);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "add");
        return -1;
    }

    soap_end(ctx->soap);
    return 0;
}

int glite_channel_drop(glite_transfer_ctx *ctx, const char *name) {
    char *sname;
    int ret;
    struct channel__dropResponse response;

    if (!is_ctx_ok(ctx))
        return -1;
    if (!name) {
        err_invarg(ctx, "drop: Channel name is missing");
        return -1;
    }

    /* We cannot turn a const pointer to a non-const, so we have to copy. */
    sname = soap_strdup(ctx->soap, name);
    if (!sname) {
        err_outofmemory(ctx);
        return -1;
    }

    /* Make the SOAP call */
    ret = soap_call_channel__drop(ctx->soap, ctx->endpoint, NULL,
                                  sname, &response);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "drop");
        return -1;
    }

    soap_end(ctx->soap);
    return 0;
}

glite_transfer_Channel *glite_channel_getChannel(glite_transfer_ctx *ctx,
        const char *name) {
    struct channel__getChannelResponse resp;
    glite_transfer_Channel *result;
    char *sname;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;
    if (!name) {
        err_invarg(ctx, "getChannel: Channel name is missing");
        return NULL;
    }

    /* We cannot turn a const pointer to a non-const, so we have to copy. */
    sname = soap_strdup(ctx->soap, name);
    if (!sname) {
        err_outofmemory(ctx);
        return NULL;
    }

    /* Make the SOAP call */
    ret = soap_call_channel__getChannel(ctx->soap, ctx->endpoint, NULL,
                                        sname, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getChannel");
        return NULL;
    }

    if (!resp._getChannelReturn) {
        glite_transfer_set_error(ctx, GLITE_TRANSFER_ERROR_SOAP,
                                 "getChannel: Service returned empty response");
        soap_end(ctx->soap);
        return NULL;
    }

    result = from_soap_Channel(ctx, resp._getChannelReturn);
    soap_end(ctx->soap);
    return result;
}

glite_transfer_Channel2 *glite_channel_getChannel2(
    glite_transfer_ctx *ctx,
    const char *name) {

    struct channel__getChannel2Response resp;
    glite_transfer_Channel2 *result;
    char *sname;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;
    if (!name) {
        err_invarg(ctx, "getChannel2: Channel name is missing");
        return NULL;
    }

    /* We cannot turn a const pointer to a non-const, so we have to copy. */
    sname = soap_strdup(ctx->soap, name);
    if (!sname) {
        err_outofmemory(ctx);
        return NULL;
    }

    /* Make the SOAP call */
    ret = soap_call_channel__getChannel2(ctx->soap, ctx->endpoint, NULL,
                                         sname, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getChannel2");
        return NULL;
    }

    if (!resp._getChannel2Return) {
        glite_transfer_set_error(ctx, GLITE_TRANSFER_ERROR_SOAP,
                                 "getChannel2: Service returned empty response");
        soap_end(ctx->soap);
        return NULL;
    }

    result = from_soap_Channel2(ctx, resp._getChannel2Return);
    soap_end(ctx->soap);
    return result;
}

glite_transfer_Channel3 *glite_channel_getChannel3(
    glite_transfer_ctx  *ctx,
    const char          *name) {
    struct channel__getChannel3Response resp;
    glite_transfer_Channel3 *result;
    char *sname;
    int ret;
    if (!is_ctx_ok(ctx))
        return NULL;
    if (!name) {
        err_invarg(ctx, "getChannel3: Channel name is missing");
        return NULL;
    }
    /* We cannot turn a const pointer to a non-const, so we have to copy. */
    sname = soap_strdup(ctx->soap, name);
    if (!sname) {
        err_outofmemory(ctx);
        return NULL;
    }
    /* Make the SOAP call */
    ret = soap_call_channel__getChannel3(
              ctx->soap,
              ctx->endpoint,
              NULL,
              sname,
              &resp);

    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getChannel3");
        return NULL;
    }

    if (!resp._getChannel3Return) {
        glite_transfer_set_error(
            ctx,
            GLITE_TRANSFER_ERROR_SOAP,
            "getChannel3: Service returned empty response");
        soap_end(ctx->soap);
        return NULL;
    }

    result = from_soap_Channel3(
                 ctx,
                 resp._getChannel3Return);
    soap_end(ctx->soap);
    return result;
}

char **glite_channel_listChannels(glite_transfer_ctx *ctx, int *resultCnt) {
    struct channel__listChannelsResponse resp;
    char **result;
    int ret;

    /* Default: error */
    if (resultCnt)
        *resultCnt = -1;

    if (!is_ctx_ok(ctx))
        return NULL;

    /* Make the SOAP call */
    ret = soap_call_channel__listChannels(ctx->soap, ctx->endpoint, NULL,
                                          &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "listChannels");
        return NULL;
    }

    result = from_soap_StringArray(ctx, resp.listChannelsReturn, resultCnt);
    soap_end(ctx->soap);
    return result;
}

int glite_channel_changeStateForHeldJob(glite_transfer_ctx *ctx,
                                        const char *jobId, glite_transfer_state state) {
    char *sid, *sstate;
    int ret;

    if (!is_ctx_ok(ctx))
        return -1;
    if (!jobId) {
        err_invarg(ctx, "changeStateForHeldJob: Job ID is missing");
        return -1;
    }

    /* We cannot turn a const pointer to a non-const, so we have to copy. */
    sid = soap_strdup(ctx->soap, jobId);
    sstate = soap_strdup(ctx->soap, glite_transfer_state_str(state));
    if (!sid || !sstate) {
        err_outofmemory(ctx);
        soap_end(ctx->soap);
        return -1;
    }

    /* Make the SOAP call */
    ret = soap_call_channel__changeStateForHeldJob(ctx->soap, ctx->endpoint,
            NULL, sid, sstate, NULL);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "changeStateForHeldJob");
        return -1;
    }

    soap_end(ctx->soap);
    return 0;
}

int glite_channel_changeStateForHeldJobsOnChannel(glite_transfer_ctx *ctx,
        const char *name, glite_transfer_state state) {
    char *sname, *sstate;
    int ret;

    if (!is_ctx_ok(ctx))
        return -1;
    if (!name) {
        err_invarg(ctx, "changeStateForHeldJobsOnChannel: Channel name is missing");
        return -1;
    }

    /* We cannot turn a const pointer to a non-const, so we have to copy. */
    sname = soap_strdup(ctx->soap, name);
    sstate = soap_strdup(ctx->soap, glite_transfer_state_str(state));
    if (!sname || !sstate) {
        err_outofmemory(ctx);
        soap_end(ctx->soap);
        return -1;
    }

    /* Make the SOAP call */
    ret = soap_call_channel__changeStateForHeldJobsOnChannel(ctx->soap,
            ctx->endpoint, NULL, sname, sstate, NULL);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "changeStateForHeldJobsOnChannel");
        return -1;
    }

    soap_end(ctx->soap);
    return 0;
}

#define GLITE_CHANNEL_SET_NUM_VALUE(param_name,soap_call,value) \
    char *sname; \
    int ret; \
    if (!is_ctx_ok(ctx)) return -1; \
    if (!name) { err_invarg(ctx, param_name ": Channel name is missing"); return -1; } \
    sname = soap_strdup(ctx->soap, name); \
    if (!sname) { err_outofmemory(ctx); return -1; } \
    ret = soap_call (ctx->soap, ctx->endpoint,NULL, sname, value, NULL); \
    if (ret != SOAP_OK) { fault_to_error(ctx, param_name); return -1; } \
    soap_end(ctx->soap); \
    return 0;

#define GLITE_CHANNEL_SET_NUM_VALUE2(param_name,soap_call,value) \
    char *sname, *smessage; \
    int ret; \
    if (!is_ctx_ok(ctx)) return -1; \
    if (!name) { err_invarg(ctx, param_name ": Channel name is missing"); return -1; } \
    sname = soap_strdup(ctx->soap, name); \
    if (!sname) { err_outofmemory(ctx); return -1; } \
    smessage = soap_strdup(ctx->soap, message); \
    if (!smessage) { err_outofmemory(ctx); return -1; } \
    ret = soap_call (ctx->soap, ctx->endpoint,NULL, sname, value, smessage, NULL); \
    if (ret != SOAP_OK) { fault_to_error(ctx, param_name); return -1; } \
    soap_end(ctx->soap); \
    return 0;

#define GLITE_CHANNEL_SET_STRING_VALUE(param_name,soap_call,value) \
    char *sname, *svalue; \
    int ret; \
    if (!is_ctx_ok(ctx)) return -1; \
    if (!name) { err_invarg(ctx, param_name ": Channel name is missing"); return -1; } \
    sname = soap_strdup(ctx->soap, name); \
    svalue = soap_strdup(ctx->soap, value); \
    if (!sname || (value && !svalue)) { err_outofmemory(ctx); return -1; } \
    ret = soap_call (ctx->soap, ctx->endpoint, NULL, sname, svalue, NULL); \
    if (ret != SOAP_OK) { fault_to_error(ctx, param_name); return -1; } \
    soap_end(ctx->soap); \
    return 0;

#define GLITE_CHANNEL_SET_STRING_VALUE2(param_name,soap_call,value) \
    char *sname, *svalue, *smessage; \
    int ret; \
    if (!is_ctx_ok(ctx)) return -1; \
    if (!name) { err_invarg(ctx, param_name ": Channel name is missing"); return -1; } \
    sname = soap_strdup(ctx->soap, name); \
    svalue = soap_strdup(ctx->soap, value); \
    if (!sname || (value && !svalue)) { err_outofmemory(ctx); return -1; } \
    smessage = soap_strdup(ctx->soap, message); \
    if (!smessage) { err_outofmemory(ctx); return -1; } \
    ret = soap_call (ctx->soap, ctx->endpoint, NULL, sname, svalue, smessage, NULL); \
    if (ret != SOAP_OK) { fault_to_error(ctx, param_name); return -1; } \
    soap_end(ctx->soap); \
    return 0;


// glite_channel_setBandwidth
//------------------------------------------------------------------------------
int glite_channel_setBandwidth(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 utilisation) {
    GLITE_CHANNEL_SET_NUM_VALUE("Bandwidth",soap_call_channel__setBandwidth,utilisation);
}

int glite_channel_setBandwidth2(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 utilisation,
    const char          *message) {
    GLITE_CHANNEL_SET_NUM_VALUE2("Bandwidth",soap_call_channel__setBandwidth2,utilisation);
}

// glite_channel_setNominalThroughput
//------------------------------------------------------------------------------
int glite_channel_setNominalThroughput(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 throughput) {
    GLITE_CHANNEL_SET_NUM_VALUE("NominalThroughput",soap_call_channel__setNominalThroughput,throughput);
}

int glite_channel_setNominalThroughput2(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 throughput,
    const char          *message) {
    GLITE_CHANNEL_SET_NUM_VALUE2("NominalThroughput",soap_call_channel__setNominalThroughput2,throughput);
}

// glite_channel_setNumberOfFiles
//------------------------------------------------------------------------------
int glite_channel_setNumberOfFiles(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 files) {
    GLITE_CHANNEL_SET_NUM_VALUE("NumberOfFiles",soap_call_channel__setNumberOfFiles,files);
}

int glite_channel_setNumberOfFiles2(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 files,
    const char          *message) {
    GLITE_CHANNEL_SET_NUM_VALUE2("NumberOfFiles",soap_call_channel__setNumberOfFiles2,files);
}

// glite_channel_setNumberOfStreams
//------------------------------------------------------------------------------
int glite_channel_setNumberOfStreams(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 streams) {
    GLITE_CHANNEL_SET_NUM_VALUE("NumberOfStreams",soap_call_channel__setNumberOfStreams,streams);
}

int glite_channel_setNumberOfStreams2(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 streams,
    const char          *message) {
    GLITE_CHANNEL_SET_NUM_VALUE2("NumberOfStreams",soap_call_channel__setNumberOfStreams2,streams);
}

// glite_channel_setContact
//------------------------------------------------------------------------------
int glite_channel_setContact(
    glite_transfer_ctx  *ctx,
    const char          *name,
    const char          *contact) {
    GLITE_CHANNEL_SET_STRING_VALUE("Contact",soap_call_channel__setContact,contact);
}

int glite_channel_setContact2(
    glite_transfer_ctx  *ctx,
    const char          *name,
    const char          *contact,
    const char          *message) {
    GLITE_CHANNEL_SET_STRING_VALUE2("Contact",soap_call_channel__setContact2,contact);
}

// glite_channel_setTCPBufferSize
//------------------------------------------------------------------------------
int glite_channel_setTCPBufferSize2(
    glite_transfer_ctx  *ctx,
    const char          *name,
    const char          *buffer_size,
    const char          *message) {
    GLITE_CHANNEL_SET_STRING_VALUE2("TCPBufferSize",soap_call_channel__setTCPBufferSize2,buffer_size);
}

// glite_channel_setState
//------------------------------------------------------------------------------
int glite_channel_setState(
    glite_transfer_ctx  *ctx,
    const char          *name,
    glite_channel_state state) {
    const char * state_str = glite_channel_state_str(state);
    GLITE_CHANNEL_SET_STRING_VALUE("State",soap_call_channel__setState,state_str);
}

int glite_channel_setState2(
    glite_transfer_ctx  *ctx,
    const char          *name,
    glite_channel_state state,
    const char          *message) {
    const char * state_str = glite_channel_state_str(state);
    GLITE_CHANNEL_SET_STRING_VALUE2("State",soap_call_channel__setState2,state_str);
}

// glite_channel_setTargetDirCheck
//------------------------------------------------------------------------------
int glite_channel_setTargetDirCheck(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 target_dir_check,
    const char*         message) {
    GLITE_CHANNEL_SET_NUM_VALUE2("TargetDirCheck",soap_call_channel__setTargetDirCheck,target_dir_check);
}

// glite_channel_setUrlCopyFirstMarkTo
//------------------------------------------------------------------------------
int glite_channel_setUrlCopyFirstMarkTo(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 url_copy_first_txmark_to,
    const char          *message) {
    GLITE_CHANNEL_SET_NUM_VALUE2("UrlCopyFirstMarkTo",soap_call_channel__setUrlCopyFirstTxmarkTo,url_copy_first_txmark_to);
}

/**
 * setBlockSize
 */
int glite_channel_setBlockSize(
    glite_transfer_ctx  *ctx,
    const char          *name,
    const char          *value,
    const char          *message){
    GLITE_CHANNEL_SET_STRING_VALUE2("BlockSize",soap_call_channel__setBlockSize,value);
}

/**
 * setHttpTimeout
 */
int glite_channel_setHttpTimeout(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 value,
    const char          *message){
    GLITE_CHANNEL_SET_NUM_VALUE2("HttpTimeout",soap_call_channel__setHttpTimeout,value);
}

/**
 * setTransferLogLevel
 */
int glite_channel_setTransferLogLevel(
    glite_transfer_ctx  *ctx,
    const char          *name,
    transfer_log_level  value,
    const char          *message){
    const char * value_str = transfer_log_level_str(value);
    if (NULL == value_str){
        err_invarg(ctx, "TransferLogLevel: invalid value");
        return -1;
    }
    GLITE_CHANNEL_SET_STRING_VALUE2("TransferLogLevel",soap_call_channel__setTransferLogLevel,value_str);
}

/**
 * setPreparingFilesRatio
 */
int glite_channel_setPreparingFilesRatio(
    glite_transfer_ctx  *ctx,
    const char          *name,
    double              value,
    const char          *message){
    if (value < 1){
        err_invarg(ctx, "PreparingFilesRatio: invalid value");
        return -1;
    }
    GLITE_CHANNEL_SET_NUM_VALUE2("PreparingFilesRatio",soap_call_channel__setPreparingFilesRatio,value);
}

#if 0
/**
 * setTransferType
 */
int glite_channel_setTransferType(
    glite_transfer_ctx  *ctx,
    const char          *name,
    const char          *value,
    const char          *message){
    GLITE_CHANNEL_SET_STRING_VALUE2("TransferType",soap_call_channel__setTransferType,value);
}
#endif

/**
 * setUrlCopyPutTimeout
 */
int glite_channel_setUrlCopyPutTimeout(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 value,
    const char          *message){
    GLITE_CHANNEL_SET_NUM_VALUE2("UrlCopyPutTimeout",soap_call_channel__setUrlCopyPutTimeout,value);
}

/**
 * setUrlCopyPutDoneTimeout
 */
int glite_channel_setUrlCopyPutDoneTimeout(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 value,
    const char          * message){
    GLITE_CHANNEL_SET_NUM_VALUE2("UrlCopyPutDoneTimeout",soap_call_channel__setUrlCopyPutDoneTimeout,value);
}

/**
 * setUrlCopyGetTimeout
 */
int glite_channel_setUrlCopyGetTimeout(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 value,
    const char          * message){
    GLITE_CHANNEL_SET_NUM_VALUE2("UrlCopyGetTimeout",soap_call_channel__setUrlCopyGetTimeout,value);
}

/**
 * setUrlCopyGetDoneTimeout
 */
int glite_channel_setUrlCopyGetDoneTimeout(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 value,
    const char          * message){
    GLITE_CHANNEL_SET_NUM_VALUE2("UrlCopyGetDoneTimeout",soap_call_channel__setUrlCopyGetDoneTimeout,value);
}

/**
 * setUrlCopyTransferTimeout
 */
int glite_channel_setUrlCopyTransferTimeout(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 value,
    const char          * message){
    GLITE_CHANNEL_SET_NUM_VALUE2("UrlCopyTransferTimeout",soap_call_channel__setUrlCopyTransferTimeout,value);
}

/**
 * setUrlCopyTransferMarkersTimeout
 */
int glite_channel_setUrlCopyTransferMarkersTimeout(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 value,
    const char          * message){
    GLITE_CHANNEL_SET_NUM_VALUE2("UrlCopyTransferMarkersTimeout",soap_call_channel__setUrlCopyTransferMarkersTimeout,value);
}

/**
 * setUrlCopyNoProgressTimeout
 */
int glite_channel_setUrlCopyNoProgressTimeout(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 value,
    const char          * message){
    GLITE_CHANNEL_SET_NUM_VALUE2("UrlCopyNoProgressTimeout",soap_call_channel__setUrlCopyNoProgressTimeout,value);
}

/**
 * setUrlCopyTransferTimeoutPerMB
 */
int glite_channel_setUrlCopyTransferTimeoutPerMB(
    glite_transfer_ctx  *ctx,
    const char          *name,
    double              value,
    const char          * message){
    GLITE_CHANNEL_SET_NUM_VALUE2("UrlCopyTransferTimeoutPerMB",soap_call_channel__setUrlCopyTransferTimeoutPerMB,value);
}

/**
 * setSrmCopyDirection
 */
int glite_channel_setSrmCopyDirection(
    glite_transfer_ctx  *ctx,
    const char          *name,
    srmcopy_direction   value,
    const char          * message){
    const char * value_str = srmcopy_direction_str(value);
    if (NULL == value_str){
        err_invarg(ctx, "SrmCopyDirection: invalid value");
        return -1;
    }
    GLITE_CHANNEL_SET_STRING_VALUE2("SrmCopyDirection",soap_call_channel__setSrmCopyDirection,value_str);
}

/**
 * setSrmCopyTimeout
 */
int glite_channel_setSrmCopyTimeout(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 value,
    const char          * message){
    GLITE_CHANNEL_SET_NUM_VALUE2("SrmCopyTimeout",soap_call_channel__setSrmCopyTimeout,value);
}

/**
 * setSrmCopyRefreshTimeout
 */
int glite_channel_setSrmCopyRefreshTimeout(
    glite_transfer_ctx  *ctx,
    const char          *name,
    int                 value,
    const char          * message){
    GLITE_CHANNEL_SET_NUM_VALUE2("SrmCopyRefreshTimeout",soap_call_channel__setSrmCopyRefreshTimeout,value);
}

/*
 * Add a manager to the given channel
 */
int glite_channel_addManager(glite_transfer_ctx *ctx, const char *name,
                             const char *principal) {
    char *sname, *sprincipal;
    struct channel__addManagerResponse response;
    int ret;

    if (!is_ctx_ok(ctx))
        return -1;
    if (!name) {
        err_invarg(ctx, "addManager: Channel name is missing");
        return -1;
    }
    if (!principal) {
        err_invarg(ctx, "addManager: Principal name is missing");
        return -1;
    }

    /* We cannot turn a const pointer to a non-const, so we have to copy. */
    sname = soap_strdup(ctx->soap, name);
    sprincipal = soap_strdup(ctx->soap, principal);
    if (!sname || !principal) {
        err_outofmemory(ctx);
        return -1;
    }

    /* Make the SOAP call */
    ret = soap_call_channel__addManager(ctx->soap, ctx->endpoint, NULL,
                                        sname, sprincipal, &response);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "addManager");
        return -1;
    }

    soap_end(ctx->soap);
    return 0;
}

/*
 * Remove a manager from the given channel
 */
int glite_channel_removeManager(glite_transfer_ctx *ctx, const char *name,
                                const char *principal) {
    char *sname, *sprincipal;
    struct channel__removeManagerResponse response;
    int ret;

    if (!is_ctx_ok(ctx))
        return -1;
    if (!name) {
        err_invarg(ctx, "removeManager: Channel name is missing");
        return -1;
    }
    if (!principal) {
        err_invarg(ctx, "removeManager: Principal name is missing");
        return -1;
    }

    /* We cannot turn a const pointer to a non-const, so we have to copy. */
    sname = soap_strdup(ctx->soap, name);
    sprincipal = soap_strdup(ctx->soap, principal);
    if (!sname || !principal) {
        err_outofmemory(ctx);
        return -1;
    }

    /* Make the SOAP call */
    ret = soap_call_channel__removeManager(ctx->soap, ctx->endpoint, NULL,
                                           sname, sprincipal, &response);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "addManager");
        return -1;
    }

    soap_end(ctx->soap);
    return 0;
}

/*
 * List managers belonging to a channel
 */
char **glite_channel_listManagers(glite_transfer_ctx *ctx, const char *name,
                                  int *resultCnt) {
    struct channel__listManagersResponse resp;
    char **result, *sname;
    int ret;

    /* Default: error */
    if (resultCnt)
        *resultCnt = -1;

    if (!is_ctx_ok(ctx))
        return NULL;
    if (!name) {
        err_invarg(ctx, "listManagers: Channel name is missing");
        return NULL;
    }

    sname = soap_strdup(ctx->soap, name);
    if (!sname) {
        err_outofmemory(ctx);
        return NULL;
    }

    /* Make the SOAP call */
    ret = soap_call_channel__listManagers(ctx->soap, ctx->endpoint, NULL,
                                          sname, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "listManagers");
        return NULL;
    }

    result = from_soap_StringArray(ctx, resp._listManagersReturn, resultCnt);
    soap_end(ctx->soap);
    return result;
}

/*
 * Sets the share of a VO on a given channel
 */
int glite_channel_setVOShare(glite_transfer_ctx *ctx, const char *name,
                             const char *VOname, int share) {
    char *sname, *sVOname;
    struct channel__setVOShareResponse response;
    int ret;

    if (!is_ctx_ok(ctx))
        return -1;
    if (!name) {
        err_invarg(ctx, "setVOShare: Channel name is missing");
        return -1;
    }
    if (!VOname) {
        err_invarg(ctx, "setVOShare: VO name is missing");
        return -1;
    }

    /* We cannot turn a const pointer to a non-const, so we have to copy. */
    sname = soap_strdup(ctx->soap, name);
    sVOname = soap_strdup(ctx->soap, VOname);
    if (!sname || !sVOname) {
        err_outofmemory(ctx);
        return -1;
    }

    /* Make the SOAP call */
    ret = soap_call_channel__setVOShare(ctx->soap, ctx->endpoint,
                                        NULL, sname, sVOname, share, &response);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "setVOShare");
        return -1;
    }

    soap_end(ctx->soap);
    return 0;
}

/*
 * Sets the transfer limit of a VO on a given channel
 */
int glite_channel_setVOLimit(glite_transfer_ctx *ctx, const char *name,
                             const char *VOname, int limit) {
    char *sname, *sVOname;
    struct channel__setVOLimitResponse response;
    int ret;

    if (!is_ctx_ok(ctx))
        return -1;
    if (!name) {
        err_invarg(ctx, "setVOLimit: Channel name is missing");
        return -1;
    }
    if (!VOname) {
        err_invarg(ctx, "setVOLimit: VO name is missing");
        return -1;
    }

    /* We cannot turn a const pointer to a non-const, so we have to copy. */
    sname = soap_strdup(ctx->soap, name);
    sVOname = soap_strdup(ctx->soap, VOname);
    if (!sname || !sVOname) {
        err_outofmemory(ctx);
        return -1;
    }

    /* Make the SOAP call */
    ret = soap_call_channel__setVOLimit(ctx->soap, ctx->endpoint,
                                        NULL, sname, sVOname, limit, &response);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "setVOLimit");
        return -1;
    }

    soap_end(ctx->soap);
    return 0;
}


glite_transfer_ChannelAuditEntry ** glite_channel_channelAudit(glite_transfer_ctx *ctx,
        const time_t startTime, const time_t endTime, const char * restrictType,
        const int lastNentries, const char * channelName, int* resultCount) {

    char * srestrictType;
    char * schannelName;
    glite_transfer_ChannelAuditEntry **result;
    struct transfer__ChannelAuditEntry **tmp;
    struct channel__channelAuditResponse resp;
    int ret, i;

    if (!is_ctx_ok(ctx))
        return NULL;
    if (!channelName) {
        err_invarg(ctx, "channelAudit: Channel name is missing");
        return NULL;
    }

    srestrictType = soap_strdup(ctx->soap, restrictType);
    schannelName = soap_strdup(ctx->soap, channelName);

    ret = soap_call_channel__channelAudit(ctx->soap, ctx->endpoint,
                                          NULL, startTime, endTime, srestrictType, lastNentries, schannelName, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "channelAudit");
        return NULL;
    }

    /* The response may be empty */
    if (!resp._channelAuditReturn) {
        if (resultCount)
            *resultCount = 0;
        soap_end(ctx->soap);
        return NULL;
    }

    result = malloc(resp._channelAuditReturn->__size * sizeof(*result));
    if (!result) {
        err_outofmemory(ctx);
        soap_end(ctx->soap);
        return NULL;
    }

    tmp = resp._channelAuditReturn->__ptr;
    for (i = 0; i < resp._channelAuditReturn->__size; i++) {
        result[i] = from_soap_ChannelAuditEntry(ctx, tmp[i]);
        if (!result[i]) {
            glite_transfer_ChannelAuditEntry_freeArray(ctx, i,
                    result);
            soap_end(ctx->soap);
            return NULL;
        }
    }

    if (resultCount)
        *resultCount = resp._channelAuditReturn->__size;
    else
        printf("toss");
    soap_end(ctx->soap);
    return result;
}

//------------------------------------------------------------------------------
//                          Groups Management
//------------------------------------------------------------------------------

/**
 * Get a list of the names of the defined site groups.
 */
char ** glite_channel_getSiteGroupNames(
    glite_transfer_ctx  *ctx,
    int                 *resultCnt){
    // check input parameters
    if (!is_ctx_ok(ctx)){
        return NULL;
    }
    if (!resultCnt){
        err_invarg(ctx, "resultCnt is null");
        return NULL;
    }

    struct channel__getSiteGroupNamesResponse resp;
    char **result = 0;
    int ret = 0;

    // SOAP call
    ret = soap_call_channel__getSiteGroupNames(ctx->soap, ctx->endpoint, NULL, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getSiteGroupNames");
        return NULL;
    }

    result = from_soap_StringArray(ctx, resp.getSiteGroupNamesReturn, resultCnt);
    soap_end(ctx->soap);
    return result;
}

/**
 * Get the list of the sites that are members of the group with given name.
 */
char ** glite_channel_getSiteGroupMembers(
    glite_transfer_ctx  *ctx,
    char                *groupName,
    int                 *resultCnt){
    // check input parameters
    if (!is_ctx_ok(ctx)){
        return NULL;
    }
    if (!groupName){
        err_invarg(ctx, "groupName is null");
        return NULL;
    }
    if (!resultCnt){
        err_invarg(ctx, "resultCnt is null");
        return NULL;
    }

    struct channel__getSiteGroupMembersResponse resp;
    char **result = 0;
    int ret = 0;
    char *sGroupName = soap_strdup(ctx->soap, groupName);
    if (!sGroupName) {
        err_outofmemory(ctx);
        return NULL;
    }

    // SOAP call
    ret = soap_call_channel__getSiteGroupMembers(ctx->soap, ctx->endpoint, NULL, sGroupName, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getSiteGroupMembers");
        return NULL;
    }

    result = from_soap_StringArray(ctx, resp._getSiteGroupMembersReturn , resultCnt);
    soap_end(ctx->soap);
    return result;
}

/**
 * Add the site with given name to members of the given group. If the group does not exist it is created.
 */
int glite_channel_addGroupMember(
    glite_transfer_ctx  *ctx,
    char                *groupName,
    char                *siteName){

    // check input parameters
    if (!is_ctx_ok(ctx)){
        return -1;
    }
    if (!groupName){
        err_invarg(ctx, "groupName is null");
        return -1;
    }
    if (!siteName){
        err_invarg(ctx, "siteName is null");
        return -1;
    }

    struct channel__addGroupMemberResponse resp;
    char *sGroupName = soap_strdup(ctx->soap, groupName);
    char *sSiteName = soap_strdup(ctx->soap, siteName);
    if (!sGroupName || !sSiteName) {
        err_outofmemory(ctx);
        return -1;
    }

    // SOAP call
    int ret = soap_call_channel__addGroupMember(ctx->soap, ctx->endpoint, NULL, sGroupName, sSiteName, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "addGroupMember");
        return -1;
    }
    soap_end(ctx->soap);
    return 0;

}

/**
 * Remove the site with given name from members of the given group. If after the operation the group is empty, the group is deleted.
 */
int glite_channel_removeGroupMember(
    glite_transfer_ctx  *ctx,
    char                *groupName,
    char                *siteName){

    // check input parameters
    if (!is_ctx_ok(ctx)){
        return -1;
    }
    if (!groupName){
        err_invarg(ctx, "groupName is null");
        return -1;
    }
    if (!siteName){
        err_invarg(ctx, "siteName is null");
        return -1;
    }

    struct channel__removeGroupMemberResponse resp;
    char *sGroupName = soap_strdup(ctx->soap, groupName);
    char *sSiteName = soap_strdup(ctx->soap, siteName);
    if (!sGroupName || !sSiteName) {
        err_outofmemory(ctx);
        return -1;
    }

    // SOAP call
    int ret = soap_call_channel__removeGroupMember(ctx->soap, ctx->endpoint, NULL, sGroupName, sSiteName, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "removeGroupMember");
        return -1;
    }
    soap_end(ctx->soap);
    return 0;
}



