/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Data Management - Soap conversion functions
 *
 *  Authors: Gabor Gombas <Gabor.Gombas@cern.ch>
 *  Version info: $Id: channel_soapconv.c,v 1.15 2009/01/19 14:29:51 ptedesco Exp $
 *  Release: $Name:  $
 *
 *  Revision history:
 *  $Log: channel_soapconv.c,v $
 *  Revision 1.15  2009/01/19 14:29:51  ptedesco
 *  glite-data-transfer-cli_R_3_6_0_1
 *
 *  Revision 1.14  2008/11/26 08:57:59  ptedesco
 *  added listing for Channel3 objects
 *
 *  Revision 1.13  2008/05/08 15:03:47  ptedesco
 *  Fix bug #33406
 *
 *  Revision 1.12  2008/03/17 15:59:09  knienart
 *  New interface methods. some bugfixes
 *
 *  Revision 1.11  2008/03/14 11:28:04  knienart
 *   New version
 *
 *  Revision 1.10  2007/03/02 12:23:26  badino
 *  Modifid in order to support gsoap 2.7.9b
 *
 *  Revision 1.9  2006/11/30 15:38:14  mccance
 *  Added support for VO limits. New cli tool and support in channel list.
 *
 *  Revision 1.8  2006/08/21 13:24:35  mccance
 *  Audit command added
 *
 *  Revision 1.7  2006/08/11 15:21:09  mccance
 *  extended channel info
 *
 *  Revision 1.6  2005/07/21 14:15:37  zfarkas
 *  Print VOName and priority when listing transfer jobs, and VO shares
 *  when listing channels.
 *
 *  Revision 1.5  2005/07/20 11:33:58  zfarkas
 *  Clients using the new interface
 *
 *  Revision 1.4  2005/03/29 15:11:48  gombasg
 *  Track interface changes
 *
 *  Revision 1.3  2005/03/04 18:12:59  gombasg
 *  Transfer CLI updates
 *
 *  Revision 1.2  2005/03/04 15:29:42  gombasg
 *  Transfer CLI updates
 *
 *  Revision 1.1  2005/03/03 17:26:00  gombasg
 *  New transfer CLI import
 *
 *
 */

#include <glite/data/transfer/c/channel-simple.h>

#include <glite/data/glite-util.h>

#include "channel_internal.h"

/**********************************************************************
 * Conversion to/from gSOAP data types
 */

/* Convert gSOAP's ArrayOf_USCOREsoapenc_USCOREstring to char ** */
char **from_soap_StringArray(glite_transfer_ctx *ctx,
                             struct ArrayOf_USCOREsoapenc_USCOREstring *resp, int *resultCount) {
    char **result;
    int64_t i;

    /* The result may be empty */
    if (!resp) {
        if (resultCount)
            *resultCount = 0;
        return NULL;
    }

    result = malloc(resp->__size* sizeof(*result));
    if (resp->__size&& !result) {
        err_outofmemory(ctx);
        return NULL;
    }

    for (i = 0; i < resp->__size; i++) {
        if (resp->__ptr[i]) {
            result[i] = strdup(resp->__ptr[i]);
            if (!result[i]) {
                err_outofmemory(ctx);
                glite_freeStringArray(i, result);
                return NULL;
            }
        } else
            result[i] = NULL;
    }

    if (resultCount)
        *resultCount = resp->__size;

    return result;
}

//------------------------------------------------------------------------------
// Channel
//------------------------------------------------------------------------------

struct transfer__Channel *to_soap_Channel(
    struct soap                     *soap,
    const glite_transfer_Channel    *channel) {
    
    struct transfer__Channel *schannel;

    if (!channel)
        return NULL;

    schannel = soap_malloc(soap, sizeof(*schannel));
    if (!schannel)
        return NULL;
    memset(schannel, 0, sizeof(*schannel));

    STRING_TO_SOAP(channel, channelName);
    STRING_TO_SOAP(channel, sourceSite);
    STRING_TO_SOAP(channel, destSite);
    STRING_TO_SOAP(channel, contact);

    schannel->state = soap_strdup(soap,
                                  glite_channel_state_str(channel->state));
    if (!schannel->state)
        return NULL;

    NUM_TO_SOAP(channel, bandwidth);
    NUM_TO_SOAP(channel, nominalThroughput);
    NUM_TO_SOAP(channel, numberOfFiles);
    NUM_TO_SOAP(channel, numberOfStreams);

    return schannel;
}

glite_transfer_Channel *from_soap_Channel(
    glite_transfer_ctx              *ctx,
    const struct transfer__Channel  *schannel) {
    
    int i;
    glite_transfer_Channel *channel;

    if (!schannel)
        return NULL;
    channel = glite_transfer_Channel_new(ctx, schannel->channelName,
                                         schannel->sourceSite, schannel->destSite);
    if (!channel)
        return NULL;

    STRING_FROM_SOAP(channel, channelName);
    STRING_FROM_SOAP(channel, contact);

    channel->state = glite_channel_state_parse(ctx, schannel->state);
    if (channel->state == GLITE_CHANNEL_UNKNOWN) {
        glite_transfer_Channel_free(ctx, channel);
        return NULL;
    }

    NUM_FROM_SOAP(channel, bandwidth);
    NUM_FROM_SOAP(channel, nominalThroughput);
    NUM_FROM_SOAP(channel, numberOfFiles);
    NUM_FROM_SOAP(channel, numberOfStreams);
    NUM_FROM_SOAP(channel, __sizeVOShares);
    channel->VOShares = g_new0(glite_transfer_VOShareElement *, channel->__sizeVOShares);
    for (i = 0; i < channel->__sizeVOShares; i++) {
        channel->VOShares[i] = g_new0(glite_transfer_VOShareElement, 1);
        #if _GSOAP_VERSION >= 0x030709

        channel->VOShares[i]->VOName = g_strdup(schannel->VOShares[i].string1);
        channel->VOShares[i]->share = g_strdup(schannel->VOShares[i].string2);
        #else

        channel->VOShares[i]->VOName = g_strdup(schannel->VOShares[i]->string1);
        channel->VOShares[i]->share = g_strdup(schannel->VOShares[i]->string2);
        #endif

    }

    return channel;
}

struct transfer__Channel2 *to_soap_Channel2(
    struct soap                     *soap,
    const glite_transfer_Channel2   *channel2) {
    
    struct transfer__Channel2 *schannel2;

    if (!channel2)
        return NULL;

    schannel2 = soap_malloc(soap, sizeof(*schannel2));
    if (!schannel2)
        return NULL;
    memset(schannel2, 0, sizeof(*schannel2));

    STRING_TO_SOAP(channel2, channelName);
    STRING_TO_SOAP(channel2, sourceSite);
    STRING_TO_SOAP(channel2, destSite);
    STRING_TO_SOAP(channel2, contact);

    schannel2->state = soap_strdup(soap,
                                   glite_channel_state_str(channel2->state));
    if (!schannel2->state)
        return NULL;

    NUM_TO_SOAP(channel2, bandwidth);
    NUM_TO_SOAP(channel2, nominalThroughput);
    NUM_TO_SOAP(channel2, numberOfFiles);
    NUM_TO_SOAP(channel2, numberOfStreams);

    return schannel2;
}


glite_transfer_Channel2 *from_soap_Channel2(
    glite_transfer_ctx              *ctx,
    const struct transfer__Channel2 *schannel2) {
    
    int i;
    glite_transfer_Channel2 *channel2;

    if (!schannel2)
        return NULL;
    channel2 = glite_transfer_Channel2_new(ctx, schannel2->channelName,
                                           schannel2->sourceSite, schannel2->destSite);
    if (!channel2)
        return NULL;

    STRING_FROM_SOAP(channel2, channelName);
    STRING_FROM_SOAP(channel2, contact);
    STRING_FROM_SOAP(channel2, lastModifierDn);
    STRING_FROM_SOAP(channel2, message);
    STRING_FROM_SOAP(channel2, tcpBufferSize);

    channel2->state = glite_channel_state_parse(ctx, schannel2->state);
    if (channel2->state == GLITE_CHANNEL_UNKNOWN) {
        glite_transfer_Channel2_free(ctx, channel2);
        return NULL;
    }

    OPT_NUM_FROM_SOAP(channel2, lastModificationTime, 0);
    NUM_FROM_SOAP(channel2, bandwidth);
    NUM_FROM_SOAP(channel2, nominalThroughput);
    NUM_FROM_SOAP(channel2, numberOfFiles);
    NUM_FROM_SOAP(channel2, numberOfStreams);
    NUM_FROM_SOAP(channel2, __sizeVOShares);
    channel2->VOShares = g_new0(glite_transfer_VOShareElement *, channel2->__sizeVOShares);
    for (i = 0; i < channel2->__sizeVOShares; i++) {
        channel2->VOShares[i] = g_new0(glite_transfer_VOShareElement, 1);
        #if _GSOAP_VERSION >= 0x030709

        channel2->VOShares[i]->VOName = g_strdup(schannel2->VOShares[i].string1);
        channel2->VOShares[i]->share = g_strdup(schannel2->VOShares[i].string2);
        #else

        channel2->VOShares[i]->VOName = g_strdup(schannel2->VOShares[i]->string1);
        channel2->VOShares[i]->share = g_strdup(schannel2->VOShares[i]->string2);
        #endif

    }
    NUM_FROM_SOAP(channel2, __sizeVOLimits);
    channel2->VOLimits = g_new0(glite_transfer_VOLimitElement *, channel2->__sizeVOLimits);
    for (i = 0; i < channel2->__sizeVOLimits; i++) {
        channel2->VOLimits[i] = g_new0(glite_transfer_VOLimitElement, 1);
        #if _GSOAP_VERSION >= 0x030709

        channel2->VOLimits[i]->VOName = g_strdup(schannel2->VOLimits[i].string1);
        channel2->VOLimits[i]->limit = g_strdup(schannel2->VOLimits[i].string2);
        #else

        channel2->VOLimits[i]->VOName = g_strdup(schannel2->VOLimits[i]->string1);
        channel2->VOLimits[i]->limit = g_strdup(schannel2->VOLimits[i]->string2);
        #endif

    }

    return channel2;
}

struct transfer__Channel3 *to_soap_Channel3(
    struct soap                     *soap,
    const glite_transfer_Channel3   *channel3) {
    
    struct transfer__Channel3 *schannel3;

    if (!channel3)
        return NULL;

    schannel3 = soap_malloc(soap, sizeof(*schannel3));
    if (!schannel3)
        return NULL;
    memset(schannel3, 0, sizeof(*schannel3));

    STRING_TO_SOAP(channel3, channelName);
    STRING_TO_SOAP(channel3, sourceSite);
    STRING_TO_SOAP(channel3, destSite);
    STRING_TO_SOAP(channel3, contact);

    schannel3->state = soap_strdup(soap,glite_channel_state_str(channel3->state));
    if (!schannel3->state)
        return NULL;

    NUM_TO_SOAP(channel3, bandwidth);
    NUM_TO_SOAP(channel3, nominalThroughput);
    NUM_TO_SOAP(channel3, numberOfFiles);
    NUM_TO_SOAP(channel3, numberOfStreams);

    return schannel3;
}


glite_transfer_Channel3 *from_soap_Channel3(
    glite_transfer_ctx              *ctx,
    const struct transfer__Channel3 *schannel3) {
    
    int i;
    glite_transfer_Channel3 *channel3;

    if (!schannel3)
        return NULL;
    channel3 = glite_transfer_Channel3_new(ctx, schannel3->channelName,schannel3->sourceSite, schannel3->destSite);
    if (!channel3)
        return NULL;

    STRING_FROM_SOAP(channel3, channelName);
    STRING_FROM_SOAP(channel3, contact);
    STRING_FROM_SOAP(channel3, lastModifierDn);
    STRING_FROM_SOAP(channel3, message);
    STRING_FROM_SOAP(channel3, tcpBufferSize);

    channel3->state = glite_channel_state_parse(ctx, schannel3->state);
    if (channel3->state == GLITE_CHANNEL_UNKNOWN) {
        glite_transfer_Channel3_free(ctx, channel3);
        return NULL;
    }

    OPT_NUM_FROM_SOAP(channel3, lastModificationTime, 0);
    NUM_FROM_SOAP(channel3, bandwidth);
    NUM_FROM_SOAP(channel3, nominalThroughput);
    NUM_FROM_SOAP(channel3, numberOfFiles);
    NUM_FROM_SOAP(channel3, numberOfStreams);
    
    NUM_FROM_SOAP(channel3, urlCopyFirstTxmarkTo);
    NUM_FROM_SOAP(channel3, targetDirCheck);
    OPT_NUM_FROM_SOAP(channel3, lastActiveTime, 0);
#if 0    
    NUM_FROM_SOAP(channel3, seLimit);
    STRING_FROM_SOAP(channel3, channelType);
#endif
    STRING_FROM_SOAP(channel3, blockSize);
    NUM_FROM_SOAP(channel3, httpTimeout);
    STRING_FROM_SOAP(channel3, transferLogLevel);
    NUM_FROM_SOAP(channel3, preparingFilesRatio);
    STRING_FROM_SOAP(channel3, transferType);
    NUM_FROM_SOAP(channel3, urlCopyPutTimeout);
    NUM_FROM_SOAP(channel3, urlCopyPutDoneTimeout);
    NUM_FROM_SOAP(channel3, urlCopyGetTimeout);
    NUM_FROM_SOAP(channel3, urlCopyGetDoneTimeout);
    NUM_FROM_SOAP(channel3, urlCopyTransferTimeout);
    NUM_FROM_SOAP(channel3, urlCopyTransferMarkersTimeout);
    NUM_FROM_SOAP(channel3, urlCopyNoProgressTimeout);
    NUM_FROM_SOAP(channel3, urlCopyTransferTimeoutPerMB);
    STRING_FROM_SOAP(channel3, srmCopyDirection);
    NUM_FROM_SOAP(channel3, srmCopyTimeout);
    NUM_FROM_SOAP(channel3, srmCopyRefreshTimeout);
    
    NUM_FROM_SOAP(channel3, __sizeVOShares);
    channel3->VOShares = g_new0(glite_transfer_VOShareElement *, channel3->__sizeVOShares);
    for (i = 0; i < channel3->__sizeVOShares; i++) {
        channel3->VOShares[i] = g_new0(glite_transfer_VOShareElement, 1);
        #if _GSOAP_VERSION >= 0x030709
            channel3->VOShares[i]->VOName = g_strdup(schannel3->VOShares[i].string1);
            channel3->VOShares[i]->share = g_strdup(schannel3->VOShares[i].string2);
        #else
            channel3->VOShares[i]->VOName = g_strdup(schannel3->VOShares[i]->string1);
            channel3->VOShares[i]->share = g_strdup(schannel3->VOShares[i]->string2);
        #endif

    }
    NUM_FROM_SOAP(channel3, __sizeVOLimits);
    channel3->VOLimits = g_new0(glite_transfer_VOLimitElement *, channel3->__sizeVOLimits);
    for (i = 0; i < channel3->__sizeVOLimits; i++) {
        channel3->VOLimits[i] = g_new0(glite_transfer_VOLimitElement, 1);
        #if _GSOAP_VERSION >= 0x030709
            channel3->VOLimits[i]->VOName = g_strdup(schannel3->VOLimits[i].string1);
            channel3->VOLimits[i]->limit = g_strdup(schannel3->VOLimits[i].string2);
        #else
            channel3->VOLimits[i]->VOName = g_strdup(schannel3->VOLimits[i]->string1);
            channel3->VOLimits[i]->limit = g_strdup(schannel3->VOLimits[i]->string2);
        #endif
    }
    return channel3;
}

//------------------------------------------------------------------------------
// ChannelAuditEntry
//------------------------------------------------------------------------------

glite_transfer_ChannelAuditEntry *from_soap_ChannelAuditEntry(glite_transfer_ctx *ctx,
        const struct transfer__ChannelAuditEntry *sca) {

    glite_transfer_ChannelAuditEntry *ca;

    if (!sca)
        return NULL;
    ca = glite_transfer_ChannelAuditEntry_new(ctx, sca->channelName);
    if (!ca)
        return NULL;

    STRING_FROM_SOAP(ca, channelName);
    STRING_FROM_SOAP(ca, contact);
    STRING_FROM_SOAP(ca, adminDN);
    STRING_FROM_SOAP(ca, ftsNode);
    STRING_FROM_SOAP(ca, clientIP);
    STRING_FROM_SOAP(ca, message);
    STRING_FROM_SOAP(ca, tcpBufferSize);

    ca->channelState = glite_channel_state_parse(ctx, sca->channelState);
    // Don't parse the bloody states: take what you're given (since some are empty by design).

    OPT_NUM_FROM_SOAP(ca, modificationTime, 0);
    NUM_FROM_SOAP(ca, bandwidth);
    NUM_FROM_SOAP(ca, nominalThroughput);
    NUM_FROM_SOAP(ca, nofiles);
    NUM_FROM_SOAP(ca, nostreams);
    NUM_FROM_SOAP(ca, auditID);

    return ca;
}

