/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Management - Basic data type methods
 *
 *  Authors: Gabor Gombas <Gabor.Gombas@cern.ch>
 *  Version info: $Id: datatypes.c,v 1.22 2009/07/06 15:30:02 szamsu Exp $
 */

#include <glite/data/transfer/c/transfer-simple.h>
#include <glite/data/glite-util.h>
#include "internal.h"

/**********************************************************************
 * Data type methods
 */

glite_transfer_FileTransferStatus *glite_transfer_FileTransferStatus_new(glite_transfer_ctx *ctx,
        const char *source, const char *dest, glite_transfer_state state) {
    glite_transfer_FileTransferStatus *tstatus;

    tstatus = g_new0(glite_transfer_FileTransferStatus, 1);
    if (!tstatus) {
        err_outofmemory(ctx);
        return NULL;
    }

    tstatus->sourceSURL = (source ? g_strdup(source) : g_strdup("(unknown)"));
    tstatus->destSURL = (dest ? g_strdup(dest) : g_strdup("(unknown)"));
    tstatus->transferFileState = state;
    return tstatus;
}

void glite_transfer_FileTransferStatus_free(glite_transfer_ctx *ctx G_GNUC_UNUSED,
        glite_transfer_FileTransferStatus *status) {
    if (!status)
        return;

    g_free(status->logicalName);
    g_free(status->sourceSURL);
    g_free(status->destSURL);
    g_free(status->reason);
    g_free(status->reason_class);
    g_free(status);
}

void glite_transfer_FileTransferStatus_freeArray(glite_transfer_ctx *ctx,
        int nitems, glite_transfer_FileTransferStatus *status[]) {
    int i;

    for (i = 0; i < nitems; i++)
        glite_transfer_FileTransferStatus_free(ctx, status[i]);
    g_free(status);
}

glite_transfer_FileTransferStatus *glite_transfer_FileTransferStatus_clone(glite_transfer_ctx *ctx,
        const glite_transfer_FileTransferStatus *orig) {
    glite_transfer_FileTransferStatus *clone;

    if (!orig)
        return NULL;

    clone = glite_transfer_FileTransferStatus_new(ctx, orig->sourceSURL,
            orig->destSURL, orig->transferFileState);
    if (!clone)
        return NULL;

    clone->logicalName = g_strdup(orig->logicalName);
    clone->reason = g_strdup(orig->reason);
    clone->reason_class = g_strdup(orig->reason_class);
    clone->numFailures = orig->numFailures;
    return clone;
}

int glite_transfer_FileTransferStatus_setLogicalName(glite_transfer_ctx *ctx,
        glite_transfer_FileTransferStatus *status, const char *name) {
    if (!status) {
        err_invarg(ctx, "Transfer status is missing");
        return -1;
    }
    if (!name) {
        err_invarg(ctx, "Logical name is missing");
        return -1;
    }

    g_free(status->logicalName);
    status->logicalName = g_strdup(name);
    return 0;
}

int glite_transfer_FileTransferStatus_setReason(glite_transfer_ctx *ctx,
        glite_transfer_FileTransferStatus *status, const char *reason) {
    if (!status) {
        err_invarg(ctx, "Transfer status is missing");
        return -1;
    }
    if (!reason) {
        err_invarg(ctx, "Reason is missing");
        return -1;
    }

    g_free(status->reason);
    status->reason = g_strdup(reason);
    return 0;
}

int glite_transfer_FileTransferStatus_setReasonClass(glite_transfer_ctx *ctx,
        glite_transfer_FileTransferStatus *status, const char *reason_class) {
    if (!status) {
        err_invarg(ctx, "Transfer status is missing");
        return -1;
    }
    if (!reason_class) {
        err_invarg(ctx, "Reason class is missing");
        return -1;
    }

    g_free(status->reason_class);
    status->reason_class = g_strdup(reason_class);
    return 0;
}

glite_transfer_JobStatus *glite_transfer_JobStatus_new(glite_transfer_ctx *ctx,
        const char *jobId, glite_transfer_state jobStatus) {
    glite_transfer_JobStatus *status;

    if (!jobId) {
        err_invarg(ctx, "Job ID is missing");
        return NULL;
    }

    status = g_new0(glite_transfer_JobStatus, 1);
    if (!status) {
        err_outofmemory(ctx);
        return NULL;
    }

    status->jobId = g_strdup(jobId);
    status->jobStatus = jobStatus;
    return status;
}

void glite_transfer_JobStatus_free(glite_transfer_ctx *ctx G_GNUC_UNUSED,
                                   glite_transfer_JobStatus *status) {
    if (!status)
        return;

    g_free(status->jobId);
    g_free(status->channelName);
    g_free(status->clientDN);
    g_free(status->reason);
    g_free(status);
}

void glite_transfer_JobStatus_freeArray(glite_transfer_ctx *ctx, int nitems,
                                        glite_transfer_JobStatus *status[]) {
    int i;

    if (!status)
        return;

    for (i = 0; i < nitems; i++)
        glite_transfer_JobStatus_free(ctx, status[i]);
    g_free(status);
}

glite_transfer_JobStatus *glite_transfer_JobStatus_clone(glite_transfer_ctx *ctx,
        const glite_transfer_JobStatus *orig) {
    glite_transfer_JobStatus *clone;

    if (!orig)
        return NULL;

    clone = glite_transfer_JobStatus_new(ctx, orig->jobId, orig->jobStatus);
    if (!clone)
        return NULL;

    clone->channelName = g_strdup(orig->channelName);
    clone->clientDN = g_strdup(orig->clientDN);
    clone->reason = g_strdup(orig->reason);
    clone->submitTime = orig->submitTime;
    clone->numFiles = orig->numFiles;
    return clone;
}

glite_transfer_PlacementJob *glite_transfer_PlacementJob_new(glite_transfer_ctx *ctx,
        const char *sourceSE, const char *destSE) {
    glite_transfer_PlacementJob *job;

    if (!sourceSE) {
        err_invarg(ctx, "Source SE is missing");
        return NULL;
    }
    if (!destSE) {
        err_invarg(ctx, "Destination SE is missing");
        return NULL;
    }

    job = g_new0(glite_transfer_PlacementJob, 1);
    if (!job)
        err_outofmemory(ctx);
    job->sourceSE = g_strdup(sourceSE);
    job->destSE = g_strdup(destSE);

    return job;
}

void glite_transfer_PlacementJob_free(glite_transfer_ctx *ctx,
                                      glite_transfer_PlacementJob *job) {
    if (!job)
        return;

    glite_freeStringArray(job->__sizeLogicalFiles, job->logicalFiles);
    g_free(job->sourceSE);
    g_free(job->destSE);
    glite_transfer_TransferParams_free(ctx, job->jobParams);
    g_free(job->credential);
    g_free(job);
}

glite_transfer_PlacementJob *glite_transfer_PlacementJob_clone(glite_transfer_ctx *ctx,
        const glite_transfer_PlacementJob *orig) {
    glite_transfer_PlacementJob *clone;
    unsigned i;

    if (!orig)
        return NULL;

    clone = glite_transfer_PlacementJob_new(ctx, orig->sourceSE,
                                            orig->destSE);
    if (!clone)
        return NULL;

    clone->logicalFiles = g_new(char *, orig->__sizeLogicalFiles);
    for (i = 0; i < orig->__sizeLogicalFiles; i++)
        clone->logicalFiles[i] = g_strdup(orig->logicalFiles[i]);
    clone->__sizeLogicalFiles = orig->__sizeLogicalFiles;

    clone->jobParams = glite_transfer_TransferParams_clone(ctx,
                       orig->jobParams);
    if (orig->jobParams && !clone->jobParams) {
        glite_transfer_PlacementJob_free(ctx, clone);
        return NULL;
    }

    clone->credential = g_strdup(orig->credential);
    return clone;
}

int glite_transfer_PlacementJob_setParams(glite_transfer_ctx *ctx,
        glite_transfer_PlacementJob *job,
        const glite_transfer_TransferParams *params) {
    glite_transfer_TransferParams *clone;

    clone = glite_transfer_TransferParams_clone(ctx, params);
    if (params && !clone)
        return -1;

    glite_transfer_TransferParams_free(ctx, job->jobParams);
    job->jobParams = clone;
    return 0;
}

int glite_transfer_PlacementJob_add(glite_transfer_ctx *ctx,
                                    glite_transfer_PlacementJob *job, const char *logicalFile) {
    char **tmp;

    tmp = g_renew(char *, job->logicalFiles, job->__sizeLogicalFiles + 1);
    if (!tmp) {
        err_outofmemory(ctx);
        return -1;
    }

    tmp[job->__sizeLogicalFiles++] = g_strdup(logicalFile);
    job->logicalFiles = tmp;
    return 0;
}

glite_transfer_TransferJobSummary *glite_transfer_TransferJobSummary_new(glite_transfer_ctx *ctx) {
    glite_transfer_TransferJobSummary *summary;

    summary = g_new0(glite_transfer_TransferJobSummary, 1);
    if (!summary)
        err_outofmemory(ctx);
    return summary;
}

void glite_transfer_TransferJobSummary_free(glite_transfer_ctx *ctx,
        glite_transfer_TransferJobSummary *summary) {
    if (!summary)
        return;

    glite_transfer_JobStatus_free(ctx, summary->jobStatus);
    g_free(summary);
}

glite_transfer_TransferJobSummary *glite_transfer_TransferJobSummary_clone(glite_transfer_ctx *ctx,
        const glite_transfer_TransferJobSummary *orig) {
    glite_transfer_TransferJobSummary *clone;

    if (!orig)
        return NULL;

    clone = glite_transfer_TransferJobSummary_new(ctx);
    if (!clone)
        return NULL;

    clone->jobStatus = glite_transfer_JobStatus_clone(ctx, orig->jobStatus);
    if (orig->jobStatus && !clone->jobStatus) {
        glite_transfer_TransferJobSummary_free(ctx, clone);
        return NULL;
    }

    clone->numDone = orig->numDone;
    clone->numActive = orig->numActive;
    clone->numPending = orig->numPending;
    clone->numCanceled = orig->numCanceled;
    clone->numCanceling = orig->numCanceling;
    clone->numFailed = orig->numFailed;
    clone->numFinished = orig->numFinished;
    clone->numSubmitted = orig->numSubmitted;
    clone->numHold = orig->numHold;
    clone->numWaiting = orig->numWaiting;
    clone->numCatalogFailed = orig->numCatalogFailed;
    clone->numRestarted = orig->numRestarted;

    return clone;
}



glite_transfer_TransferJobSummary2 *glite_transfer_TransferJobSummary2_new(glite_transfer_ctx *ctx) {
    glite_transfer_TransferJobSummary2 *summary;

    summary = g_new0(glite_transfer_TransferJobSummary2, 1);
    if (!summary)
        err_outofmemory(ctx);
    return summary;
}

void glite_transfer_TransferJobSummary2_free(glite_transfer_ctx *ctx,
        glite_transfer_TransferJobSummary2 *summary) {
    if (!summary)
        return;

    glite_transfer_JobStatus_free(ctx, summary->jobStatus);
    g_free(summary);
}

glite_transfer_TransferJobSummary2 *glite_transfer_TransferJobSummary2_clone(glite_transfer_ctx *ctx,
        const glite_transfer_TransferJobSummary2 *orig) {
    glite_transfer_TransferJobSummary2 *clone;

    if (!orig)
        return NULL;

    clone = glite_transfer_TransferJobSummary2_new(ctx);
    if (!clone)
        return NULL;

    clone->jobStatus = glite_transfer_JobStatus_clone(ctx, orig->jobStatus);
    if (orig->jobStatus && !clone->jobStatus) {
        glite_transfer_TransferJobSummary2_free(ctx, clone);
        return NULL;
    }

    clone->numDone = orig->numDone;
    clone->numActive = orig->numActive;
    clone->numPending = orig->numPending;
    clone->numCanceled = orig->numCanceled;
    clone->numCanceling = orig->numCanceling;
    clone->numFailed = orig->numFailed;
    clone->numFinished = orig->numFinished;
    clone->numSubmitted = orig->numSubmitted;
    clone->numHold = orig->numHold;
    clone->numWaiting = orig->numWaiting;
    clone->numCatalogFailed = orig->numCatalogFailed;
    clone->numRestarted = orig->numRestarted;
    clone->numReady = orig->numReady;
    clone->numFinishing = orig->numFinishing;
    clone->numAwaitingPrestage = orig->numAwaitingPrestage;
    clone->numPrestaging = orig->numPrestaging;
    clone->numWaitingCatalogRegistration = orig->numWaitingCatalogRegistration;
    clone->numWaitingCatalogResolution = orig->numWaitingCatalogResolution;
    clone->numWaitingPrestage = orig->numWaitingPrestage;

    return clone;
}












glite_transfer_TransferJobElement *glite_transfer_TransferJobElement_new(glite_transfer_ctx *ctx,
        const char *src, const char *dst, const char *cksm) {
    glite_transfer_TransferJobElement *elem;

    if (!src || *src == '\0') {
        err_invarg(ctx, "Source SURL is missing");
        return NULL;
    }
    if (!dst && !ctx)
        return NULL;

    if (*dst == '\0') {
        err_invarg(ctx, "Destination SURL is empty");
        return NULL;
    }

    if (cksm && !strchr(cksm, ':')) {
        err_invarg(ctx, "Checksum format is not valid (ALGORITHM:1234af)");
        return NULL;
    }

    elem = g_new0(glite_transfer_TransferJobElement, 1);
    if (!elem) {
        err_outofmemory(ctx);
        return NULL;
    }

    elem->source = g_strdup(src);
    elem->dest = g_strdup(dst);
    elem->checksum = g_strdup(cksm);
    return elem;
}

void glite_transfer_TransferJobElement_free(glite_transfer_ctx *ctx G_GNUC_UNUSED,
        glite_transfer_TransferJobElement *elem) {
    if (!elem)
        return;

    g_free(elem->source);
    g_free(elem->dest);
    g_free(elem->checksum);
    g_free(elem);
}

void glite_transfer_TransferJobElement_freeArray(glite_transfer_ctx *ctx,
        int nitems, glite_transfer_TransferJobElement *elems[]) {
    int i;

    for (i = 0; i < nitems; i++)
        glite_transfer_TransferJobElement_free(ctx, elems[i]);
    g_free(elems);
}

glite_transfer_TransferJobElement *glite_transfer_TransferJobElement_clone(glite_transfer_ctx *ctx,
        const glite_transfer_TransferJobElement *orig) {
    if (!orig)
        return NULL;

    return glite_transfer_TransferJobElement_new(ctx, orig->source, orig->dest, orig->checksum);
}

glite_transfer_TransferJob *glite_transfer_TransferJob_new(glite_transfer_ctx *ctx) {
    glite_transfer_TransferJob *job;

    job = g_new0(glite_transfer_TransferJob, 1);
    if (!job)
        err_outofmemory(ctx);
    return job;
}

void glite_transfer_TransferJob_free(glite_transfer_ctx *ctx,
                                     glite_transfer_TransferJob *job) {
    if (!job)
        return;

    glite_transfer_TransferJobElement_freeArray(ctx,
            job->__sizetransferJobElements, job->transferJobElements);
    glite_transfer_TransferParams_free(ctx, job->jobParams);
    g_free(job->credential);
    g_free(job);
}

glite_transfer_TransferJob *glite_transfer_TransferJob_clone(glite_transfer_ctx *ctx,
        const glite_transfer_TransferJob *orig) {
    glite_transfer_TransferJob *clone;
    unsigned i;

    if (!orig)
        return NULL;

    clone = glite_transfer_TransferJob_new(ctx);
    if (!clone)
        return NULL;

    clone->transferJobElements = g_new(glite_transfer_TransferJobElement *,
                                       orig->__sizetransferJobElements);
    for (i = 0; i < orig->__sizetransferJobElements; i++) {
        clone->transferJobElements[i] =
            glite_transfer_TransferJobElement_clone(ctx,
                                                    orig->transferJobElements[i]);
        if (!clone->transferJobElements[i]) {
            clone->__sizetransferJobElements = i;
            glite_transfer_TransferJob_free(ctx, clone);
            return NULL;
        }
    }
    clone->__sizetransferJobElements = orig->__sizetransferJobElements;

    clone->jobParams = glite_transfer_TransferParams_clone(ctx,
                       orig->jobParams);
    if (orig->jobParams && !clone->jobParams) {
        glite_transfer_TransferJob_free(ctx, clone);
        return NULL;
    }

    clone->credential = g_strdup(orig->credential);
    return clone;
}

int glite_transfer_TransferJob_setParams(glite_transfer_ctx *ctx,
        glite_transfer_TransferJob *job,
        const glite_transfer_TransferParams *params) {
    glite_transfer_TransferParams *clone;

    clone = glite_transfer_TransferParams_clone(ctx, params);
    if (params && !clone)
        return -1;

    glite_transfer_TransferParams_free(ctx, job->jobParams);
    job->jobParams = clone;
    return 0;
}

int glite_transfer_TransferJob_add(glite_transfer_ctx *ctx,
                                   glite_transfer_TransferJob *job,
                                   const char *src, const char *dst, const char *cksm) {
    glite_transfer_TransferJobElement *elem, **tmp;

    elem = glite_transfer_TransferJobElement_new(ctx, src, dst, cksm);
    if (!elem)
        return -1;

    tmp = g_renew(glite_transfer_TransferJobElement *,
                  job->transferJobElements,
                  job->__sizetransferJobElements + 1);
    if (!tmp) {
        err_outofmemory(ctx);
        glite_transfer_TransferJobElement_free(ctx, elem);
        return -1;
    }

    tmp[job->__sizetransferJobElements++] = elem;
    job->transferJobElements = tmp;
    return 0;
}

glite_transfer_TransferParams *glite_transfer_TransferParams_new(glite_transfer_ctx *ctx) {
    glite_transfer_TransferParams *params;

    params = g_new0(glite_transfer_TransferParams, 1);
    if (!params)
        err_outofmemory(ctx);
    return params;
}

void glite_transfer_TransferParams_free(glite_transfer_ctx *ctx G_GNUC_UNUSED,
                                        glite_transfer_TransferParams *params) {
    unsigned i;

    if (!params)
        return;

    for (i = 0; i < params->num; i++) {
        g_free(params->keys[i]);
        g_free(params->values[i]);
    }
    g_free(params->keys);
    g_free(params->values);
    g_free(params);
}

glite_transfer_TransferParams *glite_transfer_TransferParams_clone(glite_transfer_ctx *ctx,
        const glite_transfer_TransferParams *orig) {
    glite_transfer_TransferParams *clone;
    unsigned i;

    if (!orig)
        return NULL;

    clone = glite_transfer_TransferParams_new(ctx);
    if (!clone)
        return NULL;

    clone->keys = g_new(char *, orig->num);
    clone->values = g_new(char *, orig->num);

    for (i = 0; i < orig->num; i++) {
        clone->keys[i] = g_strdup(orig->keys[i]);
        clone->values[i] = g_strdup(orig->values[i]);
    }
    clone->num = orig->num;

    return clone;
}

int glite_transfer_TransferParams_add(glite_transfer_ctx *ctx,
                                      glite_transfer_TransferParams *params, const char *key, const char *value) {
    char **tmp;

    tmp = g_renew(char *, params->keys, params->num + 1);
    if (!tmp) {
        err_outofmemory(ctx);
        return -1;
    }
    params->keys = tmp;
    tmp = g_renew(char *, params->values, params->num + 1);
    if (!tmp) {
        err_outofmemory(ctx);
        return -1;
    }
    params->values = tmp;

    params->keys[params->num] = g_strdup(key);
    params->values[params->num] = g_strdup(value);
    params->num++;
    return 0;
}

void glite_transfer_ChannelAuditEntry_freeArray(glite_transfer_ctx *ctx,
        int nitems, glite_transfer_ChannelAuditEntry *ca[]) {
    int i;

    for (i = 0; i < nitems; i++)
        glite_transfer_ChannelAuditEntry_free(ctx, ca[i]);
    g_free(ca);
}

glite_transfer_ChannelAuditEntry *glite_transfer_ChannelAuditEntry_new(glite_transfer_ctx *ctx,
        const char *name) {
    glite_transfer_ChannelAuditEntry *channel;

    channel = g_new0(glite_transfer_ChannelAuditEntry, 1);
    if (!channel) {
        err_outofmemory(ctx);
        return NULL;
    }

    channel->channelName = g_strdup(name);

    return channel;
}

void glite_transfer_ChannelAuditEntry_free(glite_transfer_ctx *ctx G_GNUC_UNUSED,
        glite_transfer_ChannelAuditEntry *ca) {
    if (!ca)
        return;

    g_free(ca->channelName);
    g_free(ca->contact);
    g_free(ca->tcpBufferSize);
    g_free(ca->message);
    g_free(ca->adminDN);
    g_free(ca->ftsNode);
    g_free(ca->clientIP);
}

glite_transfer_ChannelAuditEntry *glite_transfer_ChannelAuditEntry_clone(glite_transfer_ctx *ctx,
        const glite_transfer_ChannelAuditEntry *orig) {
    glite_transfer_ChannelAuditEntry *clone;

    if (!orig)
        return NULL;

    clone = glite_transfer_ChannelAuditEntry_new(ctx, orig->channelName);
    if (!clone)
        return NULL;

    clone->bandwidth = orig->bandwidth;
    clone->nominalThroughput = orig->nominalThroughput;
    clone->nofiles = orig->nofiles;
    clone->nostreams = orig->nostreams;
    clone->channelState = orig->channelState;
    clone->modificationTime = orig->modificationTime;
    clone->auditID = orig->auditID;

    clone->contact = g_strdup(orig->contact);
    clone->tcpBufferSize = g_strdup(orig->tcpBufferSize);
    clone->message = g_strdup(orig->message);
    clone->adminDN = g_strdup(orig->adminDN);
    clone->ftsNode = g_strdup(orig->ftsNode);
    clone->clientIP = g_strdup(orig->clientIP);

    return clone;
}

glite_transfer_Roles *glite_transfer_Roles_new(glite_transfer_ctx *ctx G_GNUC_UNUSED) {
    glite_transfer_Roles *roles;
    roles = g_new0(glite_transfer_Roles, 1);
    return roles;
}
