/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Data Management - Simple channel management API
 *
 *  Authors:
 *      Zsolt Molnar <Zsolt.Molnar@cern.ch>
 */

#ifndef FTS_EXCEPTION_H
#define FTS_EXCEPTION_H

#include <axutil_env.h>
#include <glite/data/transfer/c/transfer-simple.h>

#include "fts_exception_id.h"

/**
 * Exception representing a SOAP fault, including the whole message object.
 * The message object uses the internal representation in the AXIS generated
 * code.
 */
typedef struct fts_exception fts_exception_t;

typedef void (*_fts_exception_freefv_t)(fts_exception_t*);
typedef const char* (*_fts_exception_get_messagefv_t)(const fts_exception_t*);
typedef const char* (*_fts_exception_get_namefv_t)(void);
typedef glite_transfer_errclass (*_fts_exception_get_glite_transfer_errclassfv)(void);

/**
 * Create a fts_exception object.
 * @param env The actual Axis environment
 * @param id The exception ID
 * @param obj The exception object
 * @param freefv The function that is able to free obj
 * @param messagefv The function that is able to fetch the message from the object.
 * @param namefv The function that is able to fetch the exception name from the object.
 * @param errclassfv Gives the glite_transfer_errclass id of the exception
 * @return The object or NULL if construction failed
 */
fts_exception_t* fts_exception_create(const axutil_env_t *env, const fts_exception_id_t id, void* obj,
		_fts_exception_freefv_t freefv, _fts_exception_get_messagefv_t messagefv,
		_fts_exception_get_namefv_t namefv, _fts_exception_get_glite_transfer_errclassfv errclassfv);

/**
 * Free an fts_exception object.
 * @param self The object to be freed
 */
void fts_exception_free(fts_exception_t* self);

/**
 * Return the ID of the exception.
 */
fts_exception_id_t fts_exception_get_id(const fts_exception_t* self);

/**
 * Return the exception object. NULL means SW crash.
 */
void* fts_exception_get_object(const fts_exception_t* self);

/**
 * Return the exception name (from the wsdf:failt). NULL in case of
 * invalid object.
 */
const char* fts_exception_get_name(const fts_exception_t* self);

/**
 * Return the message in the exception.
 */
const char* fts_exception_get_message(const fts_exception_t* self);

/**
 * Return the AXIS environment associated woth the object.
 */
const axutil_env_t* fts_exception_get_env(const fts_exception_t* self);

/**
 * Map the exception to a glite_transfer_errclass.
 *
 */
glite_transfer_errclass fts_exception_get_glite_transfer_errclass(const fts_exception_t* self);

#endif /* #define FTS_EXCEPTION_H */
