/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Data Management - Simple channel management API
 *
 *  Authors:
 *      Zsolt Molnar <Zsolt.Molnar@cern.ch>
 */

#include <stdio.h>
#include <assert.h>

#include "../fts_soap_exception_api.h"
#include "fts_soap_exception_api_impl.h"
#include "fts_xml_repr.h"

axiom_soap_fault_t* _fts_get_soap_fault(const axutil_env_t *env, axis2_svc_client_t *svc_client)
{
  	axiom_soap_envelope_t *soap_envelope = NULL;
	axiom_soap_body_t *soap_body = NULL;
	axiom_soap_fault_t *soap_fault = NULL;
	/* Parameter validity check */
	assert(env);
	assert(svc_client);
	/* Implementation */
	int http_status_code = axis2_svc_client_get_http_status_code(svc_client, env);
	printf("\n HTTP code: %d\n\n", http_status_code);

	soap_envelope = axis2_svc_client_get_last_response_soap_envelope(svc_client, env);
	assert(soap_envelope);

	if (soap_envelope)
		soap_body = axiom_soap_envelope_get_body(soap_envelope, env);

	assert(soap_body);

	if (soap_body)
		soap_fault = axiom_soap_body_get_fault(soap_body, env);

	return soap_fault;
}

fts_xml_repr_t* _fts_get_soap_header(const axutil_env_t *env, axis2_svc_client_t *svc_client)
{
	axiom_soap_fault_t *soap_fault = NULL;
	axiom_node_t *node = NULL;
	axiom_element_t *element = NULL;
	/* Parameter validity check */
	assert(env);
	assert(svc_client);
	/* Implementation */
	soap_fault = _fts_get_soap_fault(env, svc_client);
	assert(soap_fault);

	if (soap_fault) {
		node = axiom_soap_fault_get_base_node(soap_fault, env);
		assert(node);
		element = axiom_node_get_data_element(node, env);
		assert(element);
	}

	return fts_xml_repr_create(node, element);
}

fts_xml_repr_t* _fts_get_first_child_with_qname(const axutil_env_t *env, axutil_qname_t *qname, fts_xml_repr_t* xml)
{
	axiom_node_t *node = NULL;
	axiom_element_t *element = NULL;
	/* Parameter validity check */
	assert(env);
	assert(qname);
	assert(fts_xml_repr_is_valid(xml));
	/* Implementation */
	node = axiom_node_create(env);
	assert(node);

	if (node)
		element = axiom_element_get_first_child_with_qname(fts_xml_repr_element(xml),
				env, qname, fts_xml_repr_node(xml), &node);

	if (element)
		return fts_xml_repr_create(node, element);

	axiom_node_free_tree(node, env);
	return NULL;
}

fts_xml_repr_t* _fts_get_soap_fault_detail(const axutil_env_t *env, fts_xml_repr_t* header)
{
	fts_xml_repr_t *detail = NULL;
	axutil_qname_t *detail_qname = NULL;

	detail_qname = axutil_qname_create(env, "detail", NULL, "");
	assert(detail_qname);
	detail = _fts_get_first_child_with_qname(env, detail_qname, header);
	assert(detail);
	axutil_qname_free(detail_qname, env);
	detail_qname = NULL;
	return detail;
}

fts_xml_repr_t* _fts_get_exception_xml(const axis2_char_t *name, const axutil_env_t *env, fts_xml_repr_t* detail)
{
	fts_xml_repr_t *exception = NULL;
	axutil_qname_t *exception_qname = NULL;

	exception_qname = axutil_qname_create(env, name, FTS_WSDL_NAMESPACE, NULL);
	assert(exception_qname);
	exception = _fts_get_first_child_with_qname(env, exception_qname, detail);
	axutil_qname_free(exception_qname, env);
	exception_qname = NULL;
	return exception;
}

fts_exception_t* fts_get_last_soap_exception(const axutil_env_t* env, axis2_svc_client_t *svc_client)
{
	fts_exception_t *exception = NULL;

	assert(env);
	assert(svc_client);

	if (fts_is_fault_occured(env, svc_client)) {
		fts_xml_repr_t *header = _fts_get_soap_header(env, svc_client);
		assert(fts_xml_repr_is_valid(header));

		if (fts_xml_repr_is_valid(header)) {
			fts_xml_repr_t *detail = _fts_get_soap_fault_detail(env, header);
			assert(fts_xml_repr_is_valid(detail));

			if (fts_xml_repr_is_valid(detail))
				exception = _fts_build_exception(detail, env);

			fts_xml_repr_free(detail, env);
			detail = NULL;
		}

		fts_xml_repr_free(header, env);
		header = NULL;
	}

	return exception;
}


int fts_is_fault_occured(const axutil_env_t* env, axis2_svc_client_t *svc_client)
{
	assert(svc_client);
	assert(env);
	return svc_client && env && axis2_svc_client_get_last_response_has_fault(svc_client, env);
}

char* fts_get_soap_faultcode(const axutil_env_t* env, axis2_svc_client_t *svc_client)
{
	axiom_soap_fault_t *soap_fault = NULL;
	char* ret = NULL;

	assert(env);
	assert(svc_client);

	soap_fault = _fts_get_soap_fault(env, svc_client);
	assert(soap_fault);

	if (soap_fault) {
		axiom_soap_fault_code_t *code = NULL;
		axiom_soap_fault_value_t *code_value = NULL;
		char* text = NULL;

		code = axiom_soap_fault_get_code(soap_fault, env);
		assert(code);

		code_value = axiom_soap_fault_code_get_value(code, env);
		assert(code_value);

		text = axiom_soap_fault_value_get_text(code_value, env);
		assert(text);

		if (text)
			ret = strdup(text);

		axiom_soap_fault_value_free(code_value, env);
		code_value = NULL;
		axiom_soap_fault_code_free(code, env);
		code = NULL;
	}

	axiom_soap_fault_free(soap_fault, env);
	soap_fault = NULL;

	return ret;
}

char* fts_get_soap_faultstring(const axutil_env_t* env, axis2_svc_client_t *svc_client)
{
	axiom_soap_fault_t *soap_fault = NULL;
	char* ret = NULL;

	assert(env);
	assert(svc_client);

	soap_fault = _fts_get_soap_fault(env, svc_client);
	assert(soap_fault);

	if (soap_fault) {
		axiom_soap_fault_reason_t* reason = NULL;
		axiom_soap_fault_text_t* reason_text = NULL;
		char* text = NULL;

		reason = axiom_soap_fault_get_reason(soap_fault, env);
		assert(reason);

		reason_text = axiom_soap_fault_reason_get_first_soap_fault_text(reason, env);
		assert(reason_text);

		text = axiom_soap_fault_text_get_text(reason_text, env);

		if (text)
			ret = strdup(text);

		axiom_soap_fault_text_free(reason_text, env);
		reason_text = NULL;
		axiom_soap_fault_reason_free(reason, env);
		reason = NULL;
	}

	axiom_soap_fault_free(soap_fault, env);
	soap_fault = NULL;

	return ret;
}

char* fts_get_soap_detail(const axutil_env_t* env, axis2_svc_client_t *svc_client)
{
	char* ret = NULL;
	fts_xml_repr_t *header = NULL;

	assert(env);
	assert(svc_client);
	header = _fts_get_soap_header(env, svc_client);
	assert(fts_xml_repr_is_valid(header));

	if (fts_xml_repr_is_valid(header)) {
		fts_xml_repr_t *detail = _fts_get_soap_fault_detail(env, header);

		if (detail) {
			ret = axiom_node_to_string(fts_xml_repr_node(detail) ,env);
			fts_xml_repr_free(detail, env);
			detail = NULL;
		}
	}

	fts_xml_repr_free(header, env);
	header = NULL;
	return ret;
}
