/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Data Management - Simple channel management API
 *
 *  Authors:
 *      Zsolt Molnar <Zsolt.Molnar@cern.ch>
 */

#ifndef FTS_SOAP_EXCEPTION_API_IMPL_H
#define FTS_SOAP_EXCEPTION_API_IMPL_H

#include <stdio.h>
#include <assert.h>

#include <axiom_node.h>
#include <axiom_element.h>
#include <axutil_env.h>
#include <axis2_stub.h>

#include "../fts_exception.h"
#include "fts_xml_repr.h"

/**
 * Fetch the SOAP fault form the result of the last service call.
 *
 * @param env The actual AXIS environment
 * @param svc_client The service client
 * @return Tha SOAP fault structure. NULL if the SOAP message does not
 *         encode a fault or there is processing error.
 */
axiom_soap_fault_t* _fts_get_soap_fault(const axutil_env_t *env, axis2_svc_client_t *svc_client);

/**
 * Get the AXIOM representation of the SOAP header of the fault message.
 * @param env The actual AXIS environment
 * @param svc_client The service client
 * @return The AXIOM representation of the SOAP header. NULL if the SOAP message does not
 *         encode a fault or there is processing error.
 */
fts_xml_repr_t* _fts_get_soap_header(const axutil_env_t *env, axis2_svc_client_t *svc_client);

/**
 * Get the AXIOM representation of the detail field of the SOAP fault message.
 * @param env The actual AXIS environment
 * @param header The AXIOM tree representation of the SOAP header encoding a fault
 * @return The AXIOM representation of the detail element. NULL in case of processing error.
 */
fts_xml_repr_t* _fts_get_soap_fault_detail(const axutil_env_t *env, fts_xml_repr_t* header);

/**
 * Get the FTS representation of a child element in an XML tree. The child is identified
 * by its qname. The first child is returned.
 * @param env The actual AXIS environment
 * @param qname The qname of the child
 * @xml FTS representation of the XML parent tree
 * @return The FTS representation of the first found child.
 */
fts_xml_repr_t* _fts_get_first_child_with_qname(const axutil_env_t *env, axutil_qname_t *qname, fts_xml_repr_t* xml);

/**
 * Get the AXIOM representation of a given fault (exception) object. The function can
 * be used to query for the exception type: we try to build a given fault type.
 * If it is unsuccessful, then it means either processing error or the fault is not
 * of the queried type.
 * @name The fault (exception) name to be queried. Must be the name attribute of a <wsdl:fault >
 * @param env The actual AXIS environment
 * @param detail The AXIOM tree representation of the detail element of a SOAP header encoding a fault
 * @return The AXIOM representation of the fault. NULL in case of processing error or
 *         mismatching fault name.
 */
fts_xml_repr_t* _fts_get_exception_xml(const axis2_char_t *name, const axutil_env_t *env, fts_xml_repr_t* detail);

/**
 * Build the exception object, using the AXIS-generated deserializers.
 * @param detail The AXIOM tree representation of the detail element of a SOAP header encoding a fault
 * @param env The actual AXIS environment
 * @return The FTS representation of the fault: fault ID and deserialized object.
 *         NULL in case of processing error.
 */
fts_exception_t* _fts_build_exception(fts_xml_repr_t *detail, const axutil_env_t *env);

#endif /* #ifndef FTS_SOAP_EXCEPTION_API_IMPL_H */

