/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Data Management - Simple file fts API
 *
 *  Authors: Gabor Gombas <Gabor.Gombas@cern.ch>
 *  Version info: $Id: fts_simple_api.c,v 1.27 2009/12/09 11:59:18 molnarzs Exp $
 *
 */
#include <glite/data/transfer/c/fts-simple.h>

#include <glite/data/transfer/c/fts.nsmap>

#include "fts_internal.h"

#include "ServiceDiscoveryIfce.h"
#include <cgsi_plugin.h>

#include <glite/data/glite-util.h>
#include <uuid.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <errno.h>
#include <time.h>

/**********************************************************************
 * Prototypes
 */

static void fault_to_error(glite_transfer_ctx *ctx, const char *method);


/**********************************************************************
 * Global variables
 */

static const char * const patterns[] = {
    [GLITE_FTS_SURL_LFN]    = "lfn",
    [GLITE_FTS_SURL_DAY]    = "day",
    [GLITE_FTS_SURL_HOUR]   = "hour",
    [GLITE_FTS_SURL_FLAT]   = "flat"
};


/**********************************************************************
 * Local helper functions
 */

static void sd_params_free(glite_transfer_ctx *ctx) {
    sd_params *params = ctx->priv;

    if (!params)
        return;

    g_free(params->surl_prefix);
    g_free(params->srm_endpoint);
    g_free(params);
}

/* Get the transfer parameters from service discovery */
static void fill_params(glite_transfer_ctx *ctx, const char *service) {
    sd_params *params;
    SDException exc;
    char *value;

    params = g_new0(sd_params, 1);
    params->pattern = GLITE_FTS_SURL_DEFAULT;

    ctx->priv = params;
    ctx->cleanup = sd_params_free;

    value = SD_getServiceDataItem(service, GLITE_FTS_SD_PATTERN, &exc);
    if (value) {
        unsigned i;

        for (i = 0; i < G_N_ELEMENTS(patterns); i++) {
            if (patterns[i] && !strcasecmp(patterns[i], value)) {
                params->pattern = i;
                break;
            }
        }
    } else
        SD_freeException(&exc);
    g_free(value);

    value = SD_getServiceDataItem(service, GLITE_FTS_SD_SURL_PREFIX, &exc);
    params->surl_prefix = g_strdup(value);
    if (!value)
        SD_freeException(&exc);
    g_free(value);

    value = SD_getServiceDataItem(service, GLITE_FTS_SD_SRM_ENDPOINT, &exc);
    params->srm_endpoint = g_strdup(value);
    if (!value)
        SD_freeException(&exc);
    g_free(value);
}

/* Check if this is a http:// URL */
static int is_http(const char *url) {
    return url && !strncmp(url, HTTP_PREFIX, strlen(HTTP_PREFIX));
}

/* Check if this is a https:// URL */
static int is_https(const char *url) {
    return url && !strncmp(url, HTTPS_PREFIX, strlen(HTTPS_PREFIX));
}

/* Check if this is a httpg:// URL */
static int is_httpg(const char *url) {
    return url && !strncmp(url, HTTPG_PREFIX, strlen(HTTPG_PREFIX));
}

/* Check if the context strcuture is usable */
static int is_ctx_ok(glite_transfer_ctx *ctx) {
    int ret;

    /* A NULL context is not usable */
    if (!ctx)
        return FALSE;

    /* If the context is already initialized for the fts port,
     * do nothing */
    if (ctx->port_type == PORT_FTS)
        return TRUE;
    /* If the context is already initialized for some other port,
     * bail out */
    if (ctx->port_type != PORT_NONE)
        return FALSE;

    if (is_http(ctx->default_endpoint) ||
        is_https(ctx->default_endpoint) ||
        is_httpg(ctx->default_endpoint))
    {
        ctx->endpoint = g_strdup(ctx->default_endpoint);
    }
    else if (ctx->default_endpoint != NULL)
    {
        glite_transfer_set_error(ctx, GLITE_TRANSFER_ERROR_INVALIDARGUMENT,
                                 "Wrongly formatted endpoint: \"%s\"", ctx->default_endpoint);
        return FALSE;
    }
    else
    {
        SDService *service;
        char *name, *error;
        const char * fts_srvtype;
        const char * fts_version;
        SDException exc;

        if (ctx->source && ctx->destination) {
            SDServiceData datas[] =
                {
                    {"source", ctx->source},
                    {"destination", ctx->destination}
                };
            SDServiceDataList datalist = {NULL, 2, datas};
            name = glite_discover_service_by_data(GLITE_CA_SD_TYPE, &datalist, &error);
            if (name)
                ctx->default_endpoint = g_strdup(name);
        }

        if (getenv(GLITE_FTS_SD_ENV))
            fts_srvtype = getenv(GLITE_FTS_SD_ENV);
        else
            fts_srvtype = GLITE_FTS_SD_TYPE;

        if(getenv(GLITE_FTS_IFC_VERSION))
            fts_version = getenv(GLITE_FTS_IFC_VERSION);
        else
            fts_version = INTERFACE_VERSION;

        // Note: glite_discover_service will check if the service
        // name passed (ctx->default_endpoint) has an associated
        // service of the given type (fts_srvtype), if not
        // it will check if the name refers to a site name or an
        // host name
        name = glite_discover_service_by_version(fts_srvtype,
                ctx->default_endpoint, fts_version, &error);
        if (!name) {
            glite_transfer_set_error(ctx,
                                     GLITE_TRANSFER_ERROR_SERVICEDISCOVERY,
                                     "Service discovery: %s", error);
            free(error);
            return FALSE;
        }

        service = SD_getService(name, &exc);
        if (!service) {
            glite_transfer_set_error(ctx,
                                     GLITE_TRANSFER_ERROR_SERVICEDISCOVERY,
                                     "Service discovery lookup failed for %s: %s",
                                     name, exc.reason);
            SD_freeException(&exc);
            g_free(name);
            return FALSE;
        }

        ctx->endpoint = g_strdup(service->endpoint);
        fill_params(ctx, name);

        SD_freeService(service);
        g_free(name);
    }

    /* SOAP initialization */
    if (is_https(ctx->endpoint))
        ret = soap_cgsi_init(ctx->soap,
                             CGSI_OPT_DISABLE_NAME_CHECK | CGSI_OPT_SSL_COMPATIBLE);
    else if (is_httpg(ctx->endpoint))
        ret = soap_cgsi_init(ctx->soap, CGSI_OPT_DISABLE_NAME_CHECK);
    else
        ret = 0;
    if (ret) {
        glite_transfer_set_error(ctx, GLITE_TRANSFER_ERROR_SOAP,
                                 "Failed to initialize the GSI plugin");
        return FALSE;
    }

    if (soap_set_namespaces(ctx->soap, fts_namespaces)) {
        fault_to_error(ctx, "Setting SOAP namespaces");
        return FALSE;
    }

    ctx->port_type = PORT_FTS;
    return TRUE;
}

/* Decode an exception inside a SOAP fault */
static void decode_exception(glite_transfer_ctx *ctx,
                             struct SOAP_ENV__Detail *detail, const char *method) {
    char *message;
    if (!detail)
        return;
    #if _GSOAP_VERSION >= 0x020700
    #define SET_EXCEPTION(exc, err) \
    message = ((struct transfer__ ## exc *)detail->fault)->message; \
    if (!message) \
        message = #exc " received from the service"; \
    glite_transfer_set_error(ctx, GLITE_TRANSFER_EXCEPTION_ ## err, \
                             "%s: %s", method, message);
    #define SET_EXCEPTION2(exc, err) \
    message = (NULL != ((struct fts__ ## exc *)detail->fault)->fault)?((struct fts__ ## exc *)detail->fault)->fault->message:NULL; \
    if (!message) \
        message = #exc " received from the service"; \
    glite_transfer_set_error(ctx, GLITE_TRANSFER_EXCEPTION_ ## err, \
                             "%s: %s", method, message);
    #else
    #define SET_EXCEPTION(exc, err) \
    message = ((struct transfer__ ## exc *)detail->value)->message; \
    if (!message) \
        message = #exc " received from the service"; \
    glite_transfer_set_error(ctx, GLITE_TRANSFER_EXCEPTION_ ## err, \
                             "%s: %s", method, message);
    #define SET_EXCEPTION2(exc, err) \
    message = (NULL != ((struct fts__ ## exc *)detail->value)->fault)?((struct fts__ ## exc *)detail->value)->fault->message:NULL; \
    if (!message) \
        message = #exc " received from the service"; \
    glite_transfer_set_error(ctx, GLITE_TRANSFER_EXCEPTION_ ## err, \
                             "%s: %s", method, message);
    #endif

    switch (detail->__type) {
    case SOAP_TYPE_transfer__AuthorizationException:
        SET_EXCEPTION(AuthorizationException, AUTHORIZATION);
        break;
    case SOAP_TYPE_transfer__CannotCancelException:
        SET_EXCEPTION(CannotCancelException, CANNOTCANCEL);
        break;
    case SOAP_TYPE_transfer__InternalException:
        SET_EXCEPTION(InternalException, INTERNAL);
        break;
    case SOAP_TYPE_transfer__InvalidArgumentException:
        SET_EXCEPTION(InvalidArgumentException, INVALIDARGUMENT);
        break;
    case SOAP_TYPE_transfer__NotExistsException:
        SET_EXCEPTION(NotExistsException, NOTEXISTS);
        break;
    case SOAP_TYPE_transfer__ExistsException:
        SET_EXCEPTION(ExistsException, EXISTS);
        break;
    case SOAP_TYPE_transfer__ServiceBusyException:
        SET_EXCEPTION(ServiceBusyException, SERVICEBUSY);
        break;
    case SOAP_TYPE_transfer__TransferException:
        SET_EXCEPTION(TransferException, TRANSFER);
        break;
        #if _GSOAP_VERSION < 0x020709

    case SOAP_TYPE_fts__AuthorizationException:
        SET_EXCEPTION2(AuthorizationException, AUTHORIZATION);
        break;
    case SOAP_TYPE_fts__CannotCancelException:
        SET_EXCEPTION2(CannotCancelException, CANNOTCANCEL);
        break;
    case SOAP_TYPE_fts__InternalException:
        SET_EXCEPTION2(InternalException, INTERNAL);
        break;
    case SOAP_TYPE_fts__InvalidArgumentException:
        SET_EXCEPTION2(InvalidArgumentException, INVALIDARGUMENT);
        break;
    case SOAP_TYPE_fts__NotExistsException:
        SET_EXCEPTION2(NotExistsException, NOTEXISTS);
        break;
    case SOAP_TYPE_fts__ExistsException:
        SET_EXCEPTION2(ExistsException, EXISTS);
        break;
    case SOAP_TYPE_fts__ServiceBusyException:
        SET_EXCEPTION2(ServiceBusyException, SERVICEBUSY);
        break;
        #endif

    default:
        /* Let the generic error decoding handle this */
        break;
    }
    #undef SET_EXCEPTION
    #undef SET_EXCEPTION2
}

/* Convert the SOAP fault to an error message */
static void fault_to_error(glite_transfer_ctx *ctx, const char *method) {
    const char **code, **string, **detail;

    /* Reset the error class */
    ctx->errclass = GLITE_TRANSFER_ERROR_NONE;

    /* Convert the error message to a fault */
    soap_set_fault(ctx->soap);

    /* Try to decode the exception that caused the fault */
    if (ctx->soap->fault) {
        /* Look for a SOAP 1.1 fault */
        if (ctx->soap->fault->detail) {
            decode_exception(ctx,
                             ctx->soap->fault->detail, method);
        }
        /* Look for a SOAP 1.2 fault */
        if (ctx->soap->fault->SOAP_ENV__Detail) {
            decode_exception(ctx,
                             ctx->soap->fault->SOAP_ENV__Detail,
                             method);
        }
    }

    /* If we did not manage to decode the exception, try generic error
     * decoding */
    if (ctx->errclass == GLITE_TRANSFER_ERROR_NONE) {
        code = soap_faultcode(ctx->soap);
        string = soap_faultstring(ctx->soap);
        detail = soap_faultdetail(ctx->soap);

        /* If the SOAP 1.1 detail is empty, try the SOAP 1.2 detail */
        if (!detail && ctx->soap->fault &&
                ctx->soap->fault->SOAP_ENV__Detail)
            detail = (const char **)&ctx->soap->fault->SOAP_ENV__Detail->__any;

        /* Provide default messages */
        if (!code || !*code) {
            code = alloca(sizeof(*code));
            *code = "(SOAP fault code missing)";
        }
        if (!string || !*string) {
            string = alloca(sizeof(*string));
            *string = "(SOAP fault string missing)";
        }

        if (detail && *detail)
            glite_transfer_set_error(ctx, GLITE_TRANSFER_ERROR_SOAP,
                                     "%s: SOAP fault: %s - %s (%s)", method, *code,
                                     *string, *detail);
        else
            glite_transfer_set_error(ctx, GLITE_TRANSFER_ERROR_SOAP,
                                     "%s: SOAP fault: %s - %s", method, *code,
                                     *string);
    }

    soap_end(ctx->soap);
}

/**********************************************************************
 * Library interface functions
 */

const char *glite_fts_get_endpoint(glite_transfer_ctx *ctx) {
    if (!is_ctx_ok(ctx))
        return NULL;
    return ctx->endpoint;
}

/**********************************************************************
 * Service requests
 *
 * Interface: org.glite.data.transfer.ServiceBase
 */

char *glite_fts_getVersion(glite_transfer_ctx *ctx) {
    struct fts__getVersionResponse resp;
    char *version;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;

    /* Make the SOAP call */
    ret = soap_call_fts__getVersion(ctx->soap, ctx->endpoint, NULL,
                                    &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getVersion");
        return NULL;
    }

    if (resp.getVersionReturn)
        version = strdup(resp.getVersionReturn);
    else
        version = NULL;
    soap_end(ctx->soap);
    return version;
}

char *glite_fts_getSchemaVersion(glite_transfer_ctx *ctx) {
    struct fts__getSchemaVersionResponse resp;
    char *version;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;

    /* Make the SOAP call */
    ret = soap_call_fts__getSchemaVersion(ctx->soap, ctx->endpoint,
                                          NULL, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getSchemaVersion");
        return NULL;
    }

    if (resp.getSchemaVersionReturn)
        version = strdup(resp.getSchemaVersionReturn);
    else
        version = NULL;
    soap_end(ctx->soap);
    return version;
}

char *glite_fts_getInterfaceVersion(glite_transfer_ctx *ctx) {
    struct fts__getInterfaceVersionResponse resp;
    char *version;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;

    /* Make the SOAP call */
    ret = soap_call_fts__getInterfaceVersion(ctx->soap,
            ctx->endpoint, NULL, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getInterfaceVersion");
        return NULL;
    }

    if (resp.getInterfaceVersionReturn)
        version = strdup(resp.getInterfaceVersionReturn);
    else
        version = NULL;
    soap_end(ctx->soap);
    return version;
}

char *glite_fts_getServiceMetadata(glite_transfer_ctx *ctx,
                                   const char *key) {
    struct fts__getServiceMetadataResponse resp;
    char *req_key, *result;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;
    if (!key) {
        err_invarg(ctx, "getServiceMetadata: Key is missing");
        return NULL;
    }

    /* We cannot turn a const pointer to a non-const, so we have to copy. */
    req_key = soap_strdup(ctx->soap, key);
    if (!req_key) {
        err_outofmemory(ctx);
        return NULL;
    }

    /* Make the SOAP call */
    ret = soap_call_fts__getServiceMetadata(ctx->soap, ctx->endpoint,
                                            NULL, req_key, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getServiceMetadata");
        return NULL;
    }

    if (resp._getServiceMetadataReturn)
        result = strdup(resp._getServiceMetadataReturn);
    else
        result = NULL;
    soap_end(ctx->soap);
    return result;
}

/**********************************************************************
 * Interface: org.glite.data.transfer.fts.FilePlacement
 */

/* This function is here for backward compatibility */
char *glite_fts_submit(glite_transfer_ctx *ctx,
                       const glite_transfer_TransferJob *job) {
    struct fts__submitResponse resp;
    struct transfer__TransferJob *req;
    char *requestId;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;
    if (!job) {
        err_invarg(ctx, "submit: Job descriptor is missing");
        return NULL;
    }

    req = to_soap_TransferJob(ctx, ctx->soap, job);
    if (!req) {
        err_outofmemory(ctx);
        soap_end(ctx->soap);
        return NULL;
    }

    ret = soap_call_fts__submit(ctx->soap, ctx->endpoint, NULL,
                                req, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "submit");
        return NULL;
    }

    if (resp._submitReturn) {
        requestId = strdup(resp._submitReturn);
        if (!requestId)
            err_outofmemory(ctx);
    } else {
        err_soap(ctx, "submit returned empty request ID");
        requestId = NULL;
    }

    soap_end(ctx->soap);
    return requestId;
}

/**
 * glite_fts_transferSubmit
 */
char *glite_fts_transferSubmit(
    glite_transfer_ctx *ctx,
    const glite_transfer_TransferJob *job,
    int withDelegation, int withChecksums) {

    /* without checksum */
    struct fts__transferSubmitResponse resp;
    struct transfer__TransferJob *req;
    /* with checksum */
    struct fts__transferSubmit3Response resp_cksm;
    struct transfer__TransferJob2 *req_cksm;
    /* common */
    char *requestId;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;
    if (!job) {
        err_invarg(ctx, "transferSubmit: Job descriptor is missing");
        return NULL;
    }

    if (withChecksums) {
        req_cksm = to_soap_TransferJob2(ctx, ctx->soap, job);
    } else {
        req = to_soap_TransferJob(ctx, ctx->soap, job);
    }
    if (!req && !req_cksm) {
        err_outofmemory(ctx);
        soap_end(ctx->soap);
        return NULL;
    }

    if (withDelegation) {
        if (withChecksums) {
            ret = soap_call_fts__transferSubmit3(ctx->soap, ctx->endpoint, NULL, req_cksm, (struct fts__transferSubmit3Response *)&resp_cksm);
        } else {
            ret = soap_call_fts__transferSubmit2(ctx->soap, ctx->endpoint, NULL, req, (struct fts__transferSubmit2Response *)&resp);
        }
    } else {
        ret = soap_call_fts__transferSubmit(ctx->soap, ctx->endpoint, NULL, req, &resp);
    }

    if (ret != SOAP_OK) {
        fault_to_error(ctx, "transferSubmit");
        return NULL;
    }

    if (resp._transferSubmitReturn || resp_cksm._transferSubmit3Return) {
        if (withChecksums) {
            requestId = strdup(resp_cksm._transferSubmit3Return);
        } else {
            requestId = strdup(resp._transferSubmitReturn);
        }
        if (!requestId)
            err_outofmemory(ctx);
    } else {
        err_soap(ctx, "transferSubmit returned empty request ID");
        requestId = NULL;
    }

    soap_end(ctx->soap);
    return requestId;
}


char *glite_fts_placementSubmit(glite_transfer_ctx *ctx,
                                const glite_transfer_PlacementJob *job, int withDelegation) {
    struct fts__placementSubmitResponse resp;
    struct transfer__PlacementJob *req;
    char *requestId;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;
    if (!job) {
        err_invarg(ctx, "placementSubmit: Job descrtiptor is missing");
        return NULL;
    }

    req = to_soap_PlacementJob(ctx->soap, job);
    if (!req) {
        err_outofmemory(ctx);
        soap_end(ctx->soap);
        return NULL;
    }

    if (withDelegation) {
        ret = soap_call_fts__placementSubmit2(ctx->soap, ctx->endpoint, NULL, req, (struct fts__placementSubmit2Response *)&resp);
    } else {
        ret = soap_call_fts__placementSubmit(ctx->soap, ctx->endpoint, NULL, req, &resp);
    }

    if (ret != SOAP_OK) {
        fault_to_error(ctx, "placementSubmit");
        return NULL;
    }

    if (resp._placementSubmitReturn) {
        requestId = strdup(resp._placementSubmitReturn);
        if (!requestId)
            err_outofmemory(ctx);
    } else {
        err_soap(ctx, "placementSubmit returned empty request ID");
        requestId = NULL;
    }

    soap_end(ctx->soap);
    return requestId;
}

glite_transfer_JobStatus **glite_fts_listRequests(glite_transfer_ctx *ctx,
        int numStates, const char * const states[], const char *channelName,
        int *resultCount) {
    struct ArrayOf_USCOREsoapenc_USCOREstring sstates;
    struct fts__listRequestsResponse resp;
    struct transfer__JobStatus **tmp;
    glite_transfer_JobStatus **result;
    char *schname;
    int i, ret;

    /* Default: error */
    if (resultCount)
        *resultCount = -1;

    if (!is_ctx_ok(ctx))
        return NULL;

    ret = to_soap_StringArray(ctx->soap, &sstates, numStates,
                              states);
    if (ret) {
        err_outofmemory(ctx);
        return NULL;
    }

    schname = soap_strdup(ctx->soap, channelName);
    if (channelName && !schname) {
        err_outofmemory(ctx);
        soap_end(ctx->soap);
        return NULL;
    }

    ret = soap_call_fts__listRequests(ctx->soap, ctx->endpoint, NULL,
                                      &sstates, schname, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "listRequests");
        return NULL;
    }

    /* The response may be empty */
    if (!resp._listRequestsReturn) {
        if (resultCount)
            *resultCount = 0;
        soap_end(ctx->soap);
        return NULL;
    }

    result = malloc(resp._listRequestsReturn->__size * sizeof(*result));
    if (!result) {
        err_outofmemory(ctx);
        soap_end(ctx->soap);
        return NULL;
    }

    tmp = resp._listRequestsReturn->__ptr;
    for (i = 0; i < resp._listRequestsReturn->__size; i++) {
        result[i] = from_soap_JobStatus(ctx, tmp[i]);
        if (!result[i]) {
            glite_transfer_JobStatus_freeArray(ctx, i, result);
            soap_end(ctx->soap);
            return NULL;
        }
    }

    if (resultCount)
        *resultCount = resp._listRequestsReturn->__size;
    soap_end(ctx->soap);
    return result;
}

glite_transfer_JobStatus **glite_fts_listRequests2(glite_transfer_ctx *ctx,
        int numStates, const char * const states[], const char *channelName,
        const char* restrict_dn, const char* restrict_vo, int *resultCount) {
    struct ArrayOf_USCOREsoapenc_USCOREstring sstates;
    struct fts__listRequestsResponse resp;
    struct transfer__JobStatus **tmp;
    glite_transfer_JobStatus **result;
    char *schname, *srestrict_dn, *srestrict_vo;
    int i, ret;

    /* Default: error */
    if (resultCount)
        *resultCount = -1;

    if (!is_ctx_ok(ctx))
        return NULL;

    ret = to_soap_StringArray(ctx->soap, &sstates, numStates,
                              states);
    if (ret) {
        err_outofmemory(ctx);
        return NULL;
    }

    schname = soap_strdup(ctx->soap, channelName);
    if (channelName && !schname) {
        err_outofmemory(ctx);
        soap_end(ctx->soap);
        return NULL;
    }

    srestrict_dn = soap_strdup(ctx->soap, restrict_dn);
    if (restrict_dn && !srestrict_dn) {
        err_outofmemory(ctx);
        soap_end(ctx->soap);
        return NULL;
    }

    srestrict_vo = soap_strdup(ctx->soap, restrict_vo);
    if (restrict_vo && !srestrict_vo) {
        err_outofmemory(ctx);
        soap_end(ctx->soap);
        return NULL;
    }

    ret = soap_call_fts__listRequests2(
        ctx->soap,
        ctx->endpoint,
        NULL,
        &sstates,
        (schname==NULL) ? "" : schname,
        (srestrict_dn==NULL) ? "" : srestrict_dn,
        (srestrict_vo==NULL) ? "": srestrict_vo ,
        (struct fts__listRequests2Response *)&resp);

    if (ret != SOAP_OK) {
        fault_to_error(ctx, "listRequests2");
        return NULL;
    }

    /* The response may be empty */
    if (!resp._listRequestsReturn) {
        if (resultCount)
            *resultCount = 0;
        soap_end(ctx->soap);
        return NULL;
    }

    result = malloc(resp._listRequestsReturn->__size * sizeof(*result));
    if (!result) {
        err_outofmemory(ctx);
        soap_end(ctx->soap);
        return NULL;
    }

    tmp = resp._listRequestsReturn->__ptr;
    for (i = 0; i < resp._listRequestsReturn->__size; i++) {
        result[i] = from_soap_JobStatus(ctx, tmp[i]);
        if (!result[i]) {
            glite_transfer_JobStatus_freeArray(ctx, i, result);
            soap_end(ctx->soap);
            return NULL;
        }
    }

    if (resultCount)
        *resultCount = resp._listRequestsReturn->__size;
    soap_end(ctx->soap);
    return result;
}

glite_transfer_FileTransferStatus **glite_fts_getFileStatus(glite_transfer_ctx *ctx,
        const char *requestId, int offset, int limit, int *resultCount) {
    struct fts__getFileStatusResponse resp;
    struct transfer__FileTransferStatus **tmp;
    glite_transfer_FileTransferStatus **result;
    char *sreqid;
    int i, ret;

    /* Default: error */
    if (resultCount)
        *resultCount = -1;

    if (!is_ctx_ok(ctx))
        return NULL;
    if (!requestId) {
        err_invarg(ctx, "getFileStatus: Request ID is missing");
        return NULL;
    }

    sreqid = soap_strdup(ctx->soap, requestId);
    if (!sreqid) {
        err_outofmemory(ctx);
        return NULL;
    }

    ret = soap_call_fts__getFileStatus(ctx->soap, ctx->endpoint,
                                       NULL, sreqid, offset, limit, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getFileStatus");
        return NULL;
    }

    /* The response may be empty */
    if (!resp._getFileStatusReturn) {
        if (resultCount)
            *resultCount = 0;
        soap_end(ctx->soap);
        return NULL;
    }

    result = malloc(resp._getFileStatusReturn->__size * sizeof(*result));
    if (!result) {
        err_outofmemory(ctx);
        soap_end(ctx->soap);
        return NULL;
    }

    tmp = resp._getFileStatusReturn->__ptr;
    for (i = 0; i < resp._getFileStatusReturn->__size; i++) {
        result[i] = from_soap_FileTransferStatus(ctx, tmp[i]);
        if (!result[i]) {
            glite_transfer_FileTransferStatus_freeArray(ctx, i,
                    result);
            soap_end(ctx->soap);
            return NULL;
        }
    }

    if (resultCount)
        *resultCount = resp._getFileStatusReturn->__size;
    soap_end(ctx->soap);
    return result;
}

glite_transfer_JobStatus *glite_fts_getTransferJobStatus(glite_transfer_ctx *ctx,
        const char *requestId) {
    struct fts__getTransferJobStatusResponse resp;
    glite_transfer_JobStatus *result;
    char *sreqid;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;
    if (!requestId) {
        err_invarg(ctx, "getTransferJobStatus: Request ID is missing");
        return NULL;
    }

    sreqid = soap_strdup(ctx->soap, requestId);
    if (!sreqid) {
        err_outofmemory(ctx);
        return NULL;
    }

    /* Make the SOAP call */
    ret = soap_call_fts__getTransferJobStatus(ctx->soap,
            ctx->endpoint, NULL, sreqid, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getTransferJobStatus");
        return NULL;
    }

    if (resp._getTransferJobStatusReturn)
        result = from_soap_JobStatus(ctx,
                                     resp._getTransferJobStatusReturn);
    else {
        err_soap(ctx, "getTransferJobStatus: Service returned empty "
                 "status");
        result = NULL;
    }
    soap_end(ctx->soap);
    return result;
}

glite_transfer_TransferJobSummary *glite_fts_getTransferJobSummary(glite_transfer_ctx *ctx,
        const char *requestId) {
    struct fts__getTransferJobSummaryResponse resp;
    glite_transfer_TransferJobSummary *result;
    char *sreqid;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;
    if (!requestId) {
        err_invarg(ctx, "getTransferJobSummary: Request ID is missing");
        return NULL;
    }

    sreqid = soap_strdup(ctx->soap, requestId);
    if (!sreqid) {
        err_outofmemory(ctx);
        return NULL;
    }

    /* Make the SOAP call */
    ret = soap_call_fts__getTransferJobSummary(ctx->soap,
            ctx->endpoint, NULL, sreqid, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getTransferJobSummary");
        return NULL;
    }

    result = from_soap_TransferJobSummary(ctx,
                                          resp._getTransferJobSummaryReturn);

    /* RC1 compat */
    if (result && !result->jobStatus) {
        result->jobStatus = glite_transfer_JobStatus_new(ctx, requestId,
                            GLITE_TRANSFER_UNKNOWN);
        if (result->numFailed)
            result->jobStatus->jobStatus =
                GLITE_TRANSFER_TRANSFERFAILED;
        else if (result->numFinished)
            result->jobStatus->jobStatus = GLITE_TRANSFER_FINISHED;
        else if (result->numCanceled)
            result->jobStatus->jobStatus = GLITE_TRANSFER_CANCELED;
        else if (result->numCanceling)
            result->jobStatus->jobStatus = GLITE_TRANSFER_CANCELING;
        else if (result->numHold)
            result->jobStatus->jobStatus = GLITE_TRANSFER_HOLD;
        else if (result->numWaiting)
            result->jobStatus->jobStatus = GLITE_TRANSFER_WAITING;
        else if (result->numCatalogFailed)
            result->jobStatus->jobStatus = GLITE_TRANSFER_CATALOGFAILED;
        else if (result->numSubmitted)
            result->jobStatus->jobStatus = GLITE_TRANSFER_SUBMITTED;
        else if (result->numDone)
            result->jobStatus->jobStatus = GLITE_TRANSFER_DONE;
        else if (result->numActive)
            result->jobStatus->jobStatus = GLITE_TRANSFER_ACTIVE;
        else if (result->numPending)
            result->jobStatus->jobStatus = GLITE_TRANSFER_PENDING;
    }

    soap_end(ctx->soap);
    return result;
}

glite_transfer_TransferJobSummary2 *glite_fts_getTransferJobSummary2(glite_transfer_ctx *ctx,
        const char *requestId) {
    struct fts__getTransferJobSummary2Response resp;
    glite_transfer_TransferJobSummary2 *result;
    char *sreqid;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;
    if (!requestId) {
        err_invarg(ctx, "getTransferJobSummary2: Request ID is missing");
        return NULL;
    }

    sreqid = soap_strdup(ctx->soap, requestId);
    if (!sreqid) {
        err_outofmemory(ctx);
        return NULL;
    }

    /* Make the SOAP call */
    ret = soap_call_fts__getTransferJobSummary2(
        ctx->soap,
        ctx->endpoint,
        NULL,
        sreqid,
        &resp);

    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getTransferJobSummary2");
        return NULL;
    }

    result = from_soap_TransferJobSummary2(ctx,
                                           resp._getTransferJobSummary2Return);

    /* RC1 compat */
    if (result && !result->jobStatus) {
        result->jobStatus = glite_transfer_JobStatus_new(ctx, requestId,
                            GLITE_TRANSFER_UNKNOWN);
        if (result->numFailed)
            result->jobStatus->jobStatus =
                GLITE_TRANSFER_TRANSFERFAILED;
        else if (result->numFinished)
            result->jobStatus->jobStatus = GLITE_TRANSFER_FINISHED;
        else if (result->numCanceled)
            result->jobStatus->jobStatus = GLITE_TRANSFER_CANCELED;
        else if (result->numCanceling)
            result->jobStatus->jobStatus = GLITE_TRANSFER_CANCELING;
        else if (result->numHold)
            result->jobStatus->jobStatus = GLITE_TRANSFER_HOLD;
        else if (result->numWaiting)
            result->jobStatus->jobStatus = GLITE_TRANSFER_WAITING;
        else if (result->numCatalogFailed)
            result->jobStatus->jobStatus = GLITE_TRANSFER_CATALOGFAILED;
        else if (result->numSubmitted)
            result->jobStatus->jobStatus = GLITE_TRANSFER_SUBMITTED;
        else if (result->numDone)
            result->jobStatus->jobStatus = GLITE_TRANSFER_DONE;
        else if (result->numActive)
            result->jobStatus->jobStatus = GLITE_TRANSFER_ACTIVE;
        else if (result->numPending)
            result->jobStatus->jobStatus = GLITE_TRANSFER_PENDING;
    }

    soap_end(ctx->soap);
    return result;
}

int glite_fts_cancel(glite_transfer_ctx *ctx, int nitems, const char **requestIds) {
    struct ArrayOf_USCOREsoapenc_USCOREstring req;
    int ret;
    struct fts__cancelResponse response;

    if (!is_ctx_ok(ctx))
        return -1;
    if (!requestIds) {
        err_invarg(ctx, "cancel: Request ID array is missing");
        return -1;
    }

    to_soap_StringArray(ctx->soap, &req, nitems, requestIds);

    /* Make the SOAP call */
    ret = soap_call_fts__cancel(ctx->soap, ctx->endpoint, NULL,
                                &req, &response);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "cancel");
        return -1;
    }

    soap_end(ctx->soap);
    return 0;
}

int glite_fts_addVOManager(glite_transfer_ctx *ctx, const char *VOname,
                           const char *principal) {
    char *sVOname, *sprincipal;
    int ret;

    if (!is_ctx_ok(ctx))
        return -1;
    if (!VOname) {
        err_invarg(ctx, "addVOManager: VOname is missing");
        return -1;
    }
    if (!principal) {
        err_invarg(ctx, "addVOManager: principal is missing");
        return -1;
    }

    sVOname = soap_strdup(ctx->soap, VOname);
    sprincipal = soap_strdup(ctx->soap, principal);
    if (!sVOname || !sprincipal) {
        err_outofmemory(ctx);
        return -1;
    }

    /* Make the SOAP call */
    struct fts__addVOManagerResponse _param_16;
    ret = soap_call_fts__addVOManager(ctx->soap, ctx->endpoint, NULL,
                                      sVOname, sprincipal, &_param_16);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "addVOManager");
        return -1;
    }

    soap_end(ctx->soap);
    return 0;
}

int glite_fts_removeVOManager(glite_transfer_ctx *ctx, const char *VOname,
                              const char *principal) {
    char *sVOname, *sprincipal;
    struct fts__removeVOManagerResponse response;
    int ret;

    if (!is_ctx_ok(ctx))
        return -1;
    if (!VOname) {
        err_invarg(ctx, "removeVOManager: VOname is missing");
        return -1;
    }
    if (!principal) {
        err_invarg(ctx, "removeVOManager: principal is missing");
        return -1;
    }

    sVOname = soap_strdup(ctx->soap, VOname);
    sprincipal = soap_strdup(ctx->soap, principal);
    if (!sVOname || !sprincipal) {
        err_outofmemory(ctx);
        return -1;
    }

    /* Make the SOAP call */
    ret = soap_call_fts__removeVOManager(ctx->soap, ctx->endpoint, NULL,
                                         sVOname, sprincipal, &response);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "removeVOManager");
        return -1;
    }

    soap_end(ctx->soap);
    return 0;
}

char **glite_fts_listVOManagers(glite_transfer_ctx *ctx, const char *VOname,
                                int *resultcount) {
    struct fts__listVOManagersResponse resp;
    char **result, *sVOname;
    int ret;

    /* Default: error */
    if (resultcount)
        *resultcount = -1;

    if (!is_ctx_ok(ctx))
        return NULL;
    if (!VOname) {
        err_invarg(ctx, "listVOManagers: VOname is missing");
        return NULL;
    }

    sVOname = soap_strdup(ctx->soap, VOname);
    if (!sVOname) {
        err_outofmemory(ctx);
        return NULL;
    }

    /* Make the SOAP call */
    ret = soap_call_fts__listVOManagers(ctx->soap, ctx->endpoint, NULL,
                                        sVOname, &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "listVOManagers");
        return NULL;
    }

    result = from_soap_StringArray(ctx, resp._listVOManagersReturn, resultcount);
    soap_end(ctx->soap);
    return result;
}

glite_transfer_Roles *glite_fts_getRoles(glite_transfer_ctx *ctx) {
    struct fts__getRolesResponse resp;
    glite_transfer_Roles *result;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;

    ret = soap_call_fts__getRoles(ctx->soap, ctx->endpoint, NULL,
                                  &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getRoles");
        return NULL;
    }

    result = from_soap_Roles(ctx, resp.getRolesReturn);

    soap_end(ctx->soap);
    return result;
}

glite_transfer_Roles *glite_fts_getRolesOf(glite_transfer_ctx *ctx, const char *otherDN) {
    struct fts__getRolesOfResponse resp;
    glite_transfer_Roles *result;
    char * sOtherDN;
    int ret;

    if (!is_ctx_ok(ctx))
        return NULL;

    sOtherDN = soap_strdup(ctx->soap, otherDN);
    if (!sOtherDN) {
        err_outofmemory(ctx);
        return NULL;
    }


    ret = soap_call_fts__getRolesOf(ctx->soap, ctx->endpoint, NULL, sOtherDN,
                                    &resp);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "getRolesOf");
        return NULL;
    }

    result = from_soap_Roles(ctx, resp._getRolesOfReturn);

    soap_end(ctx->soap);
    return result;
}


int glite_fts_setJobPriority(glite_transfer_ctx *ctx, const char *requestID,
                             const int priority) {
    char *srequestID;
    struct fts__setJobPriorityResponse response;
    int ret;

    if (!is_ctx_ok(ctx))
        return -1;
    if (!requestID) {
        err_invarg(ctx, "setPriority: requestID is missing");
        return -1;
    }

    srequestID = soap_strdup(ctx->soap, requestID);
    if (!srequestID) {
        err_outofmemory(ctx);
        return -1;
    }

    /* Make the SOAP call */
    ret = soap_call_fts__setJobPriority(ctx->soap, ctx->endpoint, NULL,
                                        srequestID, priority, &response);
    if (ret != SOAP_OK) {
        fault_to_error(ctx, "setPriority");
        return -1;
    }

    soap_end(ctx->soap);
    return 0;
}
