/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Data Management - Soap conversion functions
 *
 *  Authors: 
 *      Gabor Gombas <Gabor.Gombas@cern.ch>
 *      Paolo Tedesco <Paolo.Tedesco@cern.ch>
 *      Akos Frohner <Akos.Frohner@cern.ch>
 *
 */

#include <glite/data/transfer/c/fts-simple.h>

#include <glite/data/glite-util.h>

#include "fts_internal.h"

/**********************************************************************
 * Conversion to/from gSOAP data types
 */

/* Convert a list of strings to gSOAP's ArrayOf_USCOREsoapenc_USCOREstring */
int to_soap_StringArray(struct soap *soap,
    struct ArrayOf_USCOREsoapenc_USCOREstring *req,
    int nitems, const char * const items[])
{
    int i;

    req->__size = nitems;
    req->__ptr = soap_malloc(soap, nitems * sizeof(*req->__ptr));
    if (nitems && !req->__ptr)
        return -1;
    for (i = 0; i < nitems; i++)
    {
        req->__ptr[i] = soap_strdup(soap, items[i]);
        if (!req->__ptr[i])
            return -1;
    }

    return 0;
}

/* Convert gSOAP's ArrayOf_USCOREsoapenc_USCOREstring to char ** */
char **from_soap_StringArray(glite_transfer_ctx *ctx,
    struct ArrayOf_USCOREsoapenc_USCOREstring *resp, int *resultCount)
{
    char **result;
    int64_t i;

    /* The result may be empty */
    if (!resp)
    {
        if (resultCount)
            *resultCount = 0;
        return NULL;
    }

    result = malloc(resp->__size * sizeof(*result));
    if (resp->__size && !result)
    {
        err_outofmemory(ctx);
        return NULL;
    }

    for (i = 0; i < resp->__size; i++)
    {
        if (resp->__ptr[i])
        {
            result[i] = strdup(resp->__ptr[i]);
            if (!result[i])
            {
                err_outofmemory(ctx);
                glite_freeStringArray(i, result);
                return NULL;
            }
        }
        else
            result[i] = NULL;
    }

    if (resultCount)
        *resultCount = resp->__size;

    return result;
}

static void from_soap_time(struct timespec *ts, int64_t time)
{
    ts->tv_sec = time / 1000;
    ts->tv_nsec = (time - ts->tv_sec * 1000) * 1000000;
}


static struct transfer__TransferJobElement *to_soap_TransferJobElement(glite_transfer_ctx *ctx,
    struct soap *soap, const glite_transfer_TransferJobElement *elem)
{
    struct transfer__TransferJobElement *selem;

    if (!elem)
        return NULL;

    selem = soap_malloc(soap, sizeof(*selem));
    if (!selem)
        return NULL;

   memcpy ( selem, elem, sizeof(*selem) );

//    memset(selem, 0, sizeof(*selem));

//    STRING_TO_SOAP(elem, source);
//    if (elem->dest)
//        STRING_TO_SOAP(elem, dest);
  
  return selem;
}

static struct transfer__TransferJobElement2 *to_soap_TransferJobElement2(glite_transfer_ctx *ctx,
    struct soap *soap, const glite_transfer_TransferJobElement *elem)
{
    struct transfer__TransferJobElement2 *selem;

    if (!elem)
        return NULL;

    selem = soap_malloc(soap, sizeof(*selem));
    if (!selem)
        return NULL;

    memcpy ( selem, elem, sizeof(*selem) );

    /* this is optional */
    if (elem->checksum)
        STRING_TO_SOAP(elem, checksum);
    return selem;
}

static struct transfer__TransferParams *to_soap_TransferParams(struct soap *soap,
    const glite_transfer_TransferParams *params)
{
    struct transfer__TransferParams *sparams;
    unsigned i;

    if (!params)
        return NULL;

    sparams = soap_malloc(soap, sizeof(*sparams));
    if (!sparams)
        return NULL;
    memset(sparams, 0, sizeof(*sparams));

    sparams->keys = soap_malloc(soap, params->num * sizeof(*sparams->keys));
    sparams->values = soap_malloc(soap, params->num *
        sizeof(*sparams->values));
    if (params->num && (!sparams->keys || !sparams->values))
        return NULL;

    for (i = 0; i < params->num; i++)
    {
        sparams->keys[i] = soap_strdup(soap, params->keys[i]);
        sparams->values[i] = soap_strdup(soap, params->values[i]);
        if (!sparams->keys[i] || (params->values[i] && !sparams->values[i]))
            return NULL;
    }

    sparams->__sizekeys = params->num;
    sparams->__sizevalues = params->num;
    return sparams;
}

struct transfer__TransferJob *to_soap_TransferJob(glite_transfer_ctx *ctx,
    struct soap *soap, const glite_transfer_TransferJob *job)
{
    struct transfer__TransferJob *sjob;
    unsigned i;

    if (!job)
        return NULL;

    sjob = soap_malloc(soap, sizeof(*sjob));
    if (!sjob)
        return NULL;
    memset(sjob, 0, sizeof(*sjob));

    if (job->__sizetransferJobElements)
    {
        sjob->transferJobElements = soap_malloc(soap,
            job->__sizetransferJobElements *
            sizeof(*sjob->transferJobElements));
        if (!sjob->transferJobElements)
            return NULL;
        for (i = 0; i < job->__sizetransferJobElements; i++)
        {
            sjob->transferJobElements[i] = to_soap_TransferJobElement(ctx,
                soap, job->transferJobElements[i]);
            if (!sjob->transferJobElements[i])
                return NULL;
        }
        sjob->__sizetransferJobElements = job->__sizetransferJobElements;
    }
    sjob->jobParams = to_soap_TransferParams(soap, job->jobParams);
    if (job->jobParams && !sjob->jobParams)
        return NULL;
    sjob->credential = soap_strdup(soap, job->credential);
    if (job->credential && !sjob->credential)
        return NULL;
    return sjob;
}

struct transfer__TransferJob2 *to_soap_TransferJob2(glite_transfer_ctx *ctx,
    struct soap *soap, const glite_transfer_TransferJob *job)
{
    struct transfer__TransferJob2 *sjob;
    unsigned i;

    if (!job)
        return NULL;

    sjob = soap_malloc(soap, sizeof(*sjob));
    if (!sjob)
        return NULL;
    memset(sjob, 0, sizeof(*sjob));

    if (job->__sizetransferJobElements)
    {
        sjob->transferJobElements = soap_malloc(soap,
            job->__sizetransferJobElements *
            sizeof(*sjob->transferJobElements));
        if (!sjob->transferJobElements)
            return NULL;
        for (i = 0; i < job->__sizetransferJobElements; i++)
        {
            sjob->transferJobElements[i] = to_soap_TransferJobElement2(ctx,
                soap, job->transferJobElements[i]);
            if (!sjob->transferJobElements[i])
                return NULL;
        }
        sjob->__sizetransferJobElements = job->__sizetransferJobElements;
    }
    sjob->jobParams = to_soap_TransferParams(soap, job->jobParams);
    if (job->jobParams && !sjob->jobParams)
        return NULL;
    sjob->credential = soap_strdup(soap, job->credential);
    if (job->credential && !sjob->credential)
        return NULL;
    return sjob;
}


glite_transfer_FileTransferStatus *from_soap_FileTransferStatus(glite_transfer_ctx *ctx,
    const struct transfer__FileTransferStatus *sstatus)
{
    glite_transfer_FileTransferStatus *status;
    glite_transfer_state fileState;

    if (!sstatus)
        return NULL;

    fileState = glite_transfer_state_parse(ctx, sstatus->transferFileState);
    if (fileState == GLITE_TRANSFER_UNKNOWN)
        return NULL;

    status = glite_transfer_FileTransferStatus_new(ctx, sstatus->sourceSURL,
        sstatus->destSURL, fileState);
    if (!status)
        return NULL;

    STRING_FROM_SOAP(status, logicalName);
    STRING_FROM_SOAP(status, reason);
    NUM_FROM_SOAP(status, duration);
    status->reason_class = g_strdup(sstatus->reason_USCOREclass);

    NUM_FROM_SOAP(status, numFailures);
    return status;
}

glite_transfer_JobStatus *from_soap_JobStatus(glite_transfer_ctx *ctx,
    const struct transfer__JobStatus *sstatus)
{
    glite_transfer_JobStatus *status;
    glite_transfer_state jobStatus;

    if (!sstatus)
        return NULL;

    jobStatus = glite_transfer_state_parse(ctx, sstatus->jobStatus);
    if (jobStatus == GLITE_TRANSFER_UNKNOWN)
        return NULL;

    status = glite_transfer_JobStatus_new(ctx, sstatus->jobID, jobStatus);
    if (!status)
        return NULL;

    STRING_FROM_SOAP(status, channelName);
    STRING_FROM_SOAP(status, clientDN);
    STRING_FROM_SOAP(status, reason);
    STRING_FROM_SOAP(status, voName);
    NUM_FROM_SOAP(status, priority);
    from_soap_time(&status->submitTime, sstatus->submitTime);

    NUM_FROM_SOAP(status, numFiles);
    return status;
}

glite_transfer_TransferJobSummary *from_soap_TransferJobSummary(glite_transfer_ctx *ctx,
    const struct transfer__TransferJobSummary *ssummary)
{
    glite_transfer_TransferJobSummary *summary;

    if (!ssummary)
        return NULL;

    summary = glite_transfer_TransferJobSummary_new(ctx);
    if (!summary)
        return NULL;

    summary->jobStatus = from_soap_JobStatus(ctx, ssummary->jobStatus);
    if (ssummary->jobStatus && !summary->jobStatus)
    {
        glite_transfer_TransferJobSummary_free(ctx, summary);
        return NULL;
    }

    NUM_FROM_SOAP(summary, numDone);
    NUM_FROM_SOAP(summary, numActive);
    NUM_FROM_SOAP(summary, numPending);
    NUM_FROM_SOAP(summary, numCanceled);
    NUM_FROM_SOAP(summary, numCanceling);
    NUM_FROM_SOAP(summary, numFailed);
    NUM_FROM_SOAP(summary, numFinished);
    NUM_FROM_SOAP(summary, numSubmitted);
    NUM_FROM_SOAP(summary, numHold);
    NUM_FROM_SOAP(summary, numWaiting);
    NUM_FROM_SOAP(summary, numCatalogFailed);
    NUM_FROM_SOAP(summary, numRestarted);

    return summary;
}

glite_transfer_TransferJobSummary2 *from_soap_TransferJobSummary2(glite_transfer_ctx *ctx,
    const struct transfer__TransferJobSummary2 *ssummary)
{
    glite_transfer_TransferJobSummary2 *summary;

    if (!ssummary)
        return NULL;

    summary = glite_transfer_TransferJobSummary2_new(ctx);
    if (!summary)
        return NULL;

    summary->jobStatus = from_soap_JobStatus(ctx, ssummary->jobStatus);
    if (ssummary->jobStatus && !summary->jobStatus)
    {
        glite_transfer_TransferJobSummary2_free(ctx, summary);
        return NULL;
    }

    NUM_FROM_SOAP(summary, numDone);
    NUM_FROM_SOAP(summary, numActive);
    NUM_FROM_SOAP(summary, numPending);
    NUM_FROM_SOAP(summary, numCanceled);
    NUM_FROM_SOAP(summary, numCanceling);
    NUM_FROM_SOAP(summary, numFailed);
    NUM_FROM_SOAP(summary, numFinished);
    NUM_FROM_SOAP(summary, numSubmitted);
    NUM_FROM_SOAP(summary, numHold);
    NUM_FROM_SOAP(summary, numWaiting);
    NUM_FROM_SOAP(summary, numCatalogFailed);
    NUM_FROM_SOAP(summary, numRestarted);
    NUM_FROM_SOAP(summary, numRestarted);
    NUM_FROM_SOAP(summary, numReady);
    NUM_FROM_SOAP(summary, numFinishing);
    NUM_FROM_SOAP(summary, numAwaitingPrestage);
    NUM_FROM_SOAP(summary, numPrestaging);
    NUM_FROM_SOAP(summary, numWaitingCatalogRegistration);
    NUM_FROM_SOAP(summary, numWaitingCatalogResolution);
   NUM_FROM_SOAP(summary, numWaitingPrestage);

    return summary;
}


glite_transfer_Roles *from_soap_Roles(glite_transfer_ctx *ctx,
    const struct transfer__Roles *sroles)
{
    int i;
    glite_transfer_Roles *roles;

    if (!sroles)
        return NULL;

    roles = glite_transfer_Roles_new(ctx);
    if (!roles)
        return NULL;

    STRING_FROM_SOAP(roles, clientDN);
    STRING_FROM_SOAP(roles, serviceAdmin);
    STRING_FROM_SOAP(roles, submitter);
    NUM_FROM_SOAP(roles, __sizeVOManager);
    roles->VOManager = g_new0(glite_transfer_StringPair *, roles->__sizeVOManager);
    for (i = 0; i < roles->__sizeVOManager; i++)
    {
        roles->VOManager[i] = g_new0(glite_transfer_StringPair, 1);
#if _GSOAP_VERSION >= 0x030709
        roles->VOManager[i]->string1 = g_strdup(sroles->VOManager[i].string1);
        roles->VOManager[i]->string2 = g_strdup(sroles->VOManager[i].string2);
#else
        roles->VOManager[i]->string1 = g_strdup(sroles->VOManager[i]->string1);
        roles->VOManager[i]->string2 = g_strdup(sroles->VOManager[i]->string2);
#endif
    }
    NUM_FROM_SOAP(roles, __sizechannelManager);
    roles->channelManager = g_new0(glite_transfer_StringPair *, roles->__sizechannelManager);
    for (i = 0; i < roles->__sizechannelManager; i++)
    {
        roles->channelManager[i] = g_new0(glite_transfer_StringPair, 1);
#if _GSOAP_VERSION >= 0x030709
        roles->channelManager[i]->string1 = g_strdup(sroles->channelManager[i].string1);
        roles->channelManager[i]->string2 = g_strdup(sroles->channelManager[i].string2);
#else
        roles->channelManager[i]->string1 = g_strdup(sroles->channelManager[i]->string1);
        roles->channelManager[i]->string2 = g_strdup(sroles->channelManager[i]->string2);
#endif
    }

    return roles;
}

struct transfer__PlacementJob *to_soap_PlacementJob(struct soap *soap,
        const glite_transfer_PlacementJob *job)
{
        struct transfer__PlacementJob *sjob;
        unsigned i;

        if (!job)
                return NULL;

        sjob = soap_malloc(soap, sizeof(*sjob));
        if (!sjob)
                return NULL;
        memset(sjob, 0, sizeof(*sjob));

        sjob->sourceSE = soap_strdup(soap, job->sourceSE);
        sjob->destSE = soap_strdup(soap, job->destSE);
        if (!sjob->sourceSE || !sjob->destSE)
                return NULL;

    if (job->__sizeLogicalFiles)
    {
            sjob->logicalFiles = soap_malloc(soap,
                    job->__sizeLogicalFiles * sizeof(*sjob->logicalFiles));
            if (!sjob->logicalFiles)
                    return NULL;
            for (i = 0; i < job->__sizeLogicalFiles; i++)
            {
                    sjob->logicalFiles[i] = soap_strdup(soap, job->logicalFiles[i]);
                    if (!sjob->logicalFiles[i])
                            return NULL;
            }
            sjob->__sizelogicalFiles = job->__sizeLogicalFiles;
    }
        sjob->jobParams = to_soap_TransferParams(soap, job->jobParams);
        if (job->jobParams && !sjob->jobParams)
                return NULL;
        sjob->credential = soap_strdup(soap, job->credential);
        if (job->credential && !sjob->credential)
                return NULL;
        return sjob;
}
