/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  methods for channel data types
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: glite_transfer_channel.c,v 1.1 2008/11/26 08:57:59 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#include <glite/data/transfer/c/transfer-simple.h>
#include <glite/data/glite-util.h>
#include "internal.h"

/**-----------------------------------------------------------------------------
 * Create a new Channel object.
------------------------------------------------------------------------------*/

glite_transfer_Channel * glite_transfer_Channel_new(
    glite_transfer_ctx *ctx,
    const char *name,
    const char *sourceSite,
    const char *destSite) {

    glite_transfer_Channel *channel;

    channel = g_new0(glite_transfer_Channel, 1);
    if (!channel) {
        err_outofmemory(ctx);
        return NULL;
    }

    channel->channelName = g_strdup(name);
    channel->sourceSite = g_strdup(sourceSite);
    channel->destSite = g_strdup(destSite);

    return channel;
}

glite_transfer_Channel2 * glite_transfer_Channel2_new(
    glite_transfer_ctx *ctx,
    const char *name,
    const char *sourceSite,
    const char *destSite) {

    glite_transfer_Channel2 *channel2;
    channel2 = g_new0(glite_transfer_Channel2, 1);
    if (!channel2) {
        err_outofmemory(ctx);
        return NULL;
    }
    channel2->channelName = g_strdup(name);
    channel2->sourceSite = g_strdup(sourceSite);
    channel2->destSite = g_strdup(destSite);
    return channel2;
}

glite_transfer_Channel3 * glite_transfer_Channel3_new(
    glite_transfer_ctx *ctx,
    const char *name,
    const char *sourceSite,
    const char *destSite) {

    glite_transfer_Channel3 *channel3;
    channel3 = g_new0(glite_transfer_Channel3, 1);
    if (!channel3) {
        err_outofmemory(ctx);
        return NULL;
    }
    channel3->channelName = g_strdup(name);
    channel3->sourceSite = g_strdup(sourceSite);
    channel3->destSite = g_strdup(destSite);
    return channel3;
}

/**-----------------------------------------------------------------------------
 * Dispose a Channel object.
------------------------------------------------------------------------------*/

void glite_transfer_Channel_free(
    glite_transfer_ctx *ctx G_GNUC_UNUSED,
    glite_transfer_Channel *channel) {

    if (!channel)
        return;

    g_free(channel->channelName);
    g_free(channel->contact);
    g_free(channel->sourceSite);
    g_free(channel->destSite);
    g_free(channel);
}

void glite_transfer_Channel2_free(
    glite_transfer_ctx *ctx G_GNUC_UNUSED,
    glite_transfer_Channel2 *channel2) {

    if (!channel2)
        return;

    g_free(channel2->channelName);
    g_free(channel2->contact);
    g_free(channel2->sourceSite);
    g_free(channel2->destSite);
    g_free(channel2);
}

void glite_transfer_Channel3_free(
    glite_transfer_ctx *ctx G_GNUC_UNUSED,
    glite_transfer_Channel3 *channel3) {

    if (!channel3)
        return;

    g_free(channel3->channelName);
    g_free(channel3->contact);
    g_free(channel3->sourceSite);
    g_free(channel3->destSite);
    g_free(channel3);
}

/**-----------------------------------------------------------------------------
 * Clone an existing Channel object.
------------------------------------------------------------------------------*/

glite_transfer_Channel *glite_transfer_Channel_clone(
    glite_transfer_ctx *ctx,
    const glite_transfer_Channel *orig) {
    glite_transfer_Channel *clone;
    if (!orig)
        return NULL;
    clone = glite_transfer_Channel_new(ctx, orig->channelName,
                                       orig->sourceSite, orig->destSite);
    if (!clone)
        return NULL;
    clone->bandwidth = orig->bandwidth;
    clone->nominalThroughput = orig->nominalThroughput;
    clone->numberOfFiles = orig->numberOfFiles;
    clone->numberOfStreams = orig->numberOfStreams;
    clone->contact = g_strdup(orig->contact);
    clone->state = orig->state;
    return clone;
}

glite_transfer_Channel2 * glite_transfer_Channel2_clone(
    glite_transfer_ctx *ctx,
    const glite_transfer_Channel2 *orig) {

    glite_transfer_Channel2 *clone;
    if (!orig)
        return NULL;
    clone = glite_transfer_Channel2_new(ctx, orig->channelName,
                                        orig->sourceSite, orig->destSite);
    if (!clone)
        return NULL;
    clone->bandwidth = orig->bandwidth;
    clone->nominalThroughput = orig->nominalThroughput;
    clone->numberOfFiles = orig->numberOfFiles;
    clone->numberOfStreams = orig->numberOfStreams;
    clone->contact = g_strdup(orig->contact);
    clone->state = orig->state;
    return clone;
}

/**-----------------------------------------------------------------------------
 * Setters
------------------------------------------------------------------------------*/

int glite_transfer_Channel_setContact(
    glite_transfer_ctx *ctx,
    glite_transfer_Channel *channel,
    const char *contact) {

    if (!channel) {
        err_invarg(ctx, "Channel is missing");
        return -1;
    }
    if (!contact) {
        err_invarg(ctx, "Contact is missing");
        return -1;
    }

    g_free(channel->contact);
    channel->contact = g_strdup(contact);
    return 0;
}

int glite_transfer_Channel_setState(
    glite_transfer_ctx *ctx,
    glite_transfer_Channel *channel, 
    glite_channel_state state) {
    
    if (!channel) {
        err_invarg(ctx, "Channel is missing");
        return -1;
    }

    channel->state = state;
    return 0;
}

