/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Data Management - Simple Transfer API
 *
 *  Authors: Gabor Gombas <Gabor.Gombas@cern.ch>
 *  Version info: $Id: transfer_simple_api.c,v 1.17 2008/11/26 08:57:59 ptedesco Exp $
 *
 */

#include <glite/data/transfer/c/transfer-simple.h>

#include "internal.h"
#include <stdsoap2.h>

#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <errno.h>

/**********************************************************************
 * Data type definitions
 */

struct transfer_stateconv
{
    const char          *name;
    glite_transfer_state        state;
};


/**********************************************************************
 * Global variables
 */

static const struct transfer_stateconv transfer_states[] =
{
    { "Submitted",      GLITE_TRANSFER_SUBMITTED },
    { "Pending",        GLITE_TRANSFER_PENDING },
    { "Active",     GLITE_TRANSFER_ACTIVE },
    { "Canceling",      GLITE_TRANSFER_CANCELING },
    { "Done",       GLITE_TRANSFER_DONE },
    { "Finished",       GLITE_TRANSFER_FINISHED },
    { "FinishedDirty",  GLITE_TRANSFER_FINISHED_DIRTY },
    { "Canceled",       GLITE_TRANSFER_CANCELED },
    { "Waiting",        GLITE_TRANSFER_WAITING },
    { "Hold",       GLITE_TRANSFER_HOLD },
    { "Failed",     GLITE_TRANSFER_TRANSFERFAILED },
    { "CatalogFailed",  GLITE_TRANSFER_CATALOGFAILED },
    { "Ready",   GLITE_TRANSFER_READY },
    { "DoneWithErrors",   GLITE_TRANSFER_DONEWITHERRORS },
    { "Finishing", GLITE_TRANSFER_FINISHING },
    { "Failed", GLITE_TRANSFER_FAILED },
    { "AwaitingPrestage" , GLITE_TRANSFER_AWAITING_PRESTAGE },
    { "Prestaging", GLITE_TRANSFER_PRESTAGING },
    { "WaitingPrestage", GLITE_TRANSFER_WAITING_PRESTAGE },
    { "WaitingCatalogResolution", GLITE_TRANSFER_WAITING_CATALOG_RESOLUTION },
    { "WaitingCatalogRegistration", GLITE_TRANSFER_WAITING_CATALOG_REGISTRATION}
};


/**********************************************************************
 * Library interface functions
 */

glite_transfer_ctx *glite_transfer_new(const char *endpoint)
{
    glite_transfer_ctx *ctx;

    ctx = g_new0(glite_transfer_ctx, 1);
    if (!ctx)
        return NULL;

    ctx->default_endpoint = g_strdup(endpoint);

    /* Initialize the SOAP descriptor */
    ctx->soap = soap_new();
    if (!ctx->soap)
    {
        glite_transfer_free(ctx);
        return NULL;
    }
    ctx->soap->send_timeout = 60;
    ctx->soap->recv_timeout = 60;
       
    return ctx;
}

void glite_transfer_free(glite_transfer_ctx *ctx)
{
    if (!ctx)
        return;

    if (ctx->cleanup)
        ctx->cleanup(ctx);

    g_free(ctx->endpoint);
    g_free(ctx->default_endpoint);
    g_free(ctx->last_error);
    g_free(ctx->source);
    g_free(ctx->destination);
    if (ctx->soap)
    {
        soap_destroy(ctx->soap);
        soap_end(ctx->soap);
        free(ctx->soap);
    }
    g_free(ctx);
}

void glite_transfer_add_sites(glite_transfer_ctx *ctx, const char *s, const char *d)
{
    if (!ctx)
        return;

    ctx->source = g_strdup(s);
    ctx->destination = g_strdup(d);
}

const char *glite_transfer_get_error(glite_transfer_ctx *ctx)
{
    /* This means we can still call glite_transfer_get_error() after
     * a failed glite_transfer_new() and get a meaningful result */
    if (!ctx)
        return "Out of memory";

    if (!ctx->last_error)
        return "No error";
    return ctx->last_error;
}

glite_transfer_errclass glite_transfer_get_errclass(glite_transfer_ctx *ctx)
{
    if (!ctx)
        return GLITE_TRANSFER_ERROR_UNKNOWN;
    return ctx->errclass;
}

void glite_transfer_set_verror(glite_transfer_ctx *ctx,
    glite_transfer_errclass errclass, const char *fmt, va_list ap)
{
    char buf[2048];

    /* Sanity check */
    if (!ctx)
        return;

    vsnprintf(buf, sizeof(buf), fmt, ap);

    free(ctx->last_error);
    ctx->last_error = strdup(buf);

    ctx->errclass = errclass;
}

void glite_transfer_set_error(glite_transfer_ctx *ctx,
    glite_transfer_errclass errclass, const char *fmt, ...)
{
    va_list ap;

    /* Sanity check */
    if (!ctx)
        return;

    va_start(ap, fmt);
    glite_transfer_set_verror(ctx, errclass, fmt, ap);
    va_end(ap);
}

glite_transfer_state glite_transfer_state_parse(glite_transfer_ctx *ctx,
    const char *state)
{
    unsigned i;

    if (!state)
    {
        err_invarg(ctx, "Transfer state is missing");
        return GLITE_TRANSFER_UNKNOWN;
    }

    for (i = 0; i < G_N_ELEMENTS(transfer_states); i++)
        if (!strcasecmp(transfer_states[i].name, state))
            return transfer_states[i].state;

    err_invarg(ctx, "Unknown transfer state %s", state);
    return GLITE_TRANSFER_UNKNOWN;
}

const char *glite_transfer_state_str(glite_transfer_state state)
{
    unsigned i;

    for (i = 0; i < G_N_ELEMENTS(transfer_states); i++)
        if (transfer_states[i].state == state)
            return transfer_states[i].name;

    return "Unknown";
}
