/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  Author: Akos Frohner <Akos.Frohner@cern.ch>
 */

#include <glite/data/transfer/c/fts-simple.h>
#include <glite/data/transfer/c/channel-simple.h>
#include <Python.h>


static PyObject *PyTransferError;

typedef struct {
    PyObject_HEAD
    glite_transfer_ctx* ctx;
} PyTransfer;

static void PyTransfer_dealloc(PyTransfer *self)
{
     glite_transfer_free(self->ctx);
     self->ob_type->tp_free((PyObject*)self);
}

static PyObject *PyTransfer_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    PyTransfer *self;

    self = (PyTransfer*)type->tp_alloc(type, 0);
    if (self != NULL) {
        /* the location parameter is optional */
        const char *location = NULL;
        if (!PyArg_ParseTuple(args, "|z", &location))
            /* error already set */
            return NULL;

        self->ctx = glite_transfer_new(location);
        if (self->ctx == NULL) {
            Py_DECREF(self);
            PyErr_SetString(PyTransferError, "Failed to allocate the transfer context");
            return NULL;
        }

    }

    return (PyObject *)self;
}


#define PYTRANSFER_GET_VERSION(vname, pyfunction, cfunction)                    \
static PyObject *pyfunction(PyTransfer *self)                                   \
{                                                                               \
    PyObject *py_##vname = NULL;                                                \
    char *c_##vname = NULL;                                                     \
                                                                                \
    if (NULL == (c_##vname = cfunction(self->ctx))) {                           \
        PyErr_SetString(PyTransferError, glite_transfer_get_error(self->ctx));  \
        return NULL;                                                            \
    }                                                                           \
                                                                                \
    if (NULL == (py_##vname = PyString_FromString(c_##vname))) {                \
        /* error already set */                                                 \
        return NULL;                                                            \
    }                                                                           \
                                                                                \
    free(c_##vname);                                                            \
                                                                                \
    return py_##vname;                                                          \
}

#define PYTRANSFER_GET_FEATURES(pyfunction, cfunction)                          \
static PyObject *pyfunction(PyTransfer *self)                                   \
{                                                                               \
    PyObject *py_features = NULL;                                               \
    char *c_features = NULL;                                                    \
                                                                                \
    if (NULL == (c_features = cfunction(self->ctx, "feature.string"))) {        \
        PyErr_SetString(PyTransferError, glite_transfer_get_error(self->ctx));  \
        return NULL;                                                            \
    }                                                                           \
                                                                                \
    if (NULL == (py_features = PyString_FromString(c_features))) {              \
        /* error already set */                                                 \
        return NULL;                                                            \
    }                                                                           \
                                                                                \
    free(c_features);                                                           \
                                                                                \
    return py_features;                                                         \
}

#define PYTRANSFER_GET_ENDPOINT(pyfunction, cfunction)                          \
static PyObject *pyfunction(PyTransfer *self)                                   \
{                                                                               \
    PyObject *py_endpoint;                                                      \
    const char *c_endpoint;                                                     \
                                                                                \
    if (NULL == (c_endpoint = cfunction(self->ctx))) {                          \
        Py_DECREF(self);                                                        \
        PyErr_SetString(PyTransferError, glite_transfer_get_error(self->ctx));  \
        return NULL;                                                            \
    }                                                                           \
                                                                                \
    if (NULL == (py_endpoint = PyString_FromString(c_endpoint))) {              \
        /* error already set */                                                 \
        return NULL;                                                            \
    }                                                                           \
                                                                                \
    return py_endpoint;                                                         \
}

/*******************************************************************/
/* File Transfer Service                                           */
/*******************************************************************/

PYTRANSFER_GET_VERSION(interface_version, PyFTS_interface_version, glite_fts_getInterfaceVersion)
PYTRANSFER_GET_VERSION(schema_version, PyFTS_schema_version, glite_fts_getSchemaVersion)
PYTRANSFER_GET_VERSION(version, PyFTS_version, glite_fts_getVersion)

PYTRANSFER_GET_FEATURES(PyFTS_features, glite_fts_getServiceMetadata)

PYTRANSFER_GET_ENDPOINT(PyFTS_endpoint, glite_fts_get_endpoint)

static PyMethodDef PyFTS_methods[] = {
    {"interface_version", (PyCFunction)PyFTS_interface_version, METH_NOARGS,
     "returns the interface version of the server"},
    {"schema_version", (PyCFunction)PyFTS_schema_version, METH_NOARGS,
     "returns the schema version of the server"},
    {"version", (PyCFunction)PyFTS_version, METH_NOARGS,
     "returns the version of the server"},
    {"features", (PyCFunction)PyFTS_features, METH_NOARGS,
     "returns the features of the server"},
    {"endpoint", (PyCFunction)PyFTS_endpoint, METH_NOARGS,
     "returns the endpoint of the server"},
    {NULL, NULL, 0, NULL}
};

static PyTypeObject PyFTSType = {
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "fts.FileTransferService", /*tp_name*/
    sizeof(PyTransfer),        /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)PyTransfer_dealloc, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
    0,                         /*tp_compare*/
    0,                         /*tp_repr*/
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
    0,                         /*tp_hash */
    0,                         /*tp_call*/
    0,                         /*tp_str*/
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT,        /*tp_flags*/
    "FileTransferService objects", /* tp_doc */
    0,                         /* tp_traverse */
    0,                         /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    PyFTS_methods,             /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    0,                         /* tp_init */
    0,                         /* tp_alloc */
    PyTransfer_new             /* tp_new */
};


/*******************************************************************/
/* Channel Management                                              */
/*******************************************************************/


PYTRANSFER_GET_VERSION(interface_version, PyCM_interface_version, glite_channel_getInterfaceVersion)
PYTRANSFER_GET_VERSION(schema_version, PyCM_schema_version, glite_channel_getSchemaVersion)
PYTRANSFER_GET_VERSION(version, PyCM_version, glite_channel_getVersion)

PYTRANSFER_GET_FEATURES(PyCM_features, glite_channel_getServiceMetadata)

PYTRANSFER_GET_ENDPOINT(PyCM_endpoint, glite_channel_get_endpoint)

static PyObject *PyCM_channel_names(PyTransfer *self)
{
    PyObject *channels, *channel;
    char **tmp;
    int count, i;

    tmp = glite_channel_listChannels(self->ctx, &count);
    if (count < 0) {
        PyErr_SetString(PyTransferError, glite_transfer_get_error(self->ctx));
        return NULL;
    }

    if (NULL == (channels = PyTuple_New(count))) {
        glite_freeStringArray(count, tmp);
        return NULL;
    }

    for (i = 0; i < count; i++) {
        if (NULL == (channel = PyString_FromString(tmp[i]))) {
            glite_freeStringArray(count, tmp);
            Py_DECREF(channels);
            return NULL;
        }
        PyTuple_SET_ITEM(channels, i, channel);
    }

    glite_freeStringArray(count, tmp);
    return channels;    
}

static PyMethodDef PyCM_methods[] = {
    {"interface_version", (PyCFunction)PyCM_interface_version, METH_NOARGS,
     "returns the interface version of the server"},
    {"schema_version", (PyCFunction)PyCM_schema_version, METH_NOARGS,
     "returns the schema version of the server"},
    {"version", (PyCFunction)PyCM_version, METH_NOARGS,
     "returns the version of the server"},
    {"features", (PyCFunction)PyCM_features, METH_NOARGS,
     "returns the features of the server"},
    {"endpoint", (PyCFunction)PyCM_endpoint, METH_NOARGS,
     "returns the endpoint of the server"},
    {"channel_names", (PyCFunction)PyCM_channel_names, METH_NOARGS,
     "returns the name of the channels"},
    {NULL, NULL, 0, NULL}
};

static PyTypeObject PyCMType = {
    PyObject_HEAD_INIT(NULL)
    0,                         /*ob_size*/
    "fts.ChannelManagement",   /*tp_name*/
    sizeof(PyTransfer),        /*tp_basicsize*/
    0,                         /*tp_itemsize*/
    (destructor)PyTransfer_dealloc, /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
    0,                         /*tp_compare*/
    0,                         /*tp_repr*/
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
    0,                         /*tp_hash */
    0,                         /*tp_call*/
    0,                         /*tp_str*/
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT,        /*tp_flags*/
    "FileTransferService objects", /* tp_doc */
    0,                         /* tp_traverse */
    0,                         /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    PyCM_methods,             /* tp_methods */
    0,                         /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    0,                         /* tp_init */
    0,                         /* tp_alloc */
    PyTransfer_new             /* tp_new */
};

static PyMethodDef transfer_methods[] = {
    {NULL}
};

#ifndef PyMODINIT_FUNC    /* declarations for DLL import/export */
#define PyMODINIT_FUNC void
#endif
PyMODINIT_FUNC
initfts(void)
{
    PyObject* m;

    if (PyType_Ready(&PyFTSType) < 0) return;
    if (PyType_Ready(&PyCMType) < 0) return;

    m = Py_InitModule3("fts", transfer_methods, "File Transfer Service");
    if (NULL == m) return;

    Py_INCREF(&PyFTSType);
    PyModule_AddObject(m, "FileTransferService", (PyObject *)&PyFTSType);

    Py_INCREF(&PyCMType);
    PyModule_AddObject(m, "ChannelManagement", (PyObject *)&PyCMType);

    PyTransferError = PyErr_NewException("fts.TransferError", NULL, NULL);
    Py_INCREF(PyTransferError);
    PyModule_AddObject(m, "TransferError", PyTransferError);
}

