/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Transfer Clients - Add a new channel
 *
 *  Authors: Gabor Gombas <Gabor.Gombas@cern.ch>
 *
 */

#include <glite/data/transfer/c/channel-simple.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "tool-main.h"

/**********************************************************************
 * Global variables
 */

/* Usage message */
const char *tool_usage = "CHANNEL SITE-A SITE-B";

/* Help message */
const char *tool_help =
    "  -b NUM                       Set the bandwidth\n"
    "  -c STRING                    Set the contact string\n"
    "  -f NUM                       Set the number of files\n"
    "  -S STRING                    Set the channel state\n"
    "  -T NUM                       Set the number of streams\n"
    "  -t NUM                       Set the nominal throughput\n"
    ;
    
/* getguid-specific command line options */
const char *tool_options = "b:c:f:S:T:t:";
NO_TOOL_LONG_OPTIONS;

/* New bandwidth */
static int bandwidth;

/* New throughput */
static int throughput;

/* New contact */
static char *contact;

/* New number of files */
static int num_files;

/* New number of streams */
static int num_streams;

/* New channel state */
static glite_channel_state state = GLITE_CHANNEL_UNKNOWN;
/* Target directory check */
static int target_dir_check=0;
/* Time pased to mark first transfer for URL copy */
//
static int  url_copy_first_txmark_to=0;

/**********************************************************************
 * Tool implementation
 */

int tool_doit(glite_transfer_ctx *ctx, int argc, char *argv[]) {
    glite_transfer_Channel *channel;
    int ret;

    if (argc != 3) {
        error("Wrong number of arguments");
        return EXIT_FAILURE;
    }

    channel = glite_transfer_Channel_new(ctx, argv[0], argv[1], argv[2]);
    if (!channel) {
        error("set: %s", glite_transfer_get_error(ctx));
        return EXIT_FAILURE;
    }

    if (bandwidth > 0)
        channel->bandwidth = bandwidth;
    if (throughput > 0)
        channel->nominalThroughput = throughput;
    if (num_files > 0)
        channel->numberOfFiles = num_files;
    if (num_streams > 0)
        channel->numberOfStreams = num_streams;

    if (target_dir_check>0)
        channel->targetDirCheck = target_dir_check;

    if (url_copy_first_txmark_to > 0 )
        channel->urlCopyFirstTxmarkTo = url_copy_first_txmark_to;


    if (contact) {
        ret = glite_transfer_Channel_setContact(ctx, channel, contact);
        if (ret) {
            error("set: %s", glite_transfer_get_error(ctx));
            return EXIT_FAILURE;
        }
    }
    if (state != GLITE_CHANNEL_UNKNOWN) {
        ret = glite_transfer_Channel_setState(ctx, channel, state);
        if (ret) {
            error("set: %s", glite_transfer_get_error(ctx));
            return EXIT_FAILURE;
        }
    }

    ret = glite_channel_add(ctx, channel);
    glite_transfer_Channel_free(ctx, channel);
    if (ret) {
        if (strstr(glite_transfer_get_error(ctx), "org.glite.data.transfer.ExistsException") != NULL) {
            error("Error: The channel %s already exists on the FTS server.", argv[0]);
            return EXIT_FAILURE;
        } else {
            error("%s", glite_transfer_get_error(ctx));
            return EXIT_FAILURE;
        }
    }
    info("Added channel %s", argv[0]);
    return EXIT_SUCCESS;
}

/**********************************************************************
 * Command line parsing
 */

int tool_parse_cmdline(int c, char *opt_arg) {
    switch (c) {
    case 'b':
        if (!sscanf(opt_arg, "%d", &bandwidth)) {
            error("Unable to parse bandwidth argument");
            return -1;
        }
        return 0;
    case 'c':
        contact = opt_arg;
        return 0;
    case 'f':
        if (!sscanf(opt_arg, "%d", &num_files)) {
            error("Unable to parse number of files argument");
            return -1;
        }
        return 0;
    case 'S':
        state = glite_channel_state_parse(NULL, opt_arg);
        if (state == GLITE_CHANNEL_UNKNOWN) {
            error("Unknown channel state");
            return -1;
        }
        return 0;
    case 'T':
        if (!sscanf(opt_arg, "%d", &num_streams)) {
            error("Unable to parse number of streams argument");
            return -1;
        }
        return 0;
    case 't':
        if (!sscanf(opt_arg, "%d", &throughput)) {
            error("Unable to parse throughput argument");
            return -1;
        }
        return 0;
    case 'k':
        target_dir_check=1;
        return 0;
    case 'x':
        if (!sscanf(opt_arg, "%d", &url_copy_first_txmark_to)) {
            error("Unable to parse throughput arguement");
            return -1;
        }
        return 0;

    default:
        break;
    }
    return -1;
}
