/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Transfer Clients - Audit channels
 *
 *  Authors: Gavin McCance <gavin.mccance@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch> 
 *  Version info: $Id: audit-channel.c,v 1.6 2009/01/29 15:31:56 riojac3 Exp $
 *  Release: $Name:  $
 *
 */

#include <glite/data/transfer/c/channel-simple.h>
#include <glite/data/glite-util.h>

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "tool-main.h"

/* borrowed from time.h */
char *strptime(const char *s, const char *format, struct tm *tm);

/**********************************************************************
 * Global variables
 */

/* Whether to get the extended channel information */
int max_number = 5;

char * restrict_type = "";

time_t starttime = 0;
time_t endtime = 0;

/* Usage message */
const char *tool_usage = "CHANNEL";

/* Help message */
const char *tool_help =
"  -m NUM                       Max number of events to fetch (default 5)\n"
"  -r TYPE                      The type of events to search for \n"
"                               (state, files, streams)\n"
"  -b TIME                      Start time\n"
"  -e TIME                      End time\n"
;

/* getguid-specific command line options */
const char *tool_options = "m:r:b:e:";
NO_TOOL_LONG_OPTIONS;


/**********************************************************************
 * Tool implementation
 */

static char * getMessage(char * mes, char * def) {
    if (mes==NULL)
        return def;
    else if (strlen(mes)<1)
        return def;
    else
        return mes;
}

int tool_doit(glite_transfer_ctx *ctx, int argc G_GNUC_UNUSED, char *argv[]) {
    int i, list_cnt;
    struct tm * tmptm;
    const size_t bufsize = 100;
    char nof[bufsize];
    char nos[bufsize];
    int istInteresting;

    if (argc != 1) {
        error("Wrong number of arguments");
        return EXIT_FAILURE;
    }

    // check for extended options
    if (!(isExtendedChannelListSupported())) {
        error("The server you are contacting does not support the channel-audit command (it is running interface version %s).", srv_iface_ver);
        return EXIT_FAILURE;
    }

    glite_transfer_ChannelAuditEntry **list;

    list = glite_channel_channelAudit(ctx, starttime, endtime, restrict_type, max_number, argv[0], &list_cnt);

    if (list_cnt < 0)
        return -1;

    if(&list == NULL)
	return -1;
    

    for (i = 0; i < list_cnt; i++) {
        istInteresting = 0;
        tmptm = localtime(&(list[i]->modificationTime));
        if (list[i]->nofiles==-1) {
            snprintf(nof, bufsize, "-");
        } else {
            istInteresting = 1;
            snprintf(nof, bufsize, "%i", list[i]->nofiles);
        }
        if (list[i]->nostreams==-1) {
            snprintf(nos, bufsize, "-");
        } else {
            istInteresting = 2;
            snprintf(nos, bufsize, "%i", list[i]->nostreams);
        }
        if (strcmp("Unknown", glite_channel_state_str(list[i]->channelState))) {
            istInteresting = 3;
        }
        if (!istInteresting)
            continue;
        printf("  Channel: %s\n    State: %s\n   #Files: %s\n #Streams: %s\n"
               "  AdminDN: %s\nTimestamp: %04d-%02d-%02d %02d:%02d:%02d\n"
               "  Message: %s\n\n",
               list[i]->channelName,
               !strcmp("Unknown", glite_channel_state_str(list[i]->channelState))?"-":glite_channel_state_str(list[i]->channelState),
               nof,
               nos,
               list[i]->adminDN,
               tmptm->tm_year+1900, tmptm->tm_mon+1, tmptm->tm_mday, tmptm->tm_hour, tmptm->tm_min, tmptm->tm_sec,
               getMessage(list[i]->message,"no message"));
    }


    return EXIT_SUCCESS;
}

/**********************************************************************
 * Command line parsing
 */
int tool_parse_cmdline(int c, char * opt_arg) {
    switch (c) {
        case 'r':
        restrict_type = opt_arg;
        return 0;
    case('m'):
        if (!sscanf(opt_arg, "%d", &max_number)) {
            error("Unable to parse max number argument");
            return -1;
        }
        return 0;
    case('b'):
        if (!sscanf(opt_arg, "%d", &starttime)) {
            error("Unable to parse start time. It should be seconds since UTC epoch.");
            return -1;
        }
        return 0;
    case('e'):
        if (!sscanf(opt_arg, "%d", &endtime)) {
            error("Unable to parse end time. It should be seconds since UTC epoch.");
            return -1;
        }
        return 0;
    default:
        break;
    }
    return -1;
}
