/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  gLite Transfer Clients - List channels
 *
 *  Authors: Gabor Gombas <Gabor.Gombas@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: list-channel.c,v 1.12 2009/01/19 14:29:51 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include <glite/data/transfer/c/channel-simple.h>
#include <glite/data/glite-util.h>

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <time.h>

#include "tool-main.h"

/**********************************************************************
 * Global variables
 */

/* Whether to get the extended channel information */
int extended_list;

/* Usage message */
const char *tool_usage = "[CHANNEL...]";

/* Help message */
const char *tool_help =
    "  -x                           Extended channel information\n"
    ;
NO_TOOL_LONG_OPTIONS;

/* getguid-specific command line options */
const char *tool_options = "x";

/**********************************************************************
 * print functions
 */

/**
 * print a timestamp
 * @param msg [IN]
 * @param t [IN] 
 */
void prtime(const char * msg,time_t * t);
void prtime(const char * msg,time_t * t){
    struct tm *tm_t = localtime(t);
    printf("%s: %02d-%02d-%02d %02d:%02d:%02d\n",
        msg,tm_t->tm_year+1900,tm_t->tm_mon+1,tm_t->tm_mday,tm_t->tm_hour,tm_t->tm_min,tm_t->tm_sec);
}

/**
 * return given value if not null, default otherwise
 * @param in [IN]
 * @param def [IN] 
 * @return value or default
 */
const char * val(const char * in,const char * def);
const char * val(const char * in,const char * def){
    return in ? in : def;
}

/**
 * print a timeout value
 * @param msg [IN] 
 * @param to [IN] the timeout value
 */
void prtout(const char * msg,int to);
void prtout(const char * msg,int to){
    if (to < 0){
        printf("%s: unset\n",msg);
    } else {
        printf("%s: %d\n",msg,to);
    }
}
 
void print_channel(glite_transfer_Channel * ch);
void print_channel(glite_transfer_Channel * ch){
    int i = 0;
    printf("Channel: %s\n", ch->channelName);
    printf("Between: %s and %s\n", ch->sourceSite,ch->destSite);
    printf("State: %s\n", glite_channel_state_str(ch->state));
    printf("Contact: %s\n", ch->contact);
    printf("Bandwidth: %d\n", ch->bandwidth);
    printf("Nominal throughput: %d\n", ch->nominalThroughput);
    printf("Number of files: %d, streams: %d\n",ch->numberOfFiles, ch->numberOfStreams);
    printf("Number of VO shares: %d\n", ch->__sizeVOShares);
    for (i = 0; i < ch->__sizeVOShares; ++i) {
        printf("VO '%s' share is: %s\n",ch->VOShares[i]->VOName,ch->VOShares[i]->share);
    }
}

void print_channel2(glite_transfer_Channel2 * ch);
void print_channel2(glite_transfer_Channel2 * ch){
    int i = 0;
    printf("Channel: %s\n",         ch->channelName);
    printf("Between: %s and %s\n",  ch->sourceSite,ch->destSite);
    printf("State: %s\n",           glite_channel_state_str(ch->state));
    printf("Contact: %s\n",         ch->contact);
    printf("Bandwidth: %d\n",       ch->bandwidth);
    printf("Nominal throughput: %d\n",  ch->nominalThroughput);
    printf("Number of files: %d, streams: %d\n",ch->numberOfFiles, ch->numberOfStreams);
    printf("TCP buffer size: %s\n", (ch->tcpBufferSize!=NULL)?ch->tcpBufferSize:"default");
    printf("Message: %s\n", (ch->message!=NULL)?ch->message:"no message");
    printf("Last modification by: %s\n", ch->lastModifierDn);
    prtime("Last modification time",&(ch->lastModificationTime));
    printf("Number of VO shares: %d\n", ch->__sizeVOShares);
    for (i = 0; i < ch->__sizeVOShares; ++i) {
        if ( ! strcmp(ch->VOLimits[i]->limit, "0") ) {
            printf("VO '%s' share is: %s and is not limited\n",ch->VOShares[i]->VOName,ch->VOShares[i]->share);
        } else {
            printf("VO '%s' share is: %s and is limited to %s transfers\n",
                ch->VOShares[i]->VOName,ch->VOShares[i]->share,ch->VOLimits[i]->limit);
        }
    }
}

void print_channel3(glite_transfer_Channel3 * ch);
void print_channel3(glite_transfer_Channel3 * ch){
    int i = 0;
    printf("Channel: %s\n",                     ch->channelName);
    printf("Between: %s and %s\n",              ch->sourceSite,ch->destSite);
    printf("State: %s\n",                       glite_channel_state_str(ch->state));
    printf("Contact: %s\n",                     ch->contact);
    printf("Bandwidth: %d\n",                   ch->bandwidth);
    printf("Nominal throughput: %d\n",          ch->nominalThroughput);
    printf("Number of files: %d, streams: %d\n",ch->numberOfFiles, ch->numberOfStreams);
    printf("TCP buffer size: %s\n",             val(ch->tcpBufferSize,"default"));
    printf("Message: %s\n",                     val(ch->message,"no message"));
    printf("Last modification by: %s\n",        ch->lastModifierDn);
    prtime("Last modification time",            &(ch->lastModificationTime));
    prtime("Last active time",                  &(ch->lastActiveTime));
#if 0 
    // do not show these values...
    printf("Storage Element limit: %d\n",       ch->seLimit);
    printf("Chanel type: %s",                   val(ch->channelType,"undefined"));
#endif
    prtout("First transfer marker timeout",     ch->urlCopyFirstTxmarkTo);
    printf("Check target directory: %s\n",      ch->targetDirCheck ? "Y" : "N");
    
    printf("Block size: %s\n",                  val(ch->blockSize,"default"));
    prtout("Http timeout",                      ch->httpTimeout);
    printf("Transfer log level: %s\n",          val(ch->transferLogLevel,"default"));
    printf("SRM negotiations/transfers : %f\n", ch->preparingFilesRatio);
    printf("Transfer type: %s\n",               val(ch->transferType,"NOT SET"));
    prtout("Put timeout",                       ch->urlCopyPutTimeout);
    prtout("Put done timeout",                  ch->urlCopyPutDoneTimeout);
    prtout("Get timeout",                       ch->urlCopyGetTimeout);
    prtout("Get done timeout",                  ch->urlCopyGetDoneTimeout);
    prtout("Url copy transfer timeout",         ch->urlCopyTransferTimeout);
    prtout("Transfer markers timeout",          ch->urlCopyTransferMarkersTimeout);
    prtout("No progress timeout",               ch->urlCopyNoProgressTimeout);
    if (ch->urlCopyTransferTimeoutPerMB > 0){
        printf("Size-based additional timeout: %f (secs/MB)\n",ch->urlCopyTransferTimeoutPerMB);
    } else {
        printf("Size-based additional timeout: unset\n");
    }
    printf("Srm copy direction: %s\n",          val(ch->srmCopyDirection,"default"));
    prtout("Srm copy timeout",                  ch->srmCopyTimeout);
    prtout("Srm copy refresh timeout",          ch->srmCopyRefreshTimeout);
    
    printf("Number of VO shares: %d\n", ch->__sizeVOShares);
    for (i = 0; i < ch->__sizeVOShares; ++i) {
        if ( ! strcmp(ch->VOLimits[i]->limit, "0") ) {
            printf("VO '%s' share is: %s and is not limited\n",ch->VOShares[i]->VOName,ch->VOShares[i]->share);
        } else {
            printf("VO '%s' share is: %s and is limited to %s transfers\n",
                ch->VOShares[i]->VOName,ch->VOShares[i]->share,ch->VOLimits[i]->limit);
        }
    }
}

/**********************************************************************
 * Tool implementation
 */

int tool_doit(glite_transfer_ctx *ctx, int argc, char *argv[]) {
    int i;

    if (!argc) {
        // no channel names specified; print list of channel names and exit
        char **tmp;
        int cnt;

        tmp = glite_channel_listChannels(ctx, &cnt);
        if (cnt < 0) {
            error("list: %s", glite_transfer_get_error(ctx));
            return EXIT_FAILURE;
        }

        for (i = 0; i < cnt; i++)
            printf("%s\n", tmp[i]);
        glite_freeStringArray(cnt, tmp);
        return EXIT_SUCCESS;
    }
    
    // channel names were specified; print details for each channel
    
    // check for extended options
    if (extended_list && !(isExtendedChannelListSupported())) {
        error("The server you are contacting does not support the -x (extended list) option of channel-list (it is running interface version %s).", srv_iface_ver);
        return EXIT_FAILURE;
    }

    for (i = 0; i < argc; i++) {
        if (i)
            printf("\n");

        if (extended_list) {
            if (isItVersion350()){
                glite_transfer_Channel3 *channel3;
                channel3 = glite_channel_getChannel3(ctx, argv[i]);
                if (!channel3) {
                    error("list: %s", glite_transfer_get_error(ctx));
                    return EXIT_FAILURE;
                }
                print_channel3(channel3);
                glite_transfer_Channel3_free(ctx, channel3);
            } else {
                glite_transfer_Channel2 *channel2;
                channel2 = glite_channel_getChannel2(ctx, argv[i]);
                if (!channel2) {
                    error("list: %s", glite_transfer_get_error(ctx));
                    return EXIT_FAILURE;
                }
                print_channel2(channel2);
                glite_transfer_Channel2_free(ctx, channel2);
            }
        } else { // old listing

            glite_transfer_Channel *channel;
            channel = glite_channel_getChannel(ctx, argv[i]);
            if (!channel) {
                error("list: %s", glite_transfer_get_error(ctx));
                return EXIT_FAILURE;
            }
            print_channel(channel);
            glite_transfer_Channel_free(ctx, channel);
        }

    }

    return EXIT_SUCCESS;
}

/**********************************************************************
 * Command line parsing
 */

int tool_parse_cmdline(int c G_GNUC_UNUSED, char *opt_arg G_GNUC_UNUSED) {
    switch (c) {
    case 'x':
        extended_list = 1;
        return 0;
    default:
        break;
    }
    return -1;
}
