/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Transfer Clients - Service info tool
 *
 *  Authors: Gabor Gombas <Gabor.Gombas@cern.ch>
 *           Zoltan Farkas <Zoltan.Farkas@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 */

#include <glite/data/glite-util.h>
#include <glite/data/transfer/c/fts-simple.h>

#include <getopt.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "tool-main.h"

/**********************************************************************
 * Global variables
 */

/* Location of the transfer service */
char *service_location;

/* Transfer context handle */
static glite_transfer_ctx *ctx;

/* Flag to turn on verbose output */
int verbose_flag = 1;

/* Flag to request quiet operation */
int quiet_flag = 0;


/* Command line options common to all the tools */
static const char *common_options = "hs:V";

/* Description of the common options */
static const char *common_help =
"  -h, --help                         Print this help text and exit.\n"
"  -s URL                             Use the transfer service at the specified URL.\n"
"      --version                      Print service version.\n"
"      --interface-version            Print service interface version.\n"
"      --schema-version               Print schema version.\n"
"      --service-metadata KEY         Print service metadata value for key KEY.\n"
"  -V                                 Print the version number and exit.\n";

/* Long options */
static struct option long_options[] = {
    {"help",                no_argument,        NULL, 'h'},
    {"version",             no_argument,        NULL, MAIN_OPT_NUM(0)},
    {"interface-version",   no_argument,        NULL, MAIN_OPT_NUM(1)},
    {"schema-version",      no_argument,        NULL, MAIN_OPT_NUM(2)},
    {"service-metadata",    required_argument,  NULL, MAIN_OPT_NUM(3)},
    {NULL,                  no_argument,        NULL, 0}
};

/* What to query */
static int get_version;
static int get_interface_version;
static int get_schema_version;
static char *get_service_meta = NULL;

/**********************************************************************
 * Main routine for the gLite file placement tools
 */

int main(int argc, char *argv[]) {
    char *options, *prog_name;
    const char *endpoint;
    int optlen, ret, c;
    int options_index = 0;
    
    ret = 0;

    /* Determine the program name we were called as */
    prog_name = strrchr(argv[0], '/');
    if (!prog_name)
        prog_name = argv[0];
    else
        prog_name++;

    /* Build the option string for getopt() */
    optlen = strlen(common_options) + 1;

    options = malloc(optlen);
    if (!options) {
        fprintf(stderr, "Out of memory\n");
        exit(EXIT_FAILURE);
    }

    strcpy(options, common_options);

    /* Parse the command line options */
    while ((c = getopt_long(argc, argv, options, long_options, &options_index)) != -1)
        switch (c) {
        case 'h':
            printf("Usage: %s [options]\n", prog_name);
            printf("Available options:\n%s", common_help);
            exit(EXIT_SUCCESS);
        case 's':
            service_location = optarg;
            break;
        case 'V':
            printf("%s\n", MODULE_VERSION);
            exit(EXIT_SUCCESS);
        case MAIN_OPT_NUM(0):
            get_version = 1;
            break;
        case MAIN_OPT_NUM(1):
            get_interface_version = 1;
            break;
        case MAIN_OPT_NUM(2):
            get_schema_version = 1;
            break;
        case MAIN_OPT_NUM(3):
            if (!get_service_meta)
                get_service_meta = strdup(optarg);
            else {
                char *t;
                if (-1 == asprintf(&t, "%s,%s", get_service_meta, optarg)) {
                    error("Memory allocation failed!");
                    exit(EXIT_FAILURE);
                }
                free(get_service_meta);
                get_service_meta = t;
            }
            break;
        case ':':
            fprintf(stderr, "Option argument is missing\n");
            exit(EXIT_FAILURE);
        case '?':
            fprintf(stderr, "Unknown command line option\n");
            exit(EXIT_FAILURE);
        default:
            fprintf(stderr, "Unknown command line option\n");
            exit(EXIT_FAILURE);
        }

    /* Drop the processed options from the argument list */
    free(options);
    argc -= optind;
    argv += optind;

    /* Allocate a transfer context */
    ctx = glite_transfer_new(service_location);
    if (!ctx) {
        error("Failed to allocate the transfer context");
        exit(EXIT_FAILURE);
    }

    if (NULL == (endpoint = glite_fts_get_endpoint(ctx))) {
        error("Failed to determine the endpoint: %s",
              glite_transfer_get_error(ctx));
        exit(EXIT_FAILURE);
    }
    if (get_version) {
        char *ver;
        ver = glite_fts_getVersion(ctx);
        if (!ver) {
            error("Failed to determine the version of the "
                  "service: %s", glite_transfer_get_error(ctx));
            exit(EXIT_FAILURE);
        }
        info("# Service version: %s", ver);
        free(ver);
    }
    if (get_interface_version) {
        char *ver;
        ver = glite_fts_getInterfaceVersion(ctx);
        if (!ver) {
            error("Failed to determine the interface version of the "
                  "service: %s", glite_transfer_get_error(ctx));
            exit(EXIT_FAILURE);
        }
        info("# Interface version: %s", ver);
        free(ver);
    }
    if (get_schema_version) {
        char *ver;
        ver = glite_fts_getSchemaVersion(ctx);
        if (!ver) {
            error("Failed to determine the schema version of "
                  "the service: %s",
                  glite_transfer_get_error(ctx));
            exit(EXIT_FAILURE);
        }
        info("# Schema version: %s", ver);
    }
    if (get_service_meta) {
        char *act;
        act = strtok(get_service_meta, ",");
        while (act) {
            char *data;
            data = glite_fts_getServiceMetadata(ctx, act);
            if (data) {
                info("# Service metadata %s key value: %s", act, data);
                free(data);
            } else {
                info("# Service metadata %s key value: %s", act, "unknown");
            }
            act = strtok(NULL, ",");
        }
    }

    /* Clean up properly so checking for memory leaks is easier */
    glite_transfer_free(ctx);
    return ret;
}
