/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Transfer Clients - Modify channel status
 *
 *  Authors: Gabor Gombas <Gabor.Gombas@cern.ch>
 */

#include <glite/data/transfer/c/channel-simple.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "tool-main.h"

/**********************************************************************
 * Global variables
 */

/* Usage message */
const char *tool_usage = "CHANNEL [CHANNEL...]";

/* Help message */
const char *tool_help =
"  -b NUM                             Set the bandwidth\n"
"  -c STRING                          Set the contact string\n"
"  -u STRING                          Set the TCP buffer size string\n"
"  -f NUM                             Set the number of files\n"
"  -S STRING                          Set the channel state [Active, Inactive, Drain,\n"
"                                         Halted, Stopped, Archived]\n"
"  -T NUM                             Set the number of streams\n"
"  -t NUM                             Set the nominal throughput\n"
"  -m MESSAGE                         A descriptive message for the change\n"
"  -k Y|N                             Perform destination directory existence checks\n"
"  -x NUM|unset                       First transfer marker timeout (seconds)\n"
"     --block-size VAL|unset          Default block size of a transfer, e.g. '1MB'\n"
"                                         or '1024kb'\n"
"     --http-timeout N|unset          Http timeout in seconds\n"
"     --tx-log-level VAL              Log level of the transfer process [DEBUG, \n"
"                                         INFO (default), WARN, ERROR]\n"
"     --preparing-ratio VAL           Ratio of SRM activity to number of transfer \n"
"                                         slots (floating point value)\n"
"     --urlcopy-put-to N|unset        Timeout on put operation for urlcopy channels \n"
"                                         (seconds)\n"
"     --urlcopy-putdone-to N|unset    Timeout on putDone operation for urlcopy \n"
"                                         channels (seconds)\n"
"     --urlcopy-get-to N|unset        Timeout on get operation for urlcopy channels \n"
"                                         (seconds)\n"
"     --urlcopy-getdone-to N|unset    Timeout on getDone operation for urlcopy \n"
"                                         channels (seconds)\n"
"     --urlcopy-tx-to N|unset         Urlcopy transfer timeout (seconds)\n"
"     --urlcopy-tx-marks-to N|unset   Urlcopy transfer markers timeout (seconds)\n"
"     --urlcopy-no-prog-to N|unset    Maximum time without transfer progress (seconds)\n"
"     --urlcopy-tx-to-per-mb N|unset  Additional size-based timeout (seconds/MB, \n"
"                                         floating point value)\n"
"     --srmcopy-direction VAL         Srmcopy direction [PUSH, PULL]\n"
"     --srmcopy-to N|unset            Srmcopy timeout (seconds)\n"
"     --srmcopy-refresh-to N|unset    Timeout between status updates (seconds)\n"
;

/* Long options */
struct option tool_long_options[] = {
    {"block-size",              required_argument,  NULL,   TOOL_OPT_NUM(0)},
    {"http-timeout",            required_argument,  NULL,   TOOL_OPT_NUM(1)},
    {"tx-log-level",            required_argument,  NULL,   TOOL_OPT_NUM(2)},
    {"preparing-ratio",         required_argument,  NULL,   TOOL_OPT_NUM(3)},
    {"urlcopy-put-to",          required_argument,  NULL,   TOOL_OPT_NUM(4)},
    {"urlcopy-putdone-to",      required_argument,  NULL,   TOOL_OPT_NUM(5)},
    {"urlcopy-get-to",          required_argument,  NULL,   TOOL_OPT_NUM(6)},
    {"urlcopy-getdone-to",      required_argument,  NULL,   TOOL_OPT_NUM(7)},
    {"urlcopy-tx-to",           required_argument,  NULL,   TOOL_OPT_NUM(8)},
    {"urlcopy-tx-marks-to",     required_argument,  NULL,   TOOL_OPT_NUM(9)},
    {"urlcopy-no-prog-to",      required_argument,  NULL,   TOOL_OPT_NUM(10)},
    {"urlcopy-tx-to-per-mb",    required_argument,  NULL,   TOOL_OPT_NUM(11)},
    {"srmcopy-direction",       required_argument,  NULL,   TOOL_OPT_NUM(12)},
    {"srmcopy-to",              required_argument,  NULL,   TOOL_OPT_NUM(13)},
    {"srmcopy-refresh-to",      required_argument,  NULL,   TOOL_OPT_NUM(14)},
        
};
SET_TOOL_LONG_OPTIONS_SIZE;
    
/* getguid-specific command line options */
const char *tool_options = "b:c:f:S:T:t:u:m:k:x:";

/* New bandwidth */
static int bandwidth=-1;

/* New throughput */
static int throughput=-1;

/* New contact */
static char *contact = NULL;

/* New TCP buffer size */
static char * buffer_size = NULL;

/* The message (why the chnage was made) */
static char * message = NULL;

/* New number of files */
static int num_files=-1;

/* New number of streams */
static int num_streams=-1;

/* New channel state */
static glite_channel_state state = GLITE_CHANNEL_UNKNOWN;

/* Number of updates */
static int updates = 0;
static int updates_340 = 0;
static int updates_350 = 0;

int get_updates(void); // note: without void produces warning 'no previous prototype'
int get_updates(){
    return (updates + updates_340 + updates_350);
}

/*  If to Check of the target directory */
static int target_dir_check=-1;

/* Time marker for the first transfer activity in seconds*/
static int url_copy_first_txmark_to = -1;
static int unset_url_copy_first_txmark_to = 0;

static char * block_size = NULL;
static int unset_block_size = 0;

static int http_timeout = -1;
static int unset_http_timeout = 0;

static transfer_log_level tx_log_level = TRANSFER_LOG_LEVEL_INVALID;
//static int unset_tx_log_level = 0;

static double preparing_ratio = -1;
//static int unset_preparing_ratio = 0;

static int urlcopy_put_to = -1;
static int unset_urlcopy_put_to = 0;

static int urlcopy_putdone_to = -1;
static int unset_urlcopy_putdone_to = 0;

static int urlcopy_get_to = -1;
static int unset_urlcopy_get_to = 0;

static int urlcopy_getdone_to = -1;
static int unset_urlcopy_getdone_to = 0;

static int urlcopy_tx_to = -1;
static int unset_urlcopy_tx_to = 0;

static int urlcopy_tx_marks_to = -1;
static int unset_urlcopy_tx_marks_to = 0;

static int urlcopy_no_prog_to = -1;
static int unset_urlcopy_no_prog_to = 0;

static double urlcopy_tx_to_per_mb = -1;
static int unset_urlcopy_tx_to_per_mb = 0;

static srmcopy_direction srmcopy_dir = SRMCOPY_DIRECTION_INVALID;
//static int unset_srmcopy_direction = 0;

static int srmcopy_to = -1;
static int unset_srmcopy_to = 0;

static int srmcopy_refresh_to = -1;
static int unset_srmcopy_refresh_to = 0;

/**********************************************************************
 * Tool implementation
 */

static int set_channel(glite_transfer_ctx *ctx, const char *name) {
    int ret;

    // check version
    if (buffer_size && !isSetTCPBufferSupported()) {
        glite_transfer_set_error(
            ctx, 
            GLITE_TRANSFER_ERROR_INVALIDARGUMENT,
            "The server (interface version %s) does not support the -u option.", 
            srv_iface_ver);
        return EXIT_FAILURE;
    }

    if (message && !isChannelMessageSupported()) {
        glite_transfer_set_error(
            ctx, 
            GLITE_TRANSFER_ERROR_INVALIDARGUMENT,
            "The server (interface version %s) does not support the -m option.", 
            srv_iface_ver);
        return EXIT_FAILURE;
    }

    if (updates_340 && !isItVersion340()) {
        glite_transfer_set_error(
            ctx, 
            GLITE_TRANSFER_ERROR_INVALIDARGUMENT,
            "The server (interface version %s) does not support the -k and -x options.", 
            srv_iface_ver);
        return EXIT_FAILURE;
    }

    if (updates_350 && !isItVersion330()) {
        glite_transfer_set_error(
            ctx, 
            GLITE_TRANSFER_ERROR_INVALIDARGUMENT,
            "The server (interface version %s) does not support one or more of the specified options.", 
            srv_iface_ver);
        return EXIT_FAILURE;
    }

    if (!message) {
        message = "no reason was provided";
    }

    if (isChannelMessageSupported()) { // call a different API

        if (bandwidth >= 0) {
            ret = glite_channel_setBandwidth2(ctx, name, bandwidth, message);
            if (ret)
                return ret;
        }
        if (throughput >= 0) {
            ret = glite_channel_setNominalThroughput2(ctx, name, throughput, message);
            if (ret)
                return ret;
        }
        if (num_files >= 0) {
            ret = glite_channel_setNumberOfFiles2(ctx, name, num_files, message);
            if (ret)
                return ret;
        }
        if (num_streams >= 0) {
            ret = glite_channel_setNumberOfStreams2(ctx, name, num_streams, message);
            if (ret)
                return ret;
        }
        if (contact) {
            ret = glite_channel_setContact2(ctx, name, contact, message);
            if (ret)
                return ret;
        }
        if (buffer_size) {
            ret = glite_channel_setTCPBufferSize2(ctx, name, buffer_size, message);
            if (ret)
                return ret;
        }
        if (state >= 0) {
            ret = glite_channel_setState2(ctx, name, state, message);
            if (ret)
                return ret;
        }

        if (target_dir_check >= 0 ) {
            ret = glite_channel_setTargetDirCheck(ctx, name, target_dir_check, message);
            if (ret)
                return ret;

        }

        if (url_copy_first_txmark_to >= 0 || unset_url_copy_first_txmark_to) {
            ret = glite_channel_setUrlCopyFirstMarkTo(ctx, name, url_copy_first_txmark_to, message);
            if (ret)
                return ret;
        }

    } else {

        if (bandwidth >= 0) {
            ret = glite_channel_setBandwidth(ctx, name, bandwidth);
            if (ret)
                return ret;
        }
        if (throughput >= 0) {
            ret = glite_channel_setNominalThroughput(ctx, name, throughput);
            if (ret)
                return ret;
        }
        if (num_files >= 0) {
            ret = glite_channel_setNumberOfFiles(ctx, name, num_files);
            if (ret)
                return ret;
        }
        if (num_streams >= 0) {
            ret = glite_channel_setNumberOfStreams(ctx, name, num_streams);
            if (ret)
                return ret;
        }
        if (contact) {
            ret = glite_channel_setContact(ctx, name, contact);
            if (ret)
                return ret;
        }
        if (buffer_size) {
            ret = glite_channel_setTCPBufferSize2(ctx, name, buffer_size, "Default comment");
            if (ret)
                return ret;
        }
        if (state >= 0) {
            ret = glite_channel_setState(ctx, name, state);
            if (ret)
                return ret;
        }
    }

#define CHANNEL_SET(function,value) \
    ret = function(ctx,name,value,message); \
    if (ret) return ret;

    if (NULL != block_size || unset_block_size){
        CHANNEL_SET(glite_channel_setBlockSize,block_size);
    }
    
    if (http_timeout >= 0 || unset_http_timeout){
        CHANNEL_SET(glite_channel_setHttpTimeout,http_timeout);
    }
    
    if (TRANSFER_LOG_LEVEL_INVALID != tx_log_level){
        CHANNEL_SET(glite_channel_setTransferLogLevel,tx_log_level);
    }
    
    if (-1 != preparing_ratio){
        CHANNEL_SET(glite_channel_setPreparingFilesRatio,preparing_ratio);
    }
    
    if (urlcopy_put_to >= 0 || unset_urlcopy_put_to){
        CHANNEL_SET(glite_channel_setUrlCopyPutTimeout,urlcopy_put_to);
    }
    
    if (urlcopy_putdone_to >= 0 || unset_urlcopy_putdone_to){
        CHANNEL_SET(glite_channel_setUrlCopyPutDoneTimeout,urlcopy_putdone_to);
    }

    if (urlcopy_get_to >= 0 || unset_urlcopy_get_to){
        CHANNEL_SET(glite_channel_setUrlCopyGetTimeout,urlcopy_get_to);
    }
    
    if (urlcopy_getdone_to >= 0 || unset_urlcopy_getdone_to){
        CHANNEL_SET(glite_channel_setUrlCopyGetDoneTimeout,urlcopy_getdone_to);
    }

    if (urlcopy_tx_to >= 0 || unset_urlcopy_tx_to){
        CHANNEL_SET(glite_channel_setUrlCopyTransferTimeout,urlcopy_tx_to);
    }
    
    if (urlcopy_tx_marks_to >= 0 || unset_urlcopy_tx_marks_to){
        CHANNEL_SET(glite_channel_setUrlCopyTransferMarkersTimeout,urlcopy_tx_marks_to);
    }
    
    if (urlcopy_no_prog_to >= 0 || unset_urlcopy_no_prog_to){
        CHANNEL_SET(glite_channel_setUrlCopyNoProgressTimeout,urlcopy_no_prog_to);
    }
    
    if (urlcopy_tx_to_per_mb >= 0 || unset_urlcopy_tx_to_per_mb){
        CHANNEL_SET(glite_channel_setUrlCopyTransferTimeoutPerMB,urlcopy_tx_to_per_mb);
    }
    
    if (SRMCOPY_DIRECTION_INVALID != srmcopy_dir){
        CHANNEL_SET(glite_channel_setSrmCopyDirection,srmcopy_dir);
    }
    
    if (srmcopy_to >= 0 || unset_srmcopy_to){
        CHANNEL_SET(glite_channel_setSrmCopyTimeout,srmcopy_to);
    }
    
    if (srmcopy_refresh_to >= 0 || unset_srmcopy_refresh_to){
        CHANNEL_SET(glite_channel_setSrmCopyRefreshTimeout,srmcopy_refresh_to);
    }

    return 0;

}

int tool_doit(glite_transfer_ctx *ctx, int argc, char *argv[]) {
    int i, ret;

    if (!argc) {
        error("No channel specified");
        return EXIT_FAILURE;
    }

    if (0 == get_updates()) {
        error("No updates specified");
        return EXIT_FAILURE;
    }

    for (i = 0; i < argc; i++) {
        ret = set_channel(ctx, argv[i]);
        if (ret) {
            error("set: %s", glite_transfer_get_error(ctx));
            return EXIT_FAILURE;
        }
        info("Modified channel %s", argv[i]);
    }

    return EXIT_SUCCESS;
}

/**********************************************************************
 * Command line parsing
 */
int get_unsettable_int(const char *,const char *,int *,int *);
int get_unsettable_int(
    const char *    arg,
    const char *    name,
    int *           value,
    int *           unset) {
    if (0 == strcasecmp(arg,"unset")){
        *unset = 1;
        return 0;
    }
    if (!sscanf(arg,"%d",value)) {
        error("Unable to parse %s argument",name);
        return -1;
    }
    return 0;
} 

int get_unsettable_double(const char *,const char *,double *,int *);
int get_unsettable_double(
    const char *    arg,
    const char *    name,
    double *        value,
    int *           unset) {
    if (0 == strcasecmp(arg,"unset")){
        *unset = 1;
        return 0;
    }
    if (!sscanf(arg,"%lf",value)) {
        error("Unable to parse %s argument",name);
        return -1;
    }
    return 0;
} 

 
int tool_parse_cmdline(int c, char *opt_arg) {
    switch (c) {
    case 'b':
        if (!sscanf(opt_arg, "%d", &bandwidth)) {
            error("Unable to parse bandwidth argument");
            return -1;
        }
        updates++;
        return 0;
    case 'c':
        contact = opt_arg;
        updates++;
        return 0;
    case 'u':
        buffer_size = opt_arg;
        updates ++;
        return 0;
    case 'f':
        if (!sscanf(opt_arg, "%d", &num_files)) {
            error("Unable to parse number of files argument");
            return -1;
        }
        updates++;
        return 0;
    case 'S':
        state = glite_channel_state_parse(NULL, opt_arg);
        if (state == GLITE_CHANNEL_UNKNOWN) {
            error("Unknown channel state");
            return -1;
        }
        updates++;
        return 0;
    case 'T':
        if (!sscanf(opt_arg, "%d", &num_streams)) {
            error("Unable to parse number of streams argument");
            return -1;
        }
        updates++;
        return 0;
    case 't':
        if (!sscanf(opt_arg, "%d", &throughput)) {
            error("Unable to parse throughput argument");
            return -1;
        }
        updates++;
        return 0;
    case 'm':
        message = opt_arg;
        return 0;
    case 'k':
        if (1 != strlen(opt_arg)) {
            error("Destination directory check flag argument too long: %s",opt_arg);
            return -1;
        }
        char flag = opt_arg[0];
        if (('y' == flag) || ('Y' == flag)) {
            target_dir_check = 1;
        } else if (('n' == flag) || ('N' == flag)) {
            target_dir_check = 0;
        } else {
            error("Invalid value for destination directory check flag: '%c'",flag);
            return -1;
        }
        ++updates_340;
        return 0;
    case 'x': // first transfer marker timeout
        ++updates_340;
        return get_unsettable_int(opt_arg,"first transfer marker timeout",&url_copy_first_txmark_to,&unset_url_copy_first_txmark_to);
    case TOOL_OPT_NUM(0): // block-size
        if (0 == strcasecmp(opt_arg,"unset")){
            unset_block_size = 1;
        } else if (validate_size_string(opt_arg)) {
            block_size = opt_arg;
        } else {
            error("Invalid value for block-size argument");
            return -1;
        }
        ++updates_350;
        return 0;
    case TOOL_OPT_NUM(1): // http-timeout
        ++updates_350;
        return get_unsettable_int(opt_arg,"http-timeout",&http_timeout,&unset_http_timeout);
    case TOOL_OPT_NUM(2): // tx-log-level
        tx_log_level = transfer_log_level_parse(opt_arg);
        if (TRANSFER_LOG_LEVEL_INVALID == tx_log_level){
            error("Unable to parse tx-log-level argument");
            return -1;
        }
        ++updates_350;
        return 0;
    case TOOL_OPT_NUM(3): // preparing-ratio
        ++updates_350;
        if (!sscanf(opt_arg, "%lf", &preparing_ratio)) {
            error("Unable to parse preparing-ratio argument");
            return -1;
        }
        return 0;
    case TOOL_OPT_NUM(4): // urlcopy-put-to
        ++updates_350;
        return get_unsettable_int(opt_arg,"urlcopy-put-to",&urlcopy_put_to,&unset_urlcopy_put_to);
    case TOOL_OPT_NUM(5): // urlcopy-putdone-to
        ++updates_350;
        return get_unsettable_int(opt_arg,"urlcopy-putdone-to",&urlcopy_putdone_to,&unset_urlcopy_putdone_to);
    case TOOL_OPT_NUM(6): // urlcopy-get-to
        ++updates_350;
        return get_unsettable_int(opt_arg,"urlcopy-get-to",&urlcopy_get_to,&unset_urlcopy_get_to);
    case TOOL_OPT_NUM(7): // urlcopy-getdone-to
        ++updates_350;
        return get_unsettable_int(opt_arg,"urlcopy-getdone-to",&urlcopy_getdone_to,&unset_urlcopy_getdone_to);
    case TOOL_OPT_NUM(8): // urlcopy-tx-to
        ++updates_350;
        return get_unsettable_int(opt_arg,"urlcopy-tx-to",&urlcopy_tx_to,&unset_urlcopy_tx_to);
    case TOOL_OPT_NUM(9): // urlcopy-tx-marks-to
        ++updates_350;
        return get_unsettable_int(opt_arg,"urlcopy-tx-marks-to",&urlcopy_tx_marks_to,&unset_urlcopy_tx_marks_to);
    case TOOL_OPT_NUM(10): // urlcopy-no-prog-to
        ++updates_350;
        return get_unsettable_int(opt_arg,"urlcopy-no-prog-to",&urlcopy_no_prog_to,&unset_urlcopy_no_prog_to);
    case TOOL_OPT_NUM(11): // urlcopy-tx-to-per-mb
        ++updates_350;
        return get_unsettable_double(opt_arg,"urlcopy-tx-to-per-mb",&urlcopy_tx_to_per_mb,&unset_urlcopy_tx_to_per_mb);
    case TOOL_OPT_NUM(12): // srmcopy-direction
        srmcopy_dir = srmcopy_direction_parse(opt_arg);
        if (SRMCOPY_DIRECTION_INVALID == srmcopy_dir){
            error("Unable to parse srmcopy-direction argument");
            return -1;
        }
        ++updates_350;
        return 0;
    case TOOL_OPT_NUM(13): // srmcopy-to
        ++updates_350;
        return get_unsettable_int(opt_arg,"srmcopy-to",&srmcopy_to,&unset_srmcopy_to);
    case TOOL_OPT_NUM(14): // srmcopy-refresh-to
        ++updates_350;
        return get_unsettable_int(opt_arg,"srmcopy-refresh-to",&srmcopy_refresh_to,&unset_srmcopy_refresh_to);
    default:
        break;
    }
    return -1;
}

