/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Transfer Clients - Show job status
 *
 *  Authors: Gabor Gombas <Gabor.Gombas@cern.ch>
 *  Version info: $Id: status.c,v 1.16 2009/01/19 14:29:51 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#include <glite/data/transfer/c/fts-simple.h>

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "tool-main.h"

/**********************************************************************
 * Constants
 */

#define DEFAULT_LIMIT           100


/**********************************************************************
 * Global variables
 */

/* Usage message */
const char *tool_usage = "JOBID [JOBID...]";

/* Help message */
const char *tool_help =
    "  -l                           List status for all files\n";

/* getguid-specific command line options */
const char *tool_options = "l";
NO_TOOL_LONG_OPTIONS;

/* True if file status listing is requested */
static int list_flag;


/**********************************************************************
 * Tool implementation
 */

static int show_job(glite_transfer_ctx *ctx, const char *jobid) {
    glite_transfer_JobStatus *status;

    status = glite_fts_getTransferJobStatus(ctx, jobid);
    if (!status)
        return -1;
    printf("%s\n", glite_transfer_state_str(status->jobStatus));
    return 0;
}

static int show_summary(glite_transfer_ctx *ctx, const char *jobid) {
    glite_transfer_TransferJobSummary *summary;

    summary = glite_fts_getTransferJobSummary(ctx, jobid);
    if (!summary)
        return -1;

    printf("Request ID:\t%s\n", jobid);
    if (summary->jobStatus)
        print_job(summary->jobStatus);

    printf("\tDone:\t\t%d\n", summary->numDone);
    printf("\tActive:\t\t%d\n", summary->numActive);
    printf("\tPending:\t%d\n", summary->numPending);
    printf("\tCanceled:\t%d\n", summary->numCanceled);
    printf("\tCanceling:\t%d\n", summary->numCanceling);
    printf("\tFailed:\t\t%d\n", summary->numFailed);
    printf("\tFinished:\t%d\n", summary->numFinished);
    printf("\tSubmitted:\t%d\n", summary->numSubmitted);
    printf("\tHold:\t\t%d\n", summary->numHold);
    printf("\tWaiting:\t%d\n", summary->numWaiting);
    printf("\tCatalogFailed:\t%d\n", summary->numCatalogFailed);
    return 0;
}

static int show_summary2(glite_transfer_ctx *ctx, const char *jobid) {
    glite_transfer_TransferJobSummary2 *summary;

    summary = glite_fts_getTransferJobSummary2(ctx, jobid);
    if (!summary)
        return -1;

    printf("Request ID:\t%s\n", jobid);
    if (summary->jobStatus)
        print_job(summary->jobStatus);

    printf("\tDone:\t\t%d\n", summary->numDone);
    printf("\tActive:\t\t%d\n", summary->numActive);
    printf("\tPending:\t%d\n", summary->numPending);
    printf("\tReady:\t\t%d\n", summary->numReady);
    printf("\tCanceled:\t%d\n", summary->numCanceled);
    printf("\tFailed:\t\t%d\n", summary->numFailed);
    printf("\tFinishing:\t%d\n", summary->numFinishing);
    printf("\tFinished:\t%d\n", summary->numFinished);
    printf("\tSubmitted:\t%d\n", summary->numSubmitted);
    printf("\tHold:\t\t%d\n", summary->numHold);
    printf("\tWaiting:\t%d\n", summary->numWaiting);
    return 0;
}


static int list_job(glite_transfer_ctx *ctx, const char *jobid) {
    glite_transfer_FileTransferStatus **list;
    int list_cnt, offset, limit, i;

    limit = DEFAULT_LIMIT;
    offset = 0;
    for (;;) {
        list = glite_fts_getFileStatus(ctx, jobid, offset, limit,
                                       &list_cnt);
        if (list_cnt < 0)
            return -1;

        for (i = 0; i < list_cnt; i++) {
            if (i)
                printf("\n");

            if((0 != list[i]->logicalName) &&
                    (0 != strlen(list[i]->logicalName))) {
                printf("  LogicalName: %s\n", list[i]->logicalName);
            }
            printf("  Source:      %s\n", list[i]->sourceSURL);
            printf("  Destination: %s\n", list[i]->destSURL);
            printf("  State:       %s\n", glite_transfer_state_str(list[i]->transferFileState));
            printf("  Retries:     %d\n", list[i]->numFailures);
            printf("  Reason:      %s\n", list[i]->reason);
            printf("  Duration:    %d\n", list[i]->duration);
        }

        glite_transfer_FileTransferStatus_freeArray(ctx, list_cnt, list);
        if (list_cnt < limit)
            break;
        offset += limit;
    }
    return 0;
}

int tool_doit(glite_transfer_ctx *ctx, int argc, char *argv[]) {
    int i, ret;

    if (!argc) {
        error("status: No request ID specified");
        return EXIT_FAILURE;
    }

    ret = 0;
    for (i = 0; i < argc; i++) {
        if (verbose_output)
            if (isVersion330StatesSupported()) {
                ret = show_summary2(ctx, argv[i]);
            } else {
                ret = show_summary(ctx, argv[i]);
            }
        else
            ret = show_job(ctx, argv[i]);
        if (ret)
            break;
        if (list_flag) {
            ret = list_job(ctx, argv[i]);
            if (ret)
                break;
        }
    }

    if (ret) {
        error("status: %s", glite_transfer_get_error(ctx));
        return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
}

/**********************************************************************
 * Command line parsing
 */

int tool_parse_cmdline(int c, char *opt_arg G_GNUC_UNUSED) {
    switch (c) {
    case 'l':
        list_flag++;
        return 0;
    default:
        break;
    }
    return -1;
}
