/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Transfer Clients - Submit a transfer job
 *
 *  Authors: Peter Kunszt <peter.kunszt@cern.ch>
 *           Gabor Gombas <Gabor.Gombas@cern.ch>
 *           Gavin McCance <Gavin.McCance@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 */

#include <glite/data/transfer/c/fts-simple.h>
#include <glite/data/delegation/c/delegation-simple.h>

#include <termios.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <openssl/x509_vfy.h>
#include <openssl/err.h>
#include <openssl/pem.h>
#include <openssl/asn1.h>
#include <time.h>
#include <gridsite.h>

#include "tool-main.h"

/**********************************************************************
 * Constants
 */

#define DEFAULT_POLL_INTERVAL       2
#define PASSWORD_LENGTH         40


/**********************************************************************
 * Global variables
 */

/* Usage message */
const char *tool_usage = "SOURCE DEST [CHECKSUM]";

/* Help message */
const char *tool_help =
"  -b                           Blocking mode, wait until the operation \n"
"                               completes.\n"
"  -f FILE                      Read job descriptions from FILE\n"
"  -g PARAM                     gridftp parameters\n"
"  -i TIME                      Interval between two poll operations in \n"
"                               blocking mode\n"
"  -m SERVER                    MyProxy server to use\n"
"  -p PASSWORD                  MyProxy password to send with the job\n"
"  -I ID                        Delegation with ID as the delegation identifier\n"
"  -e MINUTES                   Expiration time of the delegation in minutes\n"
"  -o                           Overwrite files\n"
"  -t, --dest-token TOKEN       The destination space token or its description\n"
"                               (for SRM 2.2 transfers)\n"
"  -S, --source-token TOKEN     The source space token or its descriptio\n"
"                               (for SRM 2.2 transfers)\n"
"      --copy-pin-lifetime N    pin lifetime of the copy of the file (seconds)\n"
"      --lan-connection         use LAN as ConnectionType (default = WAN)\n"
"      --fail-nearline          fail the transfer if the file is nearline\n"
"  -K, --compare-checksums      compare checksums between source and destination\n"
;


/* getguid-specific command line options */
const char *tool_options = "bf:g:i:m:p:I:e:ot:S:K";

/* Long options */
struct option tool_long_options[] = {
    {"dest-token",          required_argument,  NULL,   't'},
    {"source-token",        required_argument,  NULL,   'S'},
    {"copy-pin-lifetime",   required_argument,  NULL,   TOOL_OPT_NUM(0)},
    {"lan-connection",      no_argument,        NULL,   TOOL_OPT_NUM(1)},
    {"fail-nearline",       no_argument,        NULL,   TOOL_OPT_NUM(2)},
    {"compare-checksums",   no_argument,        NULL,   'K'}
};
SET_TOOL_LONG_OPTIONS_SIZE;

/* GridFTP parameters */
static char *gridftp_params;

/* File holding the source-dest pairs */
static char *job_file;

/* Request blocking mode */
static int blocking_flag;

/* Poll interval */
static int poll_interval;

/* MyProxy password */
static char password[PASSWORD_LENGTH] = "";

/* MyProxy password verification string */
static char password_ver[PASSWORD_LENGTH] = "";

/* MyProxy server */
static char *myproxy_server;

/* Whether to use legacy myproxy mode and not use delegation */
static int legacy_myproxy_mode = 0;

/* Do delegation */
static int delegation_flag = 0;

/* SRM2.2 overwrite flag */
static int overwrite_flag = 0;

/* User requested delegation exp time */
static int user_requested_delegation_exp_time = 0;

/* If the existing delegated proxy has less then this time left to run, it will be redelegated */
static int redelegation_time_limit = 3600*6; // 6 hours

/* Unless the user overrides it, the maximum time that a delegated credential
   will be requested for, even if the issuing certificate is longer than this (value = 1 day) */
static int maximum_time_for_delegation_request = 3600 * 24;

/* Delegation ID */
static char *delegation_id = NULL;

/* Storage class */
static char *space_token = NULL;
static char *source_space_token = NULL;

static char * copy_pin_lifetime = NULL;
static int lan_connection_flag = 0;
static int fail_nearline_flag = 0;

/* Checksums */
static int with_checksums_flag = 0;
static int compare_checksums_flag = 0;

long get_cert_time_left(char * filename);

// Return the validity time left (in seconds) for the user-proxy
long get_cert_time_left(char * filename) {

    X509 *cert;
    FILE *fp;
    char * user_proxy;

    if (filename == NULL)
        user_proxy = GRSTx509FindProxyFileName();
    else
        user_proxy = filename;

    if (!(fp = fopen(user_proxy , "r")))
        return 0;
    if (!(cert = PEM_read_X509(fp, NULL, NULL, NULL)))
        return 0;
    fclose(fp);

    // or the phantom ASN1_UTCTIME_get
    long validity = GRSTasn1TimeToTimeT(ASN1_STRING_data(X509_get_notAfter(cert)),0);
    long time_left = validity - time(NULL);
    return time_left;

}

/**********************************************************************
 * Tool implementation
 */

static void parse_line(char *buf, char **src, char **dst, char **cksm) {
    char *p;

    /* Skip leading whitespace */
    *src = buf;
    while (**src == ' ' || **src == '\t')
        (*src)++;

    /* Look for the separator */
    *dst = *src;
    while (**dst && **dst != ' ' && **dst != '\t') {
        /* Allow escaped whitespace */
        if (**dst == '\\') {
            (*dst)++;
            if (!**dst)
                break;
        }
        (*dst)++;
    }

    /* There may be only one filename */
    if (!**dst) {
        *dst = NULL;
        return;
    }

    /* Terminate the source file name */
    *(*dst)++ = '\0';

    /* Skip whitespaces/tabs before destination */
    while (**dst == ' ' || **dst == '\t')
        (*dst)++;

    /* Look for the end of the destination file name */
    *cksm = *dst;
    while (**cksm && **cksm != ' ' && **cksm != '\t' && **cksm != '\n') {
        /* Allow escaped whitespace */
        if (**cksm == '\\') {
            (*cksm)++;
            if (!**cksm)
                break;
        }
        (*cksm)++;
    }

    /* There may not be a checksum */
    if (!**cksm) {
        *cksm = NULL;
        return;
    }

    /* Terminate the destination file name */
    *(*cksm)++ = '\0';

    /* Skip whitespaces/tabs before checksum */
    while (**cksm == ' ' || **cksm == '\t')
        (*cksm)++;

    /* Look for the end of the destination file name */
    p = *cksm;
    while (*p && *p != ' ' && *p != '\t' && *p != '\n') {
        /* Allow escaped whitespace */
        if (*p == '\\') {
            p++;
            if (!*p)
                break;
        }
        p++;
    }

    /* Strip trailing white space */
    *p = '\0';

    /* Checksum is optional, turn it into NULL, if empty */
    if (!**cksm) {
        *cksm = NULL;
    }
}

int tool_doit(glite_transfer_ctx *ctx, int argc, char *argv[]) {
    glite_transfer_JobStatus *status;
    glite_transfer_TransferJob *job;
    char *id;
    int ret;

    /* Argument & option checks */
    if (!argc && !job_file) {
        error("No transfer job is specified");
        return EXIT_FAILURE;
    }
    if (argc && argc == 3 && (!isChecksumSupported())) {
        error("You have specified an optional checksum, but it is not supported by the server.");
        return EXIT_FAILURE;
    }
    if (argc && (argc < 2 || argc > 3)) {
        error("Wrong number of arguments: %d", argc);
        return EXIT_FAILURE;
    }
    if (argc && job_file) {
        error("You may not specify a transfer on the command line if the -f option is used");
        return EXIT_FAILURE;
    }

    // decide on delegation depending on server
    if (isDelegationSupported() && legacy_myproxy_mode == 0) {
        info("Server supports delegation. Delegation will be used by default.");
        delegation_flag = 1;
    }

    if (isDelegationSupported() && legacy_myproxy_mode == 1) {
        info("Server supports delegation, however a MyProxy passphrase was given: will use MyProxy legacy mode.");
        delegation_flag = 0;
    }

    // in the case of the -I option, delegation is required
    if ( delegation_flag && (!isDelegationSupported()) ) {
        error("The server you are contacting does not support credential delegation (it is running interface version %s). Please use the MyProxy password legacy method for this server.", srv_iface_ver);
        return EXIT_FAILURE;
    }

    // in the case of the -K option, checksum is required
    if ( compare_checksums_flag && (!isChecksumSupported()) ) {
        error("The server you are contacting does not support checksum checking (it is running interface version %s). Please omit checksums for this server.", srv_iface_ver);
        return EXIT_FAILURE;
    }

    /** Ask for MyProxy password if not passed as arg */
    if(!delegation_flag && (strncmp(password, "", 1) == 0)) {
        struct termios initialrsettings, newrsettings;
        unsigned int i;

        tcgetattr(fileno(stdin), &initialrsettings);
        newrsettings = initialrsettings;
        newrsettings.c_lflag &= ~ECHO;
        printf("Enter MyProxy password: ");
        if(tcsetattr(fileno(stdin), TCSAFLUSH, &newrsettings) != 0)
            fprintf(stderr,"submit: could not set terminal attributes");
        else {
            fgets(password, sizeof(password), stdin);
            tcsetattr(fileno(stdin), TCSANOW, &initialrsettings);
        }
        for(i=0; i < sizeof(password); i++) {
            if(password[i] == '\n') {
                password[i] = '\0';
                break;
            }
        }
        printf("\n");
        printf("Enter MyProxy password again: ");
        if(tcsetattr(fileno(stdin), TCSAFLUSH, &newrsettings) != 0)
            fprintf(stderr,"submit: could not set terminal attributes");
        else {
            fgets(password_ver, sizeof(password_ver), stdin);
            tcsetattr(fileno(stdin), TCSANOW, &initialrsettings);
        }
        for(i=0; i < sizeof(password_ver); i++) {
            if(password_ver[i] == '\n') {
                password_ver[i] = '\0';
                break;
            }
        }
        printf("\n");
        if (strcmp(password, password_ver)) {
            error("Entered MyProxy passwords do not match");
            return EXIT_FAILURE;
        }
    }

    job = glite_transfer_TransferJob_new(ctx);
    if (!job) {
        error("submit: %s", glite_transfer_get_error(ctx));
        return EXIT_FAILURE;
    }

    if (gridftp_params || myproxy_server || delegation_id || space_token || overwrite_flag || source_space_token || 
        copy_pin_lifetime || lan_connection_flag || fail_nearline_flag || compare_checksums_flag){
        glite_transfer_TransferParams *params;

        params = glite_transfer_TransferParams_new(ctx);
        if (!params) {
            error("submit: %s", glite_transfer_get_error(ctx));
            return EXIT_FAILURE;
        }

        if (gridftp_params) {
            ret = glite_transfer_TransferParams_add(ctx, params,
                                                    GLITE_FTS_PARAM_GRIDFTP, gridftp_params);
            if (ret) {
                error("submit: %s", glite_transfer_get_error(ctx));
                return EXIT_FAILURE;
            }
        }

        if (myproxy_server) {
            ret = glite_transfer_TransferParams_add(ctx, params,
                                                    GLITE_FTS_PARAM_MYPROXY, myproxy_server);
            if (ret) {
                error("submit: %s", glite_transfer_get_error(ctx));
                return EXIT_FAILURE;
            }
        }

        if (delegation_id) {
            ret = glite_transfer_TransferParams_add(ctx, params,
                                                    GLITE_FTS_PARAM_DELEGATIONID, delegation_id);
            if (ret) {
                error("submit: %s", glite_transfer_get_error(ctx));
                return EXIT_FAILURE;
            }
        }

        if (space_token) {
            ret = glite_transfer_TransferParams_add(ctx, params,
                                                    GLITE_FTS_PARAM_SPACETOKEN, space_token);
            if (ret) {
                error("submit: %s", glite_transfer_get_error(ctx));
                return EXIT_FAILURE;
            }
        }

        if (source_space_token) {
            ret = glite_transfer_TransferParams_add(ctx, params,
                                                    GLITE_FTS_PARAM_SPACETOKEN_SOURCE, source_space_token);
            if (ret) {
                error("submit: %s", glite_transfer_get_error(ctx));
                return EXIT_FAILURE;
            }
        }

        if (overwrite_flag) {
            ret = glite_transfer_TransferParams_add(ctx, params,
                                                    GLITE_FTS_PARAM_OVERWRITEFLAG, "Y");
            if (ret) {
                error("submit: %s", glite_transfer_get_error(ctx));
                return EXIT_FAILURE;
            }
        }
        
        if (copy_pin_lifetime){
            ret = glite_transfer_TransferParams_add(ctx, params,GLITE_FTS_PARAM_COPY_PIN_LIFETIME,copy_pin_lifetime);
            if (ret) {
                error("submit: %s", glite_transfer_get_error(ctx));
                return EXIT_FAILURE;
            }
        } 
        if (lan_connection_flag){
            ret = glite_transfer_TransferParams_add(ctx, params,GLITE_FTS_PARAM_LAN_CONNECTION,"Y");
            if (ret) {
                error("submit: %s", glite_transfer_get_error(ctx));
                return EXIT_FAILURE;
            }
        }
        if (fail_nearline_flag){
            ret = glite_transfer_TransferParams_add(ctx, params,GLITE_FTS_PARAM_FAIL_NEARLINE,"Y");
            if (ret) {
                error("submit: %s", glite_transfer_get_error(ctx));
                return EXIT_FAILURE;
            }
        }
       
        if (compare_checksums_flag){
            ret = glite_transfer_TransferParams_add(ctx, params,GLITE_FTS_PARAM_CHECKSUM_METHOD,"compare");
            if (ret) {
                error("submit: %s", glite_transfer_get_error(ctx));
                return EXIT_FAILURE;
            }
        }

         
        ret = glite_transfer_TransferJob_setParams(ctx, job, params);
        if (ret) {
            error("submit: %s", glite_transfer_get_error(ctx));
            return EXIT_FAILURE;
        }
        glite_transfer_TransferParams_free(ctx, params);
    }

    if (job_file) {
        char buf[2048];
        int line;
        FILE *fp;

        if (job_file[0] == '-' && !job_file[1])
            fp = stdin;
        else {
            fp = fopen(job_file, "r");
            if (!fp) {
                error("submit: Failed to open %s", job_file);
                return EXIT_FAILURE;
            }
        }

        ret = 0;
        line = 0;
        while (fgets(buf, sizeof(buf), fp)) {
            char *src, *dst, *cksm;

            line++;
            parse_line(buf, &src, &dst, &cksm);

            /* Skip empty lines */
            if (*src == '\n')
                continue;

            if (!dst || !*dst) {
                error("submit: Destination is missing in "
                      "line %d", line);
                return EXIT_FAILURE;
            }

            if (cksm) {
                with_checksums_flag = 1;
            }

            ret = glite_transfer_TransferJob_add(ctx, job, src, dst, cksm);
            if (ret)
                break;
        }

        if (fp != stdin)
            fclose(fp);
    } else {
        if (argc == 3) {
            ret = glite_transfer_TransferJob_add(ctx, job, argv[0], argv[1], argv[2]);
            with_checksums_flag = 1;
        } else {
            ret = glite_transfer_TransferJob_add(ctx, job, argv[0], argv[1], NULL);
        }
    }

    if (ret) {
        error("submit: %s", glite_transfer_get_error(ctx));
        return EXIT_FAILURE;
    }

    /** Set the password or delegate. */
    if (delegation_flag) {
        glite_delegation_ctx *dctx;
        int force_delegation;
        int need_delegation;
        int ret;
        time_t exp_time, current_time, request_proxy_delegation_time, time_left_on_server, local_proxy_time_left;
        char* delegation_service_location = NULL;
        char* endpt = NULL;
        // if the user has forced a FileTransfer endpoint using -s (as thay *all* do) without using service discovery,
        // set the delegation endpoint by guessing it (since we write the deployment script, we know what it will be).
        // mmmm... nice.
        if (service_location != NULL) { // user used the -s option
            endpt = strstr(service_location, "/services/FileTransfer");

            delegation_service_location = (char*)malloc(strlen(service_location) + 8);
            if (endpt != NULL) { // user specified an endpoint in -s, work out the delegation interface
                strncpy(delegation_service_location, service_location, endpt - service_location);
                delegation_service_location[endpt - service_location] = '\0';
                strcat(delegation_service_location, "/services/gridsite-delegation");

            } else { // pass the non-endpoint string and work the rest out later from service discovery
                strcpy(delegation_service_location,service_location);
            }

        } else { // pass the NULL, and it'll work it out later from service discovery
            delegation_service_location = service_location;
        }

        dctx = glite_delegation_new(delegation_service_location);
        if (dctx == NULL) {
            error("delegation: could not initialize a delegation context");
            return EXIT_FAILURE;
        }

        // get local proxy run time
        local_proxy_time_left = get_cert_time_left(NULL);
        info("Remaining time for local proxy is %ld hours and %ld minutes.",
             (long int)((local_proxy_time_left) / 3600),
             (long int)((local_proxy_time_left) % 3600 / 60));


        // the default is to delegate
        need_delegation = 1;
        // ..but not renew
        force_delegation = 0;
        // todo: why is this called "force_delegation" ?!?

        // checking if there is already a delegated credenial
        ret = glite_delegation_info(dctx, delegation_id, &exp_time);

        if (0 == ret) {   // there is already a delegated proxy on the server
            current_time = time(NULL);
            time_left_on_server = exp_time - current_time;

            if (time_left_on_server > redelegation_time_limit) {
                // don;t bother redelegating
                info("Not bothering to do delegation, since the server already has a delegated credential for this user lasting longer than 4 hours.");
                info("Remaining time on server for this credential is %ld hours and %ld minutes.",
                     (long int)((time_left_on_server) / 3600),
                     (long int)((time_left_on_server) % 3600 / 60));
                need_delegation = 0;
                force_delegation = 0;
            } else {
                // think about redelegating
                if (local_proxy_time_left > time_left_on_server) {
                    // we improve the situation (the new proxy will last longer)
                    info("Will redo delegation since the credential on the server has left that 4 hours validity left.");
                    info("Remaining time for this credential is %ld hours and %ld minutes.",
                         (long int)((time_left_on_server) / 3600),
                         (long int)((time_left_on_server) % 3600 / 60));
                    need_delegation = 1;
                    force_delegation = 1; // renew rather than put
                } else {
                    // we cannot improve the proxy on the server
                    info("Delegated proxy on server has less than 4 hours left.");
                    info("But the local proxy has less time left than the one on the server, so cannot be used to refresh it!");
                    info("Remaining time on server for this credential is %ld hours and %ld minutes.",
                         (long int)((time_left_on_server) / 3600),
                         (long int)((time_left_on_server) % 3600 / 60));
                    info("Remaining time for local proxy is %ld hours and %ld minutes.",
                         (long int)((local_proxy_time_left) / 3600),
                         (long int)((local_proxy_time_left) % 3600 / 60));
                    need_delegation=0;
                }
            }
        } else {
            // no proxy on server: do standard delegation
            info("No proxy found on server. Requesting standard delegation");
            need_delegation = 1;
            force_delegation = 0;
        }

        if (need_delegation) {
            if (user_requested_delegation_exp_time == 0) {
                request_proxy_delegation_time = local_proxy_time_left - 60; // 60 seconds off current proxy
                if (request_proxy_delegation_time > maximum_time_for_delegation_request ) {
                    request_proxy_delegation_time = maximum_time_for_delegation_request;
                }
                if (request_proxy_delegation_time <= 0) {
                    // using a proxy with less than 1 minute to go
                    error("Your local proxy has less than 1 minute to run, Please renew it before submitting a job.");
                    return EXIT_FAILURE;
                }
            } else {
                request_proxy_delegation_time = user_requested_delegation_exp_time;
            }
            info("Requesting delegated proxy for %ld hours and %ld minutes.",
                 (long int)((request_proxy_delegation_time) / 3600),
                 (long int)((request_proxy_delegation_time) % 3600 / 60));
            ret = glite_delegation_delegate(dctx, delegation_id,
                                            (request_proxy_delegation_time/60),
                                            force_delegation);
            if (-1 == ret) {
                error("delegation: %s", glite_delegation_get_error(dctx));
                return EXIT_FAILURE;
            }
            info("Credential is succesfully delegated to the service.");
        }

        glite_delegation_free(dctx);
        if(NULL != delegation_service_location) {
            free(delegation_service_location);
            delegation_service_location = NULL;
        }
    } else {
        job->credential = strdup(password);
    }

    /* Before the actual submit we check again, if the client has supplied checksums */
    if (with_checksums_flag && (!isChecksumSupported())) {
        error("You have specified an optional checksum, but it is not supported by the server.");
        return EXIT_FAILURE;
    }

    id = glite_fts_transferSubmit(ctx, job, delegation_flag, with_checksums_flag);
    if (!id) {
        error("submit: %s", glite_transfer_get_error(ctx));
        return EXIT_FAILURE;
    }

    if (!blocking_flag) {
        printf("%s\n", id);
        goto end;
    }

    info("Submitted request ID: %s", id);
    if (poll_interval <= 0)
        poll_interval = DEFAULT_POLL_INTERVAL;

    for (;;) {
        status = glite_fts_getTransferJobStatus(ctx, id);
        if (!status) {
            error("submit: %s", glite_transfer_get_error(ctx));
            return EXIT_FAILURE;
        }

        if (status->jobStatus <= 0)
            break;

        glite_transfer_JobStatus_free(ctx, status);
        sleep(poll_interval);
    }

    if (status->jobStatus < 0) {
        error("Transfer state: %s",
              glite_transfer_state_str(status->jobStatus));
        ret = EXIT_FAILURE;
    } else {
        info("Transfer state: %s",
             glite_transfer_state_str(status->jobStatus));
        ret = EXIT_SUCCESS;
    }
    glite_transfer_JobStatus_free(ctx, status);

end:
    glite_transfer_TransferJob_free(ctx, job);
    return ret;
}

/**********************************************************************
 * Command line parsing
 */

int tool_parse_cmdline(int c, char *opt_arg) {
    long user_requested_delegation_exp_time_in_minutes = 0;
    int tmp;
    switch (c) {
    case 'b':
        blocking_flag++;
        debug("got option b");
        return 0;
    case 'f':
        job_file = opt_arg;
        debug("got option f");
        return 0;
    case 'g':
        gridftp_params = opt_arg;
        debug("got gridftp_params = %s",gridftp_params);
        return 0;
    case 'i':
        if (!sscanf(opt_arg, "%d", &poll_interval)) {
            error("Unable to parse poll interval argument");
            return -1;
        }
        debug("got poll_interval = %d",poll_interval);
        return 0;
    case 'p':
        legacy_myproxy_mode = 1;
        strncpy(password, opt_arg, sizeof(password));
        debug("got password = %s",password);
        return 0;
    case 'm':
        myproxy_server = opt_arg;
        debug("got myproxy_server = %s",myproxy_server);
        return 0;
    case 't':
        space_token = opt_arg;
        debug("got space_token = %s",space_token);
        return 0;
    case 'S':
        source_space_token = opt_arg;
        debug("got source_space_token = %s",source_space_token);
        return 0;
    case 'o':
        overwrite_flag = 1;
        debug("got overwrite_flag");
        return 0;
    case 'I':
        delegation_flag = 1;
        delegation_id = opt_arg;
        debug("got delegation_id = %s",delegation_id);
        return 0;
    case 'e':
        if (!sscanf(opt_arg, "%d", &user_requested_delegation_exp_time_in_minutes)) {
            error("Unable to parse delegation expiration time argument.");
            return -1;
        }
        if (user_requested_delegation_exp_time_in_minutes != 0) {
            user_requested_delegation_exp_time = user_requested_delegation_exp_time_in_minutes * 60;
        }
        debug("got user_requested_delegation_exp_time = %d",user_requested_delegation_exp_time);
        return 0;
    case TOOL_OPT_NUM(0): // copy-pin-lifetime
        if (!sscanf(opt_arg,"%d",&tmp)) {
            error("Invalid value for copy-pin-lifetime argument.");
            return -1;
        }
        copy_pin_lifetime = opt_arg;
        return 0;
    case TOOL_OPT_NUM(1): // lan-connection
        lan_connection_flag = 1;
        return 0;
    case TOOL_OPT_NUM(2): // fail-nearline
        fail_nearline_flag = 1;
        return 0;
    case 'K': // compare checksums
        compare_checksums_flag = 1;
        return 0;
    default:
        break;
    }
    return -1;
}

