/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  gLite Transfer Clients - Main function for the tools
 *
 *  Authors: Gabor Gombas <Gabor.Gombas@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 */

#include <glite/data/glite-util.h>
#include <glite/data/transfer/c/fts-simple.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>

#include "tool-main.h"

/**********************************************************************
 * Global variables
 */

/* Interface version */
char *srv_iface_ver, *srv_iface_ver_major, *srv_iface_ver_minor, *srv_iface_ver_patch;
long iface_major, iface_minor, iface_patch;

/* Location of the transfer service */
char *service_location;

/* Transfer context handle */
static glite_transfer_ctx *ctx;

/* Source site for submits */
static char *s_site;
/* Destination site for submits */
static char *d_site;

/* Command line options common to all the tools */
static const char *common_options = "hqs:vV";

/* Description of the common options */
static const char *common_help =
"  -h, --help                         Print this help text and exit.\n"
"  -q                                 Quiet operation.\n"
"  -s URL                             Use the transfer service at the specified URL.\n"
"  -v                                 Print details about the service.\n"
"      --verbose                      Be more verbose.\n"
"      --source SSITE                 Specify source site name\n"
"      --destination DSITE            Specify destination site name\n"
"  -V                                 Print the version number and exit.\n";

/* Long options */
static struct option common_long_options[] = {
    {"help",        no_argument,        NULL,   'h'},
    {"verbose",     no_argument,        NULL,   MAIN_OPT_NUM(0)},
    {"source",      required_argument,  NULL,   MAIN_OPT_NUM(1)},
    {"destination", required_argument,  NULL,   MAIN_OPT_NUM(2)},
    {NULL,          no_argument,        NULL,   0}
};

/* Flag to turn on verbose output */
int verbose_flag;

/* Flag to print server infos */
int verbose_output;

/* Flag to request quiet operation */
int quiet_flag;

/**********************************************************************
 * Main routine for the gLite file placement tools
 */

int main(int argc, char *argv[]) {
    char *options, *prog_name;
    char * ifver_tmp;
    const char *endpoint;
    int optlen, ret, c;
    int options_index = 0;

    /* Determine the program name we were called as */
    prog_name = strrchr(argv[0], '/');
    if (!prog_name)
        prog_name = argv[0];
    else
        prog_name++;

    // Build the option string for getopt()
    optlen = strlen(common_options) + 1;
    if (tool_options)
        optlen += strlen(tool_options);
    options = malloc(optlen);
    if (!options) {
        fprintf(stderr, "Out of memory\n");
        exit(EXIT_FAILURE);
    }
    strcpy(options, common_options);
    if (tool_options)
        strcat(options, tool_options);

    // build the long options
    int common_long_options_size = sizeof(common_long_options) / sizeof(struct option);
    int options_size = tool_long_options_size + common_long_options_size;
    struct option long_options[options_size];
    int i = 0;
    for (i = 0;i < tool_long_options_size;++i){
        long_options[i] = tool_long_options[i];
    }
    for (i = 0;i < common_long_options_size;++i){
        long_options[tool_long_options_size + i] = common_long_options[i];
    }
    
    /* Parse the command line options */
    while ((c = getopt_long(argc, argv, options, long_options, &options_index)) != -1){
        switch (c) {
        case 'h':
            printf("Usage: %s [options]", prog_name);
            if (tool_usage)
                printf(" %s\n", tool_usage);
            else
                printf("\n");

            printf("Available options:\n%s", common_help);
            if (tool_help)
                printf("%s", tool_help);
            exit(EXIT_SUCCESS);
        case 'q':
            quiet_flag++;
            verbose_flag = 0;
            debug("got option q");
            break;
        case 's':
            service_location = optarg;
            debug("got service_location = %s",service_location);
            break;
        case 'v':
            verbose_flag++;
            quiet_flag = 0;
            debug("got option v");
            break;
        case 'V':
            printf("%s\n", MODULE_VERSION);
            exit(EXIT_SUCCESS);
        case MAIN_OPT_NUM(0):
            verbose_output++;
            quiet_flag = 0;
            debug("got verbose option");
            break;
        case MAIN_OPT_NUM(1):
            s_site = optarg;
            debug("got source site = %s",s_site);
            break;
        case MAIN_OPT_NUM(2):
            d_site = optarg;
            debug("got dest site = %s",d_site);
            break;
        case ':':
            fprintf(stderr, "Option argument is missing\n");
            exit(EXIT_FAILURE);
        case '?':
            fprintf(stderr, "Unknown command line option\n");
            exit(EXIT_FAILURE);
        default:
            ret = tool_parse_cmdline(c, optarg);
            if (ret) {
                fprintf(stderr, "Illegal command line arguments\n");
                exit(EXIT_FAILURE);
            }
            break;
        }
    }
    
    /* Drop the processed options from the argument list */
    free(options);
    argc -= optind;
    argv += optind;

    /* Allocate a transfer context */
    ctx = glite_transfer_new(service_location);
    if (!ctx) {
        error("Failed to allocate the transfer context");
        exit(EXIT_FAILURE);
    }

    glite_transfer_add_sites(ctx, s_site, d_site);

    if (NULL == (endpoint = glite_fts_get_endpoint(ctx))) {
        error("Failed to determine the endpoint: %s",
              glite_transfer_get_error(ctx));
        exit(EXIT_FAILURE);
    }
    if (NULL == (srv_iface_ver = glite_fts_getInterfaceVersion(ctx))) {
        error("Failed to determine the interface version of the "
              "service: %s", glite_transfer_get_error(ctx));
        exit(EXIT_FAILURE);
    } else {
        if (!glite_check_versions(INTERFACE_VERSION, srv_iface_ver)) {
            error("Service version checking failed (client"
                  "\"%s\" vs server \"%s\")", INTERFACE_VERSION, srv_iface_ver);
            exit(EXIT_FAILURE);
        }
    }

    // break up interface
    if (NULL != srv_iface_ver) {
        ifver_tmp = malloc(strlen(srv_iface_ver) + 1);
        strcpy(ifver_tmp, srv_iface_ver);
        srv_iface_ver_major = strtok(ifver_tmp, ".");
        srv_iface_ver_minor = strtok(NULL, ".");
        srv_iface_ver_patch = strtok(NULL, ".");
        errno = 0;
        iface_major = strtol(srv_iface_ver_major, NULL, 10);
        if (errno)
            iface_major = -1;
        errno = 0;
        iface_minor = strtol(srv_iface_ver_minor, NULL, 10);
        if (errno)
            iface_minor = -1;
        errno = 0;
        iface_patch = strtol(srv_iface_ver_patch, NULL, 10);
        if (errno)
            iface_patch = -1;
    }

    if (verbose_flag) {
        char *ver;

        info("# Using endpoint %s", endpoint);

        ver = glite_fts_getVersion(ctx);
        if (!ver) {
            error("Failed to determine the version of the "
                  "service: %s", glite_transfer_get_error(ctx));
            exit(EXIT_FAILURE);
        }
        info("# Service version: %s", ver);
        free(ver);

        info("# Interface version: %s", srv_iface_ver);

        ver = glite_fts_getSchemaVersion(ctx);
        if (!ver) {
            error("Failed to determine the schema version of "
                  "the service: %s",
                  glite_transfer_get_error(ctx));
            exit(EXIT_FAILURE);
        }
        info("# Schema version: %s", ver);
        free(ver);

        ver = glite_fts_getServiceMetadata(ctx, "feature.string");
        if (ver) {
            info("# Service features: %s", ver);
            free(ver);
        }

        info("# Client version: %s", MODULE_VERSION);
        info("# Client interface version: %s", INTERFACE_VERSION);
    }

    /* Perform the operation */
    ret = tool_doit(ctx, argc, argv);

    /* Clean up properly so checking for memory leaks is easier */
    glite_transfer_free(ctx);
    return ret;
}

